/*
Core.java
Copyright (C) 2010  Belledonne Communications, Grenoble, France

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

package org.linphone.core;

import androidx.annotation.Nullable;
import androidx.annotation.NonNull;

/**
* Main object to instanciate and on which to keep a reference. <br/>
* <br/>
* This object is the first object to instanciante, and will allow you to perform<br/>
* all kind of tasks. To create it, use either {@link Factory#createCore} or<br/>
* {@link Factory#createCoreWithConfig}, see {@link Config} for more information<br/>
* about factory and default config. On some platforms like Android or iOS you<br/>
* will need to give it the Context of your application.<br/>
* Once the {@link Core} is in state {@link GlobalState#Ready}, use {@link #start}<br/>
* . It will then go to state {@link GlobalState#On} and from that you can start<br/>
* using it for calls and chat messages. It is recommended to add a<br/>
* LinphoneCoreCbs listener using linphone_core_add_listener to it to monitor<br/>
* different events.<br/>
* To be able to receive events from the network, you must schedule a call {@link #iterate}<br/>
* often, like every 20ms. On Android & iOS {@link #isAutoIterateEnabled} is<br/>
* enabled by default so you don't have to worry about that unless you disable it<br/>
* using {@link #setAutoIterateEnabled} or by setting in the [misc] section of<br/>
* your configuration auto_iterate=0. warning: Our API isn't thread-safe but also<br/>
* isn't blocking, so it is strongly recommend to always call our methods from the<br/>
* main thread.<br/>
* Once you don't need it anymore, call {@link #stop} and release the reference on<br/>
* it so it can gracefully shutdown. <br/>
*/
public interface Core {
    public enum LogCollectionUploadState {
        /**
        * Delivery in progress. <br/>
        * <br/>
        */
        InProgress(0),

        /**
        * Log collection upload successfully delivered and acknowledged by remote end<br/>
        * point. <br/>
        * <br/>
        */
        Delivered(1),

        /**
        * Log collection upload was not delivered. <br/>
        * <br/>
        */
        NotDelivered(2);

        protected final int mValue;

        private LogCollectionUploadState (int value) {
            mValue = value;
        }

        static public LogCollectionUploadState fromInt(int value) throws RuntimeException {
            switch(value) {
            case 0: return InProgress;
            case 1: return Delivered;
            case 2: return NotDelivered;
            default:
                throw new RuntimeException("Unhandled enum value " + value + " for LogCollectionUploadState");
            }
        }

        public int toInt() {
            return mValue;
        }
    };

    /**
      * Gets the mediastreamer's factory
      */
    public org.linphone.mediastream.Factory getMediastreamerFactory();
    /**
    * Returns which adaptive rate algorithm is currently configured for future calls. <br/>
    * <br/>
    * see: {@link #setAdaptiveRateAlgorithm} <br/>
    * @return the adaptive rate algorithm. Currently two values are supported:<br/>
    * 'advanced', which is the default value, or 'basic'.   <br/>
    */
    @NonNull
    public String getAdaptiveRateAlgorithm();

    /**
    * Sets adaptive rate algorithm. <br/>
    * <br/>
    * It will be used for each new calls starting from now. Calls already started<br/>
    * will not be updated. <br/>
    * @param algorithm the adaptive rate control algorithm. Currently two values are<br/>
    * supported: 'advanced', which is the default value, or 'basic'.   <br/>
    */
    public void setAdaptiveRateAlgorithm(@NonNull String algorithm);

    /**
    * Returns whether adaptive rate control is enabled. <br/>
    * <br/>
    * see: {@link #enableAdaptiveRateControl}<br/>
    * @return true if adaptive rate control is enabled, false otherwise <br/>
    */
    public boolean adaptiveRateControlEnabled();

    /**
    * Enable adaptive rate control. <br/>
    * <br/>
    * Adaptive rate control consists in using RTCP feedback provided information to<br/>
    * dynamically control the output bitrate of the audio and video encoders, so that<br/>
    * we can adapt to the network conditions and available bandwidth. Control of the<br/>
    * audio encoder is done in case of audio-only call, and control of the video<br/>
    * encoder is done for audio & video calls. Adaptive rate control feature is<br/>
    * enabled by default.<br/>
    * @param enabled true to enable adaptive rate control, false otherwise <br/>
    */
    public void enableAdaptiveRateControl(boolean enabled);

    /**
    * Tells whether the audio adaptive jitter compensation is enabled. <br/>
    * <br/>
    * @return true if the audio adaptive jitter compensation is enabled, false<br/>
    * otherwise. <br/>
    */
    public boolean audioAdaptiveJittcompEnabled();

    /**
    * Enable or disable the audio adaptive jitter compensation. <br/>
    * <br/>
    * @param enable true to enable the audio adaptive jitter compensation, false to<br/>
    * disable it. <br/>
    */
    public void enableAudioAdaptiveJittcomp(boolean enable);

    /**
    * Returns a list of audio devices, with only the first device for each type To<br/>
    * have the list of all audio devices, use {@link #getExtendedAudioDevices} <br/>
    * <br/>
    * @return   A list with the first {@link AudioDevice} of each type     <br/>
    */
    @NonNull
    public AudioDevice[] getAudioDevices();

    /**
    * Get the DSCP field for outgoing audio streams. <br/>
    * <br/>
    * The DSCP defines the quality of service in IP packets. <br/>
    * @return The current DSCP value <br/>
    */
    public int getAudioDscp();

    /**
    * Set the DSCP field for outgoing audio streams. <br/>
    * <br/>
    * The DSCP defines the quality of service in IP packets. <br/>
    * @param dscp The DSCP value to set <br/>
    */
    public void setAudioDscp(int dscp);

    /**
    * Returns the nominal audio jitter buffer size in milliseconds. <br/>
    * <br/>
    * @return The nominal audio jitter buffer size in milliseconds <br/>
    */
    public int getAudioJittcomp();

    /**
    * Sets the nominal audio jitter buffer size in milliseconds. <br/>
    * <br/>
    * The value takes effect immediately for all running and pending calls, if any. A<br/>
    * value of 0 disables the jitter buffer.<br/>
    * @param milliseconds the audio jitter buffer size to set in milliseconds <br/>
    */
    public void setAudioJittcomp(int milliseconds);

    /**
    * Use to get multicast address to be used for audio stream. <br/>
    * <br/>
    * @return an ipv4/6 multicast address or default value.   <br/>
    */
    @Nullable
    public String getAudioMulticastAddr();

    /**
    * Use to set multicast address to be used for audio stream. <br/>
    * <br/>
    * @param ip an ipv4/6 multicast address.   <br/>
    * @return 0 in case of success <br/>
    */
    public int setAudioMulticastAddr(@Nullable String ip);

    /**
    * Use to get multicast state of audio stream. <br/>
    * <br/>
    * @return true if subsequent calls will propose multicast ip set by {@link #setAudioMulticastAddr}<br/>
    */
    public boolean audioMulticastEnabled();

    /**
    * Use to enable multicast rtp for audio stream. <br/>
    * <br/>
    * If enabled, outgoing calls put a multicast address from {@link #getVideoMulticastAddr}<br/>
    * into audio cline. In case of outgoing call audio stream is sent to this<br/>
    * multicast address. For incoming calls behavior is unchanged. <br/>
    * @param yesno if yes, subsequent calls will propose multicast ip set by {@link #setAudioMulticastAddr}<br/>
    */
    public void enableAudioMulticast(boolean yesno);

    /**
    * Use to get multicast ttl to be used for audio stream. <br/>
    * <br/>
    * @return a time to leave value <br/>
    */
    public int getAudioMulticastTtl();

    /**
    * Use to set multicast ttl to be used for audio stream. <br/>
    * <br/>
    * @param ttl value or -1 if not used. [0..255] default value is 1 <br/>
    * @return 0 in case of success <br/>
    */
    public int setAudioMulticastTtl(int ttl);

    /**
    * Return the list of the available audio payload types. <br/>
    * <br/>
    * @return A freshly allocated list of the available payload types. The list must<br/>
    * be destroyed with bctbx_list_free() after usage. The elements of the list<br/>
    * haven't to be unref.      <br/>
    */
    @NonNull
    public PayloadType[] getAudioPayloadTypes();

    /**
    * Redefine the list of the available payload types. <br/>
    * <br/>
    * @param payloadTypes The new list of payload types. The core does not take<br/>
    * ownership on it.    <br/>
    */
    public void setAudioPayloadTypes(@Nullable PayloadType[] payloadTypes);

    /**
    * Gets the UDP port used for audio streaming. <br/>
    * <br/>
    * @return The UDP port used for audio streaming <br/>
    */
    public int getAudioPort();

    /**
    * Sets the UDP port used for audio streaming. <br/>
    * <br/>
    * A value of -1 will request the system to allocate the local port randomly. This<br/>
    * is recommended in order to avoid firewall warnings. <br/>
    * @param port The UDP port to use for audio streaming <br/>
    */
    public void setAudioPort(int port);

    /**
    * Get the audio port range from which is randomly chosen the UDP port used for<br/>
    * audio streaming. <br/>
    * <br/>
    * @return a {@link Range} object     <br/>
    */
    @NonNull
    public Range getAudioPortsRange();

    /**
    * Returns an unmodifiable list of currently entered {@link AuthInfo}. <br/>
    * <br/>
    * @return A list of {@link AuthInfo}.    <br/>
    */
    @NonNull
    public AuthInfo[] getAuthInfoList();

    /**
    * Enable or disable the automatic schedule of {@link #iterate} method on Android<br/>
    * & iOS. <br/>
    * <br/>
    * If enabled, {@link #iterate} will be called on the main thread every 20ms<br/>
    * automatically. If disabled, it is the application that must do this job. <br/>
    * @param enable true to enable auto iterate, false to disable <br/>
    */
    public void setAutoIterateEnabled(boolean enable);

    /**
    * Return AVPF enablement. <br/>
    * <br/>
    * See {@link #setAvpfMode} . <br/>
    * @return The current {@link AVPFMode} mode <br/>
    */
    public AVPFMode getAvpfMode();

    /**
    * Enable RTCP feedback (also known as RTP/AVPF profile). <br/>
    * <br/>
    * Setting {@link AVPFMode#Default} is equivalent to LinphoneAVPFDisabled. This<br/>
    * setting can be overriden per {@link ProxyConfig} with {@link ProxyConfig#setAvpfMode}<br/>
    * . The value set here is used for calls placed or received out of any proxy<br/>
    * configured, or if the proxy config is configured with LinphoneAVPFDefault. <br/>
    * @param mode The AVPF mode to use. <br/>
    */
    public void setAvpfMode(AVPFMode mode);

    /**
    * Return the avpf report interval in seconds. <br/>
    * <br/>
    * @return The current AVPF report interval in seconds <br/>
    */
    public int getAvpfRrInterval();

    /**
    * Set the avpf report interval in seconds. <br/>
    * <br/>
    * This value can be overriden by the proxy config using {@link ProxyConfig#setAvpfRrInterval}<br/>
    * . <br/>
    * @param interval The report interval in seconds <br/>
    */
    public void setAvpfRrInterval(int interval);

    /**
    * Get the list of call logs (past calls). <br/>
    * <br/>
    * @return A list of {@link CallLog}.    <br/>
    */
    @NonNull
    public CallLog[] getCallLogs();

    /**
    * Gets the database filename where call logs will be stored. <br/>
    * <br/>
    * @return filesystem path.   <br/>
    */
    @Nullable
    public String getCallLogsDatabasePath();

    /**
    * Sets the database filename where call logs will be stored. <br/>
    * <br/>
    * If the file does not exist, it will be created.<br/>
    * @param path filesystem path   <br/>
    */
    public void setCallLogsDatabasePath(@Nullable String path);

    /**
    * Special function to check if the callkit is enabled, False by default. <br/>
    * <br/>
    * @return true if callkit is enabled, false otherwise. <br/>
    */
    public boolean callkitEnabled();

    /**
    * Special function to enable the callkit. <br/>
    * <br/>
    * @param enabled true to enable callkit, false to disable it. <br/>
    */
    public void enableCallkit(boolean enabled);

    /**
    * Gets the current list of calls. <br/>
    * <br/>
    * Note that this list is read-only and might be changed by the core after a<br/>
    * function call to {@link #iterate}. Similarly the {@link Call} objects inside it<br/>
    * might be destroyed without prior notice. To hold references to {@link Call}<br/>
    * object into your program, you must use linphone_call_ref. <br/>
    * @return A list of {@link Call}    <br/>
    */
    @NonNull
    public Call[] getCalls();

    /**
    * Get the number of Call. <br/>
    * <br/>
    * @return The current number of calls <br/>
    */
    public int getCallsNb();

    /**
    * Get the camera sensor rotation. <br/>
    * <br/>
    * This is needed on some mobile platforms to get the number of degrees the camera<br/>
    * sensor is rotated relative to the screen. <br/>
    * @return The camera sensor rotation in degrees (0 to 360) or -1 if it could not<br/>
    * be retrieved <br/>
    */
    public int getCameraSensorRotation();

    /**
    * Gets the name of the currently assigned sound device for capture. <br/>
    * <br/>
    * @return The name of the currently assigned sound device for capture.   <br/>
    */
    @Nullable
    public String getCaptureDevice();

    /**
    * Sets the sound device used for capture. <br/>
    * <br/>
    * @param devid The device name as returned by linphone_core_get_sound_devices   <br/>
    * @return 0 <br/>
    */
    public int setCaptureDevice(@Nullable String devid);

    /**
    * Returns whether chat is enabled. <br/>
    * <br/>
    * @return true if chat is enabled, false otherwise <br/>
    */
    public boolean chatEnabled();

    /**
    * Returns a list of chat rooms. <br/>
    * <br/>
    * @return List of chat rooms.    <br/>
    */
    @NonNull
    public ChatRoom[] getChatRooms();

    /**
    * Get a pointer on the internal conference object. <br/>
    * <br/>
    * @return A pointer on {@link Conference} or null if no conference are going on. <br/>
    *  <br/>
    */
    @Nullable
    public Conference getConference();

    /**
    * Get the set input volume of the local participant. <br/>
    * <br/>
    * @return A value inside [0.0 ; 1.0] <br/>
    */
    public float getConferenceLocalInputVolume();

    /**
    * Tells whether the conference server feature is enabled. <br/>
    * <br/>
    * @return A boolean value telling whether the conference server feature is<br/>
    * enabled or not <br/>
    */
    public boolean conferenceServerEnabled();

    /**
    * Enable the conference server feature. <br/>
    * <br/>
    * This has the effect to listen of the conference factory uri to create new<br/>
    * conferences when receiving INVITE messages there. <br/>
    * @param enable A boolean value telling whether to enable or disable the<br/>
    * conference server feature <br/>
    */
    public void enableConferenceServer(boolean enable);

    /**
    * Get the number of participant in the running conference. <br/>
    * <br/>
    * The local participant is included in the count only if it is in the conference. <br/>
    * @return The number of participant <br/>
    */
    public int getConferenceSize();

    /**
    * Returns the config object used to manage the storage (config) file. <br/>
    * <br/>
    * @return a {@link Config} object.   <br/>
    */
    @NonNull
    public Config getConfig();

    /**
    * Get my consolidated presence. <br/>
    * <br/>
    * @return My {@link ConsolidatedPresence} presence <br/>
    */
    public ConsolidatedPresence getConsolidatedPresence();

    /**
    * Set my consolidated presence. <br/>
    * <br/>
    * @param presence {@link ConsolidatedPresence} value <br/>
    */
    public void setConsolidatedPresence(ConsolidatedPresence presence);

    /**
    * Gets the current call. <br/>
    * <br/>
    * @return The current call or null if no call is running.   <br/>
    */
    @Nullable
    public Call getCurrentCall();

    /**
    * Get the remote address of the current call. <br/>
    * <br/>
    * @return The remote address of the current call or null if there is no current<br/>
    * call.   <br/>
    */
    @Nullable
    public Address getCurrentCallRemoteAddress();

    /**
    * Get the effective video definition provided by the camera for the captured<br/>
    * video. <br/>
    * <br/>
    * When preview is disabled or not yet started this function returns a 0x0 video<br/>
    * definition. <br/>
    * @return The captured {@link VideoDefinition}.  <br/>
    * see: {@link #setPreviewVideoDefinition} <br/>
    */
    @NonNull
    public VideoDefinition getCurrentPreviewVideoDefinition();

    /**
    * Retrieves the first list of {@link Friend} from the core. <br/>
    * <br/>
    * @return the first {@link FriendList} object or null.   <br/>
    */
    @Nullable
    public FriendList getDefaultFriendList();

    /**
    * Gets the default input audio device. <br/>
    * <br/>
    * @return The default input audio device   <br/>
    */
    @NonNull
    public AudioDevice getDefaultInputAudioDevice();

    /**
    * Sets the given {@link AudioDevice} as default input for next calls. <br/>
    * <br/>
    * @param audioDevice The {@link AudioDevice}   <br/>
    */
    public void setDefaultInputAudioDevice(@NonNull AudioDevice audioDevice);

    /**
    * Gets the default output audio device. <br/>
    * <br/>
    * @return The default output audio device   <br/>
    */
    @NonNull
    public AudioDevice getDefaultOutputAudioDevice();

    /**
    * Sets the given {@link AudioDevice} as default output for next calls. <br/>
    * <br/>
    * @param audioDevice The {@link AudioDevice}   <br/>
    */
    public void setDefaultOutputAudioDevice(@NonNull AudioDevice audioDevice);

    /**
    * Returns the default proxy configuration, that is the one used to determine the<br/>
    * current identity. <br/>
    * <br/>
    * @return The default proxy configuration.   <br/>
    */
    @Nullable
    public ProxyConfig getDefaultProxyConfig();

    /**
    * Sets the default proxy. <br/>
    * <br/>
    * This default proxy must be part of the list of already entered<br/>
    * LinphoneProxyConfig. Toggling it as default will make {@link Core} use the<br/>
    * identity associated with the proxy configuration in all incoming and outgoing<br/>
    * calls. <br/>
    * @param config The proxy configuration to use as the default one.   <br/>
    */
    public void setDefaultProxyConfig(@Nullable ProxyConfig config);

    /**
    * Get the name of the default mediastreamer2 filter used for rendering video on<br/>
    * the current platform. <br/>
    * <br/>
    * This is for advanced users of the library, mainly to expose mediastreamer video<br/>
    * filter name and status. <br/>
    * @return The default video display filter.   <br/>
    */
    @NonNull
    public String getDefaultVideoDisplayFilter();

    /**
    * Gets the delayed timeout See {@link #setDelayedTimeout} for details. <br/>
    * <br/>
    * @return The current delayed timeout in seconds <br/>
    */
    public int getDelayedTimeout();

    /**
    * Set the in delayed timeout in seconds. <br/>
    * <br/>
    * After this timeout period, a delayed call (internal call initialisation or<br/>
    * resolution) is resumed. <br/>
    * @param seconds The new delayed timeout <br/>
    */
    public void setDelayedTimeout(int seconds);

    /**
    * Gets the current device orientation. <br/>
    * <br/>
    * @return The current device orientation<br/>
    * see: {@link #setDeviceRotation} <br/>
    */
    public int getDeviceRotation();

    /**
    * Tells the core the device current orientation. <br/>
    * <br/>
    * This can be used by capture filters on mobile devices to select between<br/>
    * portrait/landscape mode and to produce properly oriented images. The exact<br/>
    * meaning of the value in rotation if left to each device specific<br/>
    * implementations. IOS supported values are 0 for UIInterfaceOrientationPortrait<br/>
    * and 270 for UIInterfaceOrientationLandscapeRight. <br/>
    * @param rotation The orientation to use <br/>
    */
    public void setDeviceRotation(int rotation);

    /**
    * Get whether the microphone will be completely deactivated when muted. <br/>
    * <br/>
    * Please refer to {@link #setDisableRecordOnMute}.<br/>
    * @return True if you wish to entirely stop the audio recording when muting the<br/>
    * microphone. <br/>
    */
    public boolean getDisableRecordOnMute();

    /**
    * Calling this method with disable=true will cause the microhone to be completely<br/>
    * deactivated when muted, when given possible by the implementation on the<br/>
    * platform on which liblinphone is running. <br/>
    * <br/>
    * Otherwise, sound recording remains active but silence is sent instead of<br/>
    * recorded audio. Playback sound will be briefly interrupted while the audio is<br/>
    * reconfigured. Currently only implemented for IOS, it will also disable Apple's<br/>
    * microphone recording indicator when microphone is muted.<br/>
    * @param disable True if you wish to entirely stop the audio recording when<br/>
    * muting the microphone. <br/>
    */
    public void setDisableRecordOnMute(boolean disable);

    /**
    * Tells whether DNS search (use of local domain if the fully qualified name did<br/>
    * return results) is enabled. <br/>
    * <br/>
    * @return true if DNS search is enabled, false if disabled. <br/>
    */
    public boolean dnsSearchEnabled();

    /**
    * Enable or disable DNS search (use of local domain if the fully qualified name<br/>
    * did return results). <br/>
    * <br/>
    * @param enable true to enable DNS search, false to disable it. <br/>
    */
    public void enableDnsSearch(boolean enable);

    /**
    * Forces liblinphone to use the supplied list of dns servers, instead of system's<br/>
    * ones. <br/>
    * <br/>
    * @param servers A list of strings containing the IP addresses of DNS servers to<br/>
    * be used.  Setting to null restores default behaviour, which is to use the DNS<br/>
    * server list provided by the system. The list is copied internally.   <br/>
    */
    public void setDnsServers(@Nullable String[] servers);

    /**
    * Forces liblinphone to use the supplied list of dns servers, instead of system's<br/>
    * ones and set dns_set_by_app at true or false according to value of servers<br/>
    * list. <br/>
    * <br/>
    * @param servers A list of strings containing the IP addresses of DNS servers to<br/>
    * be used.  Setting to null restores default behaviour, which is to use the DNS<br/>
    * server list provided by the system. The list is copied internally.   <br/>
    */
    public void setDnsServersApp(@Nullable String[] servers);

    /**
    * Tells if the DNS was set by an application. <br/>
    * <br/>
    * @return true if DNS was set by app, false otherwise. <br/>
    */
    public boolean getDnsSetByApp();

    /**
    * Tells whether DNS SRV resolution is enabled. <br/>
    * <br/>
    * @return true if DNS SRV resolution is enabled, false if disabled. <br/>
    */
    public boolean dnsSrvEnabled();

    /**
    * Enable or disable DNS SRV resolution. <br/>
    * <br/>
    * @param enable true to enable DNS SRV resolution, false to disable it. <br/>
    */
    public void enableDnsSrv(boolean enable);

    /**
    * Retrieve the maximum available download bandwidth. <br/>
    * <br/>
    * This value was set by {@link #setDownloadBandwidth}. <br/>
    * @return the download bandiwdth in kbits/s, 0 for infinite <br/>
    */
    public int getDownloadBandwidth();

    /**
    * Sets maximum available download bandwidth This is IP bandwidth, in kbit/s. <br/>
    * <br/>
    * This information is used signaled to other parties during calls (within SDP<br/>
    * messages) so that the remote end can have sufficient knowledge to properly<br/>
    * configure its audio & video codec output bitrate to not overflow available<br/>
    * bandwidth.<br/>
    * @param bandwidth the bandwidth in kbits/s, 0 for infinite <br/>
    */
    public void setDownloadBandwidth(int bandwidth);

    /**
    * Get audio packetization time linphone expects to receive from peer. <br/>
    * <br/>
    * A value of zero means that ptime is not specified. <br/>
    * @return the download packetization time set <br/>
    */
    public int getDownloadPtime();

    /**
    * Set audio packetization time linphone expects to receive from peer. <br/>
    * <br/>
    * A value of zero means that ptime is not specified. <br/>
    * @param ptime the download packetization time to set <br/>
    */
    public void setDownloadPtime(int ptime);

    /**
    * Returns true if echo cancellation is enabled. <br/>
    * <br/>
    * @return A boolean value telling whether echo cancellation is enabled or<br/>
    * disabled <br/>
    */
    public boolean echoCancellationEnabled();

    /**
    * Enables or disable echo cancellation. <br/>
    * <br/>
    * Value is saved and used for subsequent calls. This actually controls software<br/>
    * echo cancellation. If hardware echo cancellation is available, it will be<br/>
    * always used and activated for calls, regardless of the value passed to this<br/>
    * function. When hardware echo cancellation is available, the software one is of<br/>
    * course not activated. <br/>
    * @param enable A boolean value telling whether echo cancellation is to be<br/>
    * enabled or disabled. <br/>
    */
    public void enableEchoCancellation(boolean enable);

    /**
    * Get the name of the mediastreamer2 filter used for echo cancelling. <br/>
    * <br/>
    * @return The name of the mediastreamer2 filter used for echo cancelling.   <br/>
    */
    @Nullable
    public String getEchoCancellerFilterName();

    /**
    * Set the name of the mediastreamer2 filter to be used for echo cancelling. <br/>
    * <br/>
    * This is for advanced users of the library. <br/>
    * @param filtername The name of the mediastreamer2 filter to be used for echo<br/>
    * cancelling.   <br/>
    */
    public void setEchoCancellerFilterName(@Nullable String filtername);

    /**
    * Tells whether echo limiter is enabled. <br/>
    * <br/>
    * @return true if the echo limiter is enabled, false otherwise. <br/>
    */
    public boolean echoLimiterEnabled();

    /**
    * Enables or disable echo limiter. <br/>
    * <br/>
    * @param enable true to enable echo limiter, false to disable it. <br/>
    */
    public void enableEchoLimiter(boolean enable);

    /**
    * Enable or disable the UPDATE method support. <br/>
    * <br/>
    * @param value Enable or disable it <br/>
    */
    public void setEnableSipUpdate(int value);

    /**
    * Sets expected available upload bandwidth This is IP bandwidth, in kbit/s. <br/>
    * <br/>
    * This information is used by liblinphone together with remote side available<br/>
    * bandwidth signaled in SDP messages to properly configure audio & video codec's<br/>
    * output bitrate.<br/>
    * @param bandwidth the bandwidth in kbits/s, 0 for infinite <br/>
    */
    public void setExpectedBandwidth(int bandwidth);

    /**
    * Returns the list of all audio devices. <br/>
    * <br/>
    * @return   A list of all {@link AudioDevice}     <br/>
    */
    @NonNull
    public AudioDevice[] getExtendedAudioDevices();

    /**
    * Get the globaly set http file transfer server to be used for content type<br/>
    * application/vnd.gsma.rcs-ft-http+xml. <br/>
    * <br/>
    * Url may be like: "https://file.linphone.org/upload.php". <br/>
    * @return URL of the file server.   <br/>
    */
    @Nullable
    public String getFileTransferServer();

    /**
    * Globaly set an http file transfer server to be used for content type<br/>
    * application/vnd.gsma.rcs-ft-http+xml. <br/>
    * <br/>
    * Url may be like: "https://file.linphone.org/upload.php". This value can also be<br/>
    * set for a dedicated account using<br/>
    * linphone_proxy_config_set_file_transfer_server. <br/>
    * @param serverUrl URL of the file server.   <br/>
    */
    public void setFileTransferServer(@Nullable String serverUrl);

    /**
    * Indicates whether the ICE relay path is forcibly selected. <br/>
    * <br/>
    * @return a boolean value indicating whether forced relay is enabled.<br/>
    * see: {@link #enableForcedIceRelay}. <br/>
    */
    public boolean forcedIceRelayEnabled();

    /**
    * Artificially cause the relay path to be selected when ICE is used. <br/>
    * <br/>
    * This is mainly a function for test, for example to validate that the relay<br/>
    * service (ever TURN or media-aware SIP proxy) is working as expected. Indeed, in<br/>
    * many cases a path through host or server reflexive candidate will be found by<br/>
    * ICE, which makes difficult to make sure that the relay service is working as<br/>
    * expected. <br/>
    * @param enable boolean value <br/>
    */
    public void enableForcedIceRelay(boolean enable);

    /**
    * Sets whether or not to start friend lists subscription when in foreground. <br/>
    * <br/>
    * @param enable whether or not to enable the feature <br/>
    */
    public void enableFriendListSubscription(boolean enable);

    /**
    * Gets the database filename where friends will be stored. <br/>
    * <br/>
    * @return filesystem path.   <br/>
    */
    @Nullable
    public String getFriendsDatabasePath();

    /**
    * Sets the database filename where friends will be stored. <br/>
    * <br/>
    * If the file does not exist, it will be created.<br/>
    * @param path filesystem path.   <br/>
    */
    public void setFriendsDatabasePath(@Nullable String path);

    /**
    * Retrieves the list of {@link FriendList} from the core. <br/>
    * <br/>
    * @return A list of {@link FriendList}.    <br/>
    */
    @NonNull
    public FriendList[] getFriendsLists();

    /**
    * Returns the global state of core. <br/>
    * <br/>
    * @return a {@link GlobalState} enum.   <br/>
    */
    @NonNull
    public GlobalState getGlobalState();

    /**
    * Returns true if hostname part of primary contact is guessed automatically. <br/>
    * <br/>
    * @return true if guess hostname enabled, false otherwise. <br/>
    */
    public boolean getGuessHostname();

    /**
    * Tells {@link Core} to guess local hostname automatically in primary contact. <br/>
    * <br/>
    * @param enable whether to enable the guess hostname feature or not <br/>
    */
    public void setGuessHostname(boolean enable);

    /**
    * Get http proxy address to be used for signaling. <br/>
    * <br/>
    * @return hostname of IP adress of the http proxy (can be null to disable).   <br/>
    */
    @Nullable
    public String getHttpProxyHost();

    /**
    * Set http proxy address to be used for signaling during next channel connection. <br/>
    * <br/>
    * Use {@link #setNetworkReachable} FASLE/true to force channel restart. <br/>
    * @param host Hostname of IP adress of the http proxy (can be null to disable).   <br/>
    */
    public void setHttpProxyHost(@Nullable String host);

    /**
    * Get http proxy port to be used for signaling. <br/>
    * <br/>
    * @return port of the http proxy. <br/>
    */
    public int getHttpProxyPort();

    /**
    * Set http proxy port to be used for signaling. <br/>
    * <br/>
    * @param port of the http proxy. <br/>
    */
    public void setHttpProxyPort(int port);

    /**
    * Gets the default identity SIP address. <br/>
    * <br/>
    * This is an helper function. If no default proxy is set, this will return the<br/>
    * primary contact ( see {@link #getPrimaryContact} ). If a default proxy is set<br/>
    * it returns the registered identity on the proxy. <br/>
    * @return The default identity SIP address.   <br/>
    */
    @NonNull
    public String getIdentity();

    /**
    * Get the {@link ImNotifPolicy} object controlling the instant messaging<br/>
    * notifications. <br/>
    * <br/>
    * @return A {@link ImNotifPolicy} object.   <br/>
    */
    @Nullable
    public ImNotifPolicy getImNotifPolicy();

    /**
    * Gets the in call timeout See {@link #setInCallTimeout} for details. <br/>
    * <br/>
    * @return The current in call timeout in seconds <br/>
    */
    public int getInCallTimeout();

    /**
    * Set the in call timeout in seconds. <br/>
    * <br/>
    * After this timeout period, the call is automatically hangup. <br/>
    * @param seconds The new timeout in seconds <br/>
    */
    public void setInCallTimeout(int seconds);

    /**
    * Returns the incoming call timeout See {@link #setIncTimeout} for details. <br/>
    * <br/>
    * @return The current incoming call timeout in seconds <br/>
    */
    public int getIncTimeout();

    /**
    * Set the incoming call timeout in seconds. <br/>
    * <br/>
    * If an incoming call isn't answered for this timeout period, it is automatically<br/>
    * declined. <br/>
    * @param seconds The new timeout in seconds <br/>
    */
    public void setIncTimeout(int seconds);

    /**
    * Gets the input audio device for the current call. <br/>
    * <br/>
    * @return The input audio device for the current or first call, null if there is<br/>
    * no call.   <br/>
    */
    @Nullable
    public AudioDevice getInputAudioDevice();

    /**
    * Sets the given {@link AudioDevice} as input for all active calls. <br/>
    * <br/>
    * @param audioDevice The {@link AudioDevice}. null does nothing.   <br/>
    */
    public void setInputAudioDevice(@Nullable AudioDevice audioDevice);

    /**
    * Tells whether IPv6 is enabled or not. <br/>
    * <br/>
    * @return A boolean value telling whether IPv6 is enabled or not<br/>
    * see: {@link #enableIpv6} for more details on how IPv6 is supported in<br/>
    * liblinphone. <br/>
    */
    public boolean ipv6Enabled();

    /**
    * Turns IPv6 support on or off. <br/>
    * <br/>
    * @param enable A boolean value telling whether to enable IPv6 support <br/>
    */
    public void enableIpv6(boolean enable);

    /**
    * Gets whether auto iterate is enabled or not (Android & iOS only). <br/>
    * <br/>
    * @return true if {@link #iterate} is scheduled automatically, false otherwise <br/>
    */
    public boolean isAutoIterateEnabled();

    /**
    * Check whether the device is echo canceller calibration is required. <br/>
    * <br/>
    * @return true if it is required, false otherwise <br/>
    */
    public boolean isEchoCancellerCalibrationRequired();

    /**
    * Returns whether or not friend lists subscription are enabled. <br/>
    * <br/>
    * @return whether or not the feature is enabled <br/>
    */
    public boolean isFriendListSubscriptionEnabled();

    /**
    * Gets whether the Core is considering itself in background or not. <br/>
    * <br/>
    * The Core foreground/background state depends on the last call made to {@link #enterBackground}<br/>
    * or {@link #enterForeground}. @return true if the Core is in background, false<br/>
    * otherwise. <br/>
    */
    public boolean isInBackground();

    /**
    * Indicates whether the local participant is part of a conference. <br/>
    * <br/>
    * warning: That function automatically fails in the case of conferences using a<br/>
    * conferencet server (focus). If you use such a conference, you should use {@link Conference#removeParticipant}<br/>
    * instead. <br/>
    * @return true if the local participant is in a conference, false otherwise. <br/>
    */
    public boolean isInConference();

    /**
    * Tells whether there is an incoming invite pending. <br/>
    * <br/>
    * @return A boolean telling whether an incoming invite is pending or not. <br/>
    */
    public boolean isIncomingInvitePending();

    /**
    * Check if the configured media encryption is mandatory or not. <br/>
    * <br/>
    * @return true if media encryption is mandatory; false otherwise. <br/>
    */
    public boolean isMediaEncryptionMandatory();

    /**
    * Returns whether the native ringing is enabled or not. <br/>
    * <br/>
    * @return True if we use the native ringing, false otherwise <br/>
    */
    public boolean isNativeRingingEnabled();

    /**
    * return network state either as positioned by the application or by linphone<br/>
    * itself. <br/>
    * <br/>
    * @return true if network is reachable, false otherwise <br/>
    */
    public boolean isNetworkReachable();

    /**
    * Gets whether push notifications are available or not (Android & iOS only). <br/>
    * <br/>
    * @return true if push notifications are available, false otherwise <br/>
    */
    public boolean isPushNotificationAvailable();

    /**
    * Gets whether push notifications are enabled or not (Android & iOS only). <br/>
    * <br/>
    * @return true if push notifications are enabled, false otherwise <br/>
    */
    public boolean isPushNotificationEnabled();

    /**
    * Returns whether or not sender name is hidden in forward message. <br/>
    * <br/>
    * @return whether or not the feature <br/>
    */
    public boolean isSenderNameHiddenInForwardMessage();

    /**
    * Gets whether the device will vibrate while an incoming call is ringing (Android<br/>
    * only). <br/>
    * <br/>
    * @return true if the device will vibrate (if possible), false otherwise <br/>
    */
    public boolean isVibrationOnIncomingCallEnabled();

    /**
    * Is signaling keep alive enabled. <br/>
    * <br/>
    * @return A boolean value telling whether signaling keep alive is enabled <br/>
    */
    public boolean keepAliveEnabled();

    /**
    * Enables signaling keep alive, small udp packet sent periodically to keep udp<br/>
    * NAT association. <br/>
    * <br/>
    * @param enable A boolean value telling whether signaling keep alive is to be<br/>
    * enabled <br/>
    */
    public void enableKeepAlive(boolean enable);

    /**
    * Get the latest outgoing call log. <br/>
    * <br/>
    * @return The last outgoing call log if any.   <br/>
    */
    @Nullable
    public CallLog getLastOutgoingCallLog();

    /**
    * Tells wether LIME X3DH is enabled or not. <br/>
    * <br/>
    * @return The current lime state <br/>
    */
    public boolean limeX3DhEnabled();

    /**
    * Tells to LinphoneCore to use LIME X3DH. <br/>
    * <br/>
    * @param enable A boolean value telling whether to enable or disable LIME X3DH <br/>
    */
    public void enableLimeX3Dh(boolean enable);

    /**
    * Get the x3dh server url. <br/>
    * <br/>
    * @return The x3dh server url.   <br/>
    */
    @Nullable
    public String getLimeX3DhServerUrl();

    /**
    * Set the x3dh server url. <br/>
    * <br/>
    * If empty, this function will disable LIME X3DH from core. Otherwise, or if<br/>
    * different from the existing value, this will (re-)initialize the LIME X3DH<br/>
    * engine. <br/>
    * @param url The x3dh server url.   <br/>
    */
    public void setLimeX3DhServerUrl(@Nullable String url);

    /**
    * Get the linphone specs value telling what functionalities the linphone client<br/>
    * supports. <br/>
    * <br/>
    * @return The linphone specs telling what functionalities the linphone client<br/>
    * supports  <br/>
    * @deprecated 07/02/2019 Use linphone_core_get_linphone_specs_list instead. <br/>
    */
    @Deprecated
    @Nullable
    public String getLinphoneSpecs();

    /**
    * Set the linphone specs value telling what functionalities the linphone client<br/>
    * supports. <br/>
    * <br/>
    * @param specs The linphone specs to set  <br/>
    * @deprecated 07/02/2019 Use linphone_core_set_linphone_specs_list or<br/>
    * linphone_core_add_linphone_spec instead. <br/>
    */
    @Deprecated
    public void setLinphoneSpecs(@Nullable String specs);

    /**
    * Get the list of linphone specs string values representing what functionalities<br/>
    * the linphone client supports. <br/>
    * <br/>
    * @return A list of supported specs. The list must be freed with<br/>
    * bctbx_list_free() after usage.    <br/>
    */
    @NonNull
    public String[] getLinphoneSpecsList();

    /**
    * Set the linphone specs list value telling what functionalities the linphone<br/>
    * client supports. <br/>
    * <br/>
    * @param specs The list of string specs to set.    <br/>
    */
    public void setLinphoneSpecsList(@Nullable String[] specs);

    /**
    * Gets the url of the server where to upload the collected log files. <br/>
    * <br/>
    * @return The url of the server where to upload the collected log files.   <br/>
    */
    @Nullable
    public String getLogCollectionUploadServerUrl();

    /**
    * Set the url of the server where to upload the collected log files. <br/>
    * <br/>
    * @param serverUrl The url of the server where to upload the collected log files.<br/>
    *   <br/>
    */
    public void setLogCollectionUploadServerUrl(@Nullable String serverUrl);

    /**
    * Get the maximum number of simultaneous calls Linphone core can manage at a<br/>
    * time. <br/>
    * <br/>
    * All new call above this limit are declined with a busy answer <br/>
    * @return max number of simultaneous calls <br/>
    */
    public int getMaxCalls();

    /**
    * Set the maximum number of simultaneous calls Linphone core can manage at a<br/>
    * time. <br/>
    * <br/>
    * All new call above this limit are declined with a busy answer <br/>
    * @param max number of simultaneous calls <br/>
    */
    public void setMaxCalls(int max);

    /**
    * Gets the size under which incoming files in chat messages will be downloaded<br/>
    * automatically. <br/>
    * <br/>
    * @return The size in bytes, -1 if autodownload feature is disabled, 0 to<br/>
    * download them all no matter the size <br/>
    */
    public int getMaxSizeForAutoDownloadIncomingFiles();

    /**
    * Sets the size under which incoming files in chat messages will be downloaded<br/>
    * automatically. <br/>
    * <br/>
    * @param size The size in bytes, -1 to disable the autodownload feature, 0 to<br/>
    * download them all no matter the size <br/>
    */
    public void setMaxSizeForAutoDownloadIncomingFiles(int size);

    /**
    * Gets the name of the currently assigned sound device for media. <br/>
    * <br/>
    * @return The name of the currently assigned sound device for capture.   <br/>
    */
    @Nullable
    public String getMediaDevice();

    /**
    * Sets the sound device used for media. <br/>
    * <br/>
    * @param devid The device name as returned by linphone_core_get_sound_devices   <br/>
    * @return 0 <br/>
    */
    public int setMediaDevice(@Nullable String devid);

    /**
    * Get the media encryption policy being used for RTP packets. <br/>
    * <br/>
    * @return The {@link MediaEncryption} policy being used. <br/>
    */
    public MediaEncryption getMediaEncryption();

    /**
    * Choose the media encryption policy to be used for RTP packets. <br/>
    * <br/>
    * @param menc The media encryption policy to be used. <br/>
    * @return 0 if successful, any other value otherwise. <br/>
    */
    public int setMediaEncryption(MediaEncryption menc);

    /**
    * Define whether the configured media encryption is mandatory, if it is and the<br/>
    * negotation cannot result in the desired media encryption then the call will<br/>
    * fail. <br/>
    * <br/>
    * If not an INVITE will be resent with encryption disabled. <br/>
    * @param mandatory true to set it mandatory; false otherwise. <br/>
    */
    public void setMediaEncryptionMandatory(boolean mandatory);

    /**
    * This method is called by the application to notify the linphone core library<br/>
    * when the media (RTP) network is reachable. <br/>
    * <br/>
    * This is for advanced usage, when SIP and RTP layers are required to use<br/>
    * different interfaces. Most applications just need {@link #setNetworkReachable}.<br/>
    * @param reachable true if network is reachable, false otherwise <br/>
    */
    public void setMediaNetworkReachable(boolean reachable);

    /**
    * This function returns the media resource mode for this core. <br/>
    * <br/>
    * @return The media resource mode <br/>
    */
    public MediaResourceMode getMediaResourceMode();

    /**
    * Sets the media resources mode. <br/>
    * <br/>
    * Value values are: unique and shared. When the mode is set to unique, then only<br/>
    * one call in the state StreamsRunning is allowed. While acepting a call, the<br/>
    * core will try to free media resource used by the current call. If it is<br/>
    * unsuccessful, then the call is not accepted. If mode is set to shared, then the<br/>
    * media resources of the current call (if any) are not emptied when taking a new<br/>
    * call. If the user whishes to free them, he/she is responsible to call<br/>
    * linphone_core_preempt_sound_resources himself/herself <br/>
    * @param mode the chosen mode <br/>
    */
    public void setMediaResourceMode(MediaResourceMode mode);

    /**
    * Tells whether the microphone is enabled. <br/>
    * <br/>
    * @return true if the microphone is enabled, false if disabled. <br/>
    */
    public boolean micEnabled();

    /**
    * Enable or disable the microphone. <br/>
    * <br/>
    * @param enable true to enable the microphone, false to disable it. <br/>
    */
    public void enableMic(boolean enable);

    /**
    * Get microphone gain in db. <br/>
    * <br/>
    * @return The current microphone gain <br/>
    */
    public float getMicGainDb();

    /**
    * Allow to control microphone level: gain in db. <br/>
    * <br/>
    * @param level The new microphone level <br/>
    */
    public void setMicGainDb(float level);

    /**
    * Get the number of missed calls. <br/>
    * <br/>
    * Once checked, this counter can be reset with {@link #resetMissedCallsCount}. <br/>
    * @return The number of missed calls. <br/>
    */
    public int getMissedCallsCount();

    /**
    * Returns the maximum transmission unit size in bytes. <br/>
    * <br/>
    */
    public int getMtu();

    /**
    * Sets the maximum transmission unit size in bytes. <br/>
    * <br/>
    * This information is useful for sending RTP packets. Default value is 1500. <br/>
    * @param mtu The MTU in bytes <br/>
    */
    public void setMtu(int mtu);

    /**
    * Get the public IP address of NAT being used. <br/>
    * <br/>
    * @return The public IP address of NAT being used.   <br/>
    */
    @Nullable
    public String getNatAddress();

    /**
    * Set the public IP address of NAT when using the firewall policy is set to use<br/>
    * NAT. <br/>
    * <br/>
    * @param addr The public IP address of NAT to use.   <br/>
    */
    public void setNatAddress(@Nullable String addr);

    /**
    * Get The policy that is used to pass through NATs/firewalls. <br/>
    * <br/>
    * It may be overridden by a NAT policy for a specific proxy config. <br/>
    * @return {@link NatPolicy} object in use.  <br/>
    * see: {@link ProxyConfig#getNatPolicy} <br/>
    */
    @Nullable
    public NatPolicy getNatPolicy();

    /**
    * Set the policy to use to pass through NATs/firewalls. <br/>
    * <br/>
    * It may be overridden by a NAT policy for a specific proxy config. <br/>
    * @param policy {@link NatPolicy} object  <br/>
    * see: {@link ProxyConfig#setNatPolicy} <br/>
    */
    public void setNatPolicy(@Nullable NatPolicy policy);

    /**
    * Get the native window handle of the video preview window. <br/>
    * <br/>
    * @return The native window handle of the video preview window.   <br/>
    */
    @Nullable
    public Object getNativePreviewWindowId();

    /**
    * Set the native window id where the preview video (local camera) is to be<br/>
    * displayed. <br/>
    * <br/>
    * This has to be used in conjonction with {@link #usePreviewWindow}. MacOS,<br/>
    * Linux, Windows: if not set or zero the core will create its own window, unless<br/>
    * the special id -1 is given. <br/>
    * @param windowId The native window id where the preview video is to be<br/>
    * displayed.   <br/>
    */
    public void setNativePreviewWindowId(@Nullable Object windowId);

    /**
    * Sets whether to use the native ringing (Android only). <br/>
    * <br/>
    * @param enable True to enable native ringing, false otherwise <br/>
    */
    public void setNativeRingingEnabled(boolean enable);

    /**
    * Get the native window handle of the video window. <br/>
    * <br/>
    * @return The native window handle of the video window.   <br/>
    */
    @Nullable
    public Object getNativeVideoWindowId();

    /**
    * Set the native video window id where the video is to be displayed. <br/>
    * <br/>
    * For MacOS, Linux, Windows: if not set or LINPHONE_VIDEO_DISPLAY_AUTO the core<br/>
    * will create its own window, unless the special id LINPHONE_VIDEO_DISPLAY_NONE<br/>
    * is given. <br/>
    * @param windowId The native window id where the remote video is to be displayed.<br/>
    *   <br/>
    */
    public void setNativeVideoWindowId(@Nullable Object windowId);

    /**
    * This method is called by the application to notify the linphone core library<br/>
    * when network is reachable. <br/>
    * <br/>
    * Calling this method with true trigger linphone to initiate a registration<br/>
    * process for all proxies. Calling this method disables the automatic network<br/>
    * detection mode. It means you must call this method after each network state<br/>
    * changes.<br/>
    * @param reachable true if network is reachable, false otherwise <br/>
    */
    public void setNetworkReachable(boolean reachable);

    /**
    * Gets the value of the no-rtp timeout. <br/>
    * <br/>
    * When no RTP or RTCP packets have been received for a while {@link Core} will<br/>
    * consider the call is broken (remote end crashed or disconnected from the<br/>
    * network), and thus will terminate the call. The no-rtp timeout is the duration<br/>
    * above which the call is considered broken. <br/>
    * @return The value of the no-rtp timeout in seconds <br/>
    */
    public int getNortpTimeout();

    /**
    * Sets the no-rtp timeout value in seconds. <br/>
    * <br/>
    * @param seconds The no-rtp timeout value to use in seconds<br/>
    * see: {@link #getNortpTimeout} for details. <br/>
    */
    public void setNortpTimeout(int seconds);

    /**
    * Gets the output audio device for the current call. <br/>
    * <br/>
    * @return The output audio device for the current or first call, null if there is<br/>
    * no call.   <br/>
    */
    @Nullable
    public AudioDevice getOutputAudioDevice();

    /**
    * Sets the given {@link AudioDevice} as output for all active calls. <br/>
    * <br/>
    * @param audioDevice The {@link AudioDevice}. null does nothing.   <br/>
    */
    public void setOutputAudioDevice(@Nullable AudioDevice audioDevice);

    /**
    * Get the wav file that is played when putting somebody on hold, or when files<br/>
    * are used instead of soundcards (see {@link #setUseFiles}). <br/>
    * <br/>
    * The file is a 16 bit linear wav file. <br/>
    * @return The path to the file that is played when putting somebody on hold.   <br/>
    */
    @Nullable
    public String getPlayFile();

    /**
    * Sets a wav file to be played when putting somebody on hold, or when files are<br/>
    * used instead of soundcards (see {@link #setUseFiles}). <br/>
    * <br/>
    * The file must be a 16 bit linear wav file. <br/>
    * @param file The path to the file to be played when putting somebody on hold.   <br/>
    */
    public void setPlayFile(@Nullable String file);

    /**
    * Gets the name of the currently assigned sound device for playback. <br/>
    * <br/>
    * @return The name of the currently assigned sound device for playback.   <br/>
    */
    @Nullable
    public String getPlaybackDevice();

    /**
    * Sets the sound device used for playback. <br/>
    * <br/>
    * @param devid The device name as returned by linphone_core_get_sound_devices   <br/>
    * @return 0 <br/>
    */
    public int setPlaybackDevice(@Nullable String devid);

    /**
    * Get playback gain in db before entering sound card. <br/>
    * <br/>
    * @return The current playback gain <br/>
    */
    public float getPlaybackGainDb();

    /**
    * Allow to control play level before entering sound card: gain in db. <br/>
    * <br/>
    * @param level The new play level <br/>
    */
    public void setPlaybackGainDb(float level);

    /**
    * Returns the preferred video framerate, previously set by {@link #setPreferredFramerate}<br/>
    * . <br/>
    * <br/>
    * @return frame rate in number of frames per seconds. <br/>
    */
    public float getPreferredFramerate();

    /**
    * Set the preferred frame rate for video. <br/>
    * <br/>
    * Based on the available bandwidth constraints and network conditions, the video<br/>
    * encoder remains free to lower the framerate. There is no warranty that the<br/>
    * preferred frame rate be the actual framerate. used during a call. Default value<br/>
    * is 0, which means "use encoder's default fps value". <br/>
    * @param fps the target frame rate in number of frames per seconds. <br/>
    */
    public void setPreferredFramerate(float fps);

    /**
    * Get the preferred video definition for the stream that is captured and sent to<br/>
    * the remote party. <br/>
    * <br/>
    * @return The preferred {@link VideoDefinition}   <br/>
    */
    @NonNull
    public VideoDefinition getPreferredVideoDefinition();

    /**
    * Set the preferred video definition for the stream that is captured and sent to<br/>
    * the remote party. <br/>
    * <br/>
    * All standard video definitions are accepted on the receive path. <br/>
    * @param videoDefinition {@link VideoDefinition} object   <br/>
    */
    public void setPreferredVideoDefinition(@NonNull VideoDefinition videoDefinition);

    /**
    * Sets the preferred video definition by its name. <br/>
    * <br/>
    * Call {@link Factory#getSupportedVideoDefinitions} to have a list of supported<br/>
    * video definitions.<br/>
    * @param name The name of the definition to set   <br/>
    */
    public void setPreferredVideoDefinitionByName(@NonNull String name);

    /**
    * Get my presence model. <br/>
    * <br/>
    * @return A {@link PresenceModel} object, or null if no presence model has been<br/>
    * set.   <br/>
    */
    @Nullable
    public PresenceModel getPresenceModel();

    /**
    * Set my presence model. <br/>
    * <br/>
    * @param presence {@link PresenceModel}   <br/>
    */
    public void setPresenceModel(@Nullable PresenceModel presence);

    /**
    * Get the definition of the captured video. <br/>
    * <br/>
    * @return The captured {@link VideoDefinition} if it was previously set by {@link #setPreviewVideoDefinition}<br/>
    * , otherwise a 0x0 LinphoneVideoDefinition.   <br/>
    * see: {@link #setPreviewVideoDefinition} <br/>
    */
    @Nullable
    public VideoDefinition getPreviewVideoDefinition();

    /**
    * Set the video definition for the captured (preview) video. <br/>
    * <br/>
    * This method is for advanced usage where a video capture must be set<br/>
    * independently of the definition of the stream actually sent through the call.<br/>
    * This allows for example to have the preview window in High Definition even if<br/>
    * due to bandwidth constraint the sent video definition is small. Using this<br/>
    * feature increases the CPU consumption, since a rescaling will be done<br/>
    * internally. <br/>
    * @param videoDefinition {@link VideoDefinition} object   <br/>
    */
    public void setPreviewVideoDefinition(@Nullable VideoDefinition videoDefinition);

    /**
    * Returns the default identity when no proxy configuration is used. <br/>
    * <br/>
    * @return the primary contact identity   <br/>
    */
    @NonNull
    public String getPrimaryContact();

    /**
    * Sets the local "from" identity. <br/>
    * <br/>
    * This data is used in absence of any proxy configuration or when no default<br/>
    * proxy configuration is set. See {@link ProxyConfig} <br/>
    * @param contact the contact to set   <br/>
    * @return 0 if successful, -1 otherwise <br/>
    */
    public int setPrimaryContact(@NonNull String contact);

    /**
    * Same as {@link #getPrimaryContact} but the result is a {@link Address} object<br/>
    * instead of const char *. <br/>
    * <br/>
    * @return a {@link Address} object.    <br/>
    * @deprecated 22/10/2018 Use {@link #createPrimaryContactParsed} instead. <br/>
    */
    @Deprecated
    @Nullable
    public Address getPrimaryContactParsed();

    /**
    * Get provisioning URI. <br/>
    * <br/>
    * @return the provisioning URI.   <br/>
    */
    @Nullable
    public String getProvisioningUri();

    /**
    * Set URI where to download xml configuration file at startup. <br/>
    * <br/>
    * This can also be set from configuration file or factory config file, from<br/>
    * [misc] section, item "config-uri". Calling this function does not load the<br/>
    * configuration. It will write the value into configuration so that configuration<br/>
    * from remote URI will take place at next {@link Core} start. <br/>
    * @param uri the http or https uri to use in order to download the configuration.<br/>
    * Passing null will disable remote provisioning.   <br/>
    * @return -1 if uri could not be parsed, 0 otherwise. Note that this does not<br/>
    * check validity of URI endpoint nor scheme and download may still fail. <br/>
    */
    public int setProvisioningUri(@Nullable String uri);

    /**
    * Returns an unmodifiable list of entered proxy configurations. <br/>
    * <br/>
    * @return A list of {@link ProxyConfig}.    <br/>
    */
    @NonNull
    public ProxyConfig[] getProxyConfigList();

    /**
    * Returns the push incoming call timeout See {@link #setPushIncomingCallTimeout}<br/>
    * for details. <br/>
    * <br/>
    * @return The current push incoming call timeout in seconds <br/>
    */
    public int getPushIncomingCallTimeout();

    /**
    * Configure the minimum interval between a push notification and the<br/>
    * corresponding incoming INVITE. <br/>
    * <br/>
    * If exceeded, Linphone Call is transitioned to CallError and further incoming<br/>
    * invite associated to this push is declined if any. <br/>
    * @param seconds The new timeout in seconds <br/>
    */
    public void setPushIncomingCallTimeout(int seconds);

    /**
    * Enable or disable push notifications on Android & iOS. <br/>
    * <br/>
    * If enabled, it will try to get the push token add configure each proxy config<br/>
    * with push_notification_allowed set to true with push parameters. <br/>
    * @param enable true to enable push notifications, false to disable <br/>
    */
    public void setPushNotificationEnabled(boolean enable);

    /**
    * Tells whether QRCode is enabled in the preview. <br/>
    * <br/>
    * @return A boolean value telling whether QRCode is enabled in the preview. <br/>
    */
    public boolean qrcodeVideoPreviewEnabled();

    /**
    * Controls QRCode enablement. <br/>
    * <br/>
    * @param enable A boolean value telling whether the QRCode is enabled in the<br/>
    * preview. <br/>
    */
    public void enableQrcodeVideoPreview(boolean enable);

    /**
    * Gets if realtime text is enabled or not. <br/>
    * <br/>
    * @return true if realtime text is enabled, false otherwise <br/>
    */
    public boolean realtimeTextEnabled();

    /**
    * Get the wav file where incoming stream is recorded, when files are used instead<br/>
    * of soundcards (see {@link #setUseFiles}). <br/>
    * <br/>
    * This feature is different from call recording ({@link CallParams#setRecordFile}<br/>
    * ) The file is a 16 bit linear wav file. <br/>
    * @return The path to the file where incoming stream is recorded.   <br/>
    */
    @Nullable
    public String getRecordFile();

    /**
    * Sets a wav file where incoming stream is to be recorded, when files are used<br/>
    * instead of soundcards (see {@link #setUseFiles}). <br/>
    * <br/>
    * This feature is different from call recording ({@link CallParams#setRecordFile}<br/>
    * ) The file will be a 16 bit linear wav file. <br/>
    * @param file The path to the file where incoming stream is to be recorded.   <br/>
    */
    public void setRecordFile(@Nullable String file);

    /**
    * Get the ring back tone played to far end during incoming calls. <br/>
    * <br/>
    * @return the path to the remote ring back tone to be played.   <br/>
    */
    @Nullable
    public String getRemoteRingbackTone();

    /**
    * Specify a ring back tone to be played to far end during incoming calls. <br/>
    * <br/>
    * @param ring The path to the remote ring back tone to be played.   <br/>
    */
    public void setRemoteRingbackTone(@Nullable String ring);

    /**
    * Tells whether NACK context is enabled or not. <br/>
    * <br/>
    * @return A boolean value telling whether NACK context is enabled or not <br/>
    */
    public boolean retransmissionOnNackEnabled();

    /**
    * Turns NACK context on or off. <br/>
    * <br/>
    * @param enable A boolean value telling whether to enable NACK context <br/>
    */
    public void enableRetransmissionOnNack(boolean enable);

    /**
    * Returns the path to the wav file used for ringing. <br/>
    * <br/>
    * @return The path to the wav file used for ringing.   <br/>
    */
    @Nullable
    public String getRing();

    /**
    * Sets the path to a wav file used for ringing. <br/>
    * <br/>
    * The file must be a wav 16bit linear. If null, ringing is disable unless<br/>
    * #linphone_core_get_use_native_ringing() is enabled, in which case we use the<br/>
    * device ringtone. <br/>
    * @param path The path to a wav file to be used for ringing, null to disable or<br/>
    * use device ringing depending on #linphone_core_get_use_native_ringing().   <br/>
    */
    public void setRing(@Nullable String path);

    /**
    * Tells whether the ring play is enabled during an incoming early media call. <br/>
    * <br/>
    */
    public boolean getRingDuringIncomingEarlyMedia();

    /**
    * Enable or disable the ring play during an incoming early media call. <br/>
    * <br/>
    * @param enable A boolean value telling whether to enable ringing during an<br/>
    * incoming early media call. <br/>
    */
    public void setRingDuringIncomingEarlyMedia(boolean enable);

    /**
    * Returns the path to the wav file used for ringing back. <br/>
    * <br/>
    * @return The path to the wav file used for ringing back.   <br/>
    */
    @Nullable
    public String getRingback();

    /**
    * Sets the path to a wav file used for ringing back. <br/>
    * <br/>
    * Ringback means the ring that is heard when it's ringing at the remote party.<br/>
    * The file must be a wav 16bit linear. <br/>
    * @param path The path to a wav file to be used for ringing back.   <br/>
    */
    public void setRingback(@Nullable String path);

    /**
    * Gets the name of the currently assigned sound device for ringing. <br/>
    * <br/>
    * @return The name of the currently assigned sound device for ringing.   <br/>
    */
    @Nullable
    public String getRingerDevice();

    /**
    * Sets the sound device used for ringing. <br/>
    * <br/>
    * @param devid The device name as returned by linphone_core_get_sound_devices   <br/>
    * @return 0 <br/>
    */
    public int setRingerDevice(@Nullable String devid);

    /**
    * Gets the path to a file or folder containing the trusted root CAs (PEM format) <br/>
    * <br/>
    * @return The path to a file or folder containing the trusted root CAs.   <br/>
    */
    @Nullable
    public String getRootCa();

    /**
    * Sets the path to a file or folder containing trusted root CAs (PEM format) <br/>
    * <br/>
    * @param path The path to a file or folder containing trusted root CAs.   <br/>
    */
    public void setRootCa(@Nullable String path);

    /**
    * Sets the trusted root CAs (PEM format) <br/>
    * <br/>
    * @param data The trusted root CAs as a string   <br/>
    */
    public void setRootCaData(@Nullable String data);

    /**
    * Returns whether RTP bundle mode (also known as Media Multiplexing) is enabled. <br/>
    * <br/>
    * See https://tools.ietf.org/html/draft-ietf-mmusic-sdp-bundle-negotiation-54 for<br/>
    * more information. <br/>
    * @return a boolean indicating the enablement of rtp bundle mode. <br/>
    */
    public boolean rtpBundleEnabled();

    /**
    * Enables or disables RTP bundle mode (Media Multiplexing). <br/>
    * <br/>
    * See https://tools.ietf.org/html/draft-ietf-mmusic-sdp-bundle-negotiation-54 for<br/>
    * more information about the feature. When enabled, liblinphone will try to<br/>
    * negociate the use of a single port for all streams when doing an outgoing call.<br/>
    * It automatically enables rtcp-mux. This feature can also be enabled per-call<br/>
    * using {@link CallParams}. <br/>
    * @param value a boolean to indicate whether the feature is to be enabled. <br/>
    */
    public void enableRtpBundle(boolean value);

    /**
    * Media offer control param for SIP INVITE. <br/>
    * <br/>
    * @return true if INVITE has to be sent whitout SDP. <br/>
    */
    public boolean sdp200AckEnabled();

    /**
    * Control when media offer is sent in SIP INVITE. <br/>
    * <br/>
    * @param enable true if INVITE has to be sent whitout SDP. <br/>
    */
    public void enableSdp200Ack(boolean enable);

    /**
    * Tells whether video self view during call is enabled or not. <br/>
    * <br/>
    * @return A boolean value telling whether self view is enabled <br/>
    * see: {@link #enableSelfView} for details. <br/>
    */
    public boolean selfViewEnabled();

    /**
    * Enables or disable self view during calls. <br/>
    * <br/>
    * @param enable A boolean value telling whether to enable self view Self-view<br/>
    * refers to having local webcam image inserted in corner of the video window<br/>
    * during calls. This function works at any time, including during calls. <br/>
    */
    public void enableSelfView(boolean enable);

    /**
    * Enable whether or not to hide sender name in forward message. <br/>
    * <br/>
    * @param enable whether or not to enable the feature <br/>
    */
    public void enableSenderNameHiddenInForwardMessage(boolean enable);

    /**
    * Check if the Session Timers feature is enabled. <br/>
    * <br/>
    */
    public boolean getSessionExpiresEnabled();

    /**
    * Enable the Session Timers support. <br/>
    * <br/>
    * @param enabled Enable or disable it <br/>
    */
    public void setSessionExpiresEnabled(boolean enabled);

    /**
    * Returns the session expires min value, 90 by default. <br/>
    * <br/>
    */
    public int getSessionExpiresMinValue();

    /**
    * Sets the session expires minSE value, forced to a minimum of 90 by default. <br/>
    * <br/>
    */
    public void setSessionExpiresMinValue(int min);

    /**
    * Returns the session expires refresher value. <br/>
    * <br/>
    */
    public SessionExpiresRefresher getSessionExpiresRefresherValue();

    /**
    * Sets the session expires refresher value. <br/>
    * <br/>
    * @param refresher The {@link SessionExpiresRefresher} configuration value <br/>
    */
    public void setSessionExpiresRefresherValue(SessionExpiresRefresher refresher);

    /**
    * Returns the session expires value. <br/>
    * <br/>
    */
    public int getSessionExpiresValue();

    /**
    * Sets the session expires value, 0 by default. <br/>
    * <br/>
    */
    public void setSessionExpiresValue(int expires);

    /**
    * Get the DSCP field for SIP signaling channel. <br/>
    * <br/>
    * The DSCP defines the quality of service in IP packets. <br/>
    * @return The current DSCP value <br/>
    */
    public int getSipDscp();

    /**
    * Set the DSCP field for SIP signaling channel. <br/>
    * <br/>
    * The DSCP defines the quality of service in IP packets. <br/>
    * @param dscp The DSCP value to set <br/>
    */
    public void setSipDscp(int dscp);

    /**
    * This method is called by the application to notify the linphone core library<br/>
    * when the SIP network is reachable. <br/>
    * <br/>
    * This is for advanced usage, when SIP and RTP layers are required to use<br/>
    * different interfaces. Most applications just need {@link #setNetworkReachable}.<br/>
    * @param reachable true if network is reachable, false otherwise <br/>
    */
    public void setSipNetworkReachable(boolean reachable);

    /**
    * Get the SIP transport timeout. <br/>
    * <br/>
    * @return The SIP transport timeout in milliseconds. <br/>
    */
    public int getSipTransportTimeout();

    /**
    * Set the SIP transport timeout. <br/>
    * <br/>
    * @param timeoutMs The SIP transport timeout in milliseconds. <br/>
    */
    public void setSipTransportTimeout(int timeoutMs);

    /**
    * Gets the list of the available sound devices. <br/>
    * <br/>
    * @return An unmodifiable array of strings contanining the names of the available<br/>
    * sound devices that is null terminated.      <br/>
    * @deprecated 10/04/2021 Use {@link #getAudioDevices} instead.<br/>
    */
    @Deprecated
    @NonNull
    public String[] getSoundDevicesList();

    /**
    * Get the path to the image file streamed when "Static picture" is set as the<br/>
    * video device. <br/>
    * <br/>
    * @return The path to the image file streamed when "Static picture" is set as the<br/>
    * video device.   <br/>
    */
    @Nullable
    public String getStaticPicture();

    /**
    * Set the path to the image file to stream when "Static picture" is set as the<br/>
    * video device. <br/>
    * <br/>
    * @param path The path to the image file to use.   <br/>
    */
    public int setStaticPicture(@Nullable String path);

    /**
    * Get the frame rate for static picture. <br/>
    * <br/>
    * @return The frame rate used for static picture. <br/>
    */
    public float getStaticPictureFps();

    /**
    * Set the frame rate for static picture. <br/>
    * <br/>
    * @param fps The new frame rate to use for static picture. <br/>
    */
    public int setStaticPictureFps(float fps);

    /**
    * Get the STUN server address being used. <br/>
    * <br/>
    * @return The STUN server address being used.   <br/>
    */
    @Nullable
    public String getStunServer();

    /**
    * Set the STUN server address to use when the firewall policy is set to STUN. <br/>
    * <br/>
    * @param server The STUN server address to use.   <br/>
    */
    public void setStunServer(@Nullable String server);

    /**
    * Returns a null terminated table of strings containing the file format extension<br/>
    * supported for call recording. <br/>
    * <br/>
    * @return The supported formats, typically 'wav' and 'mkv'.      <br/>
    */
    @NonNull
    public String[] getSupportedFileFormatsList();

    /**
    * Set the supported tags. <br/>
    * <br/>
    */
    public void setSupportedTag(String tags);

    /**
    * Return the list of the available text payload types. <br/>
    * <br/>
    * @return A freshly allocated list of the available payload types. The list must<br/>
    * be destroyed with bctbx_list_free() after usage. The elements of the list<br/>
    * haven't to be unref.      <br/>
    */
    @NonNull
    public PayloadType[] getTextPayloadTypes();

    /**
    * Redefine the list of the available payload types. <br/>
    * <br/>
    * @param payloadTypes The new list of payload types. The core does not take<br/>
    * ownership on it.    <br/>
    */
    public void setTextPayloadTypes(@Nullable PayloadType[] payloadTypes);

    /**
    * Gets the UDP port used for text streaming. <br/>
    * <br/>
    * @return The UDP port used for text streaming <br/>
    */
    public int getTextPort();

    /**
    * Sets the UDP port used for text streaming. <br/>
    * <br/>
    * A value if -1 will request the system to allocate the local port randomly. This<br/>
    * is recommended in order to avoid firewall warnings. <br/>
    * @param port The UDP port to use for text streaming <br/>
    */
    public void setTextPort(int port);

    /**
    * Get the text port range from which is randomly chosen the UDP port used for<br/>
    * text streaming. <br/>
    * <br/>
    * @return a {@link Range} object     <br/>
    */
    @NonNull
    public Range getTextPortsRange();

    /**
    * Gets the TLS certificate. <br/>
    * <br/>
    * @return the TLS certificate or null if not set yet.   <br/>
    */
    @Nullable
    public String getTlsCert();

    /**
    * Sets a TLS certificate used for TLS authentication The certificate won't be<br/>
    * stored, you have to set it after each {@link Core} startup. <br/>
    * <br/>
    * @param tlsCert the TLS certificate.   <br/>
    */
    public void setTlsCert(@Nullable String tlsCert);

    /**
    * Gets the path to the TLS certificate file. <br/>
    * <br/>
    * @return the TLS certificate path or null if not set yet.   <br/>
    */
    @Nullable
    public String getTlsCertPath();

    /**
    * Sets a TLS certificate path used for TLS authentication The path will be stored<br/>
    * in the rc file and automatically restored on startup. <br/>
    * <br/>
    * @param tlsCertPath path to the TLS certificate.   <br/>
    */
    public void setTlsCertPath(@Nullable String tlsCertPath);

    /**
    * Gets the TLS key. <br/>
    * <br/>
    * @return the TLS key or null if not set yet.   <br/>
    */
    @Nullable
    public String getTlsKey();

    /**
    * Sets a TLS key used for TLS authentication The key won't be stored, you have to<br/>
    * set it after each {@link Core} startup. <br/>
    * <br/>
    * @param tlsKey the TLS key.   <br/>
    */
    public void setTlsKey(@Nullable String tlsKey);

    /**
    * Gets the path to the TLS key file. <br/>
    * <br/>
    * @return the TLS key path or null if not set yet.   <br/>
    */
    @Nullable
    public String getTlsKeyPath();

    /**
    * Sets a TLS key path used for TLS authentication The path will be stored in the<br/>
    * rc file and automatically restored on startup. <br/>
    * <br/>
    * @param tlsKeyPath path to the TLS key.   <br/>
    */
    public void setTlsKeyPath(@Nullable String tlsKeyPath);

    /**
    * Retrieves the port configuration used for each transport (udp, tcp, tls). <br/>
    * <br/>
    * A zero value port for a given transport means the transport is not used. A<br/>
    * value of LC_SIP_TRANSPORT_RANDOM (-1) means the port is to be chosen randomly<br/>
    * by the system. <br/>
    * @return A {@link Transports} structure with the configured ports     <br/>
    */
    @NonNull
    public Transports getTransports();

    /**
    * Sets the ports to be used for each of transport (UDP or TCP) A zero value port<br/>
    * for a given transport means the transport is not used. <br/>
    * <br/>
    * A value of LC_SIP_TRANSPORT_RANDOM (-1) means the port is to be choosen<br/>
    * randomly by the system. <br/>
    * @param transports A #LinphoneSipTransports structure giving the ports to use   <br/>
    * @return 0 <br/>
    */
    public int setTransports(@NonNull Transports transports);

    /**
    * Retrieves the real port number assigned for each sip transport (udp, tcp, tls). <br/>
    * <br/>
    * A zero value means that the transport is not activated. If<br/>
    * LC_SIP_TRANSPORT_RANDOM was passed to linphone_core_set_sip_transports, the<br/>
    * random port choosed by the system is returned. <br/>
    * @return A {@link Transports} structure with the ports being used     <br/>
    */
    @NonNull
    public Transports getTransportsUsed();

    /**
    * get tunnel instance if available <br/>
    * <br/>
    * @return {@link Tunnel} or null if not available.   <br/>
    */
    @Nullable
    public Tunnel getTunnel();

    /**
    * Return the global unread chat message count. <br/>
    * <br/>
    * @return The global unread chat message count. <br/>
    */
    public int getUnreadChatMessageCount();

    /**
    * Return the unread chat message count for all active local address. <br/>
    * <br/>
    * (Primary contact + proxy configs.) <br/>
    * @return The unread chat message count. <br/>
    */
    public int getUnreadChatMessageCountFromActiveLocals();

    /**
    * Retrieve the maximum available upload bandwidth. <br/>
    * <br/>
    * This value was set by {@link #setUploadBandwidth}. <br/>
    * @return the upload bandiwdth in kbits/s, 0 for infinite <br/>
    */
    public int getUploadBandwidth();

    /**
    * Sets maximum available upload bandwidth This is IP bandwidth, in kbit/s. <br/>
    * <br/>
    * This information is used by liblinphone together with remote side available<br/>
    * bandwidth signaled in SDP messages to properly configure audio & video codec's<br/>
    * output bitrate.<br/>
    * @param bandwidth the bandwidth in kbits/s, 0 for infinite <br/>
    */
    public void setUploadBandwidth(int bandwidth);

    /**
    * Set audio packetization time linphone will send (in absence of requirement from<br/>
    * peer) A value of 0 stands for the current codec default packetization time. <br/>
    * <br/>
    * @return the upload packetization time set <br/>
    */
    public int getUploadPtime();

    /**
    * Set audio packetization time linphone will send (in absence of requirement from<br/>
    * peer) A value of 0 stands for the current codec default packetization time. <br/>
    * <br/>
    * @param ptime the upload packetization time to set <br/>
    */
    public void setUploadPtime(int ptime);

    /**
    * Return the external ip address of router. <br/>
    * <br/>
    * In some cases the uPnP can have an external ip address but not a usable uPnP<br/>
    * (state different of Ok).<br/>
    * @return a null terminated string containing the external ip address. If the the<br/>
    * external ip address is not available return null.   <br/>
    */
    @Nullable
    public String getUpnpExternalIpaddress();

    /**
    * Return the internal state of uPnP. <br/>
    * <br/>
    * @return an LinphoneUpnpState. <br/>
    */
    public UpnpState getUpnpState();

    /**
    * Gets whether linphone is currently streaming audio from and to files, rather<br/>
    * than using the soundcard. <br/>
    * <br/>
    * @return A boolean value representing whether linphone is streaming audio from<br/>
    * and to files or not. <br/>
    */
    public boolean getUseFiles();

    /**
    * Ask the core to stream audio from and to files, instead of using the soundcard. <br/>
    * <br/>
    * @param yesno A boolean value asking to stream audio from and to files or not. <br/>
    */
    public void setUseFiles(boolean yesno);

    /**
    * Indicates whether SIP INFO is used to send digits. <br/>
    * <br/>
    * @return A boolean value telling whether SIP INFO is used to send digits <br/>
    */
    public boolean getUseInfoForDtmf();

    /**
    * Sets whether SIP INFO is to be used to send digits. <br/>
    * <br/>
    * @param useInfo A boolean value telling whether to use SIP INFO to send digits <br/>
    */
    public void setUseInfoForDtmf(boolean useInfo);

    /**
    * Indicates whether RFC2833 is used to send digits. <br/>
    * <br/>
    * @return A boolean value telling whether RFC2833 is used to send digits <br/>
    */
    public boolean getUseRfc2833ForDtmf();

    /**
    * Sets whether RFC2833 is to be used to send digits. <br/>
    * <br/>
    * @param useRfc2833 A boolean value telling whether to use RFC2833 to send digits <br/>
    */
    public void setUseRfc2833ForDtmf(boolean useRfc2833);

    /**
    * Gets the user-agent as a string. <br/>
    * <br/>
    * @return liblinphone's user agent as a string.   <br/>
    */
    @NonNull
    public String getUserAgent();

    /**
    * Get the path to the directory storing the user's certificates. <br/>
    * <br/>
    * @return The path to the directory storing the user's certificates.   <br/>
    */
    @Nullable
    public String getUserCertificatesPath();

    /**
    * Set the path to the directory storing the user's x509 certificates (used by<br/>
    * dtls) <br/>
    * <br/>
    * @param path The path to the directory to use to store the user's certificates. <br/>
    *  <br/>
    */
    public void setUserCertificatesPath(@Nullable String path);

    /**
    * Enable vibration will incoming call is ringing (Android only). <br/>
    * <br/>
    * @return true if the device should vibrate while an incoming call is ringing,<br/>
    * false otherwise <br/>
    */
    public void setVibrationOnIncomingCallEnabled(boolean enable);

    /**
    * Get the default policy for video. <br/>
    * <br/>
    * See {@link #setVideoActivationPolicy} for more details. <br/>
    * @return The video policy being used     <br/>
    */
    @NonNull
    public VideoActivationPolicy getVideoActivationPolicy();

    /**
    * Sets the default policy for video. <br/>
    * <br/>
    * This policy defines whether:<br/>
    * @param policy The {@link VideoActivationPolicy} to use   <br/>
    */
    public void setVideoActivationPolicy(@NonNull VideoActivationPolicy policy);

    /**
    * Tells whether the video adaptive jitter compensation is enabled. <br/>
    * <br/>
    * @return true if the video adaptive jitter compensation is enabled, false<br/>
    * otherwise. <br/>
    */
    public boolean videoAdaptiveJittcompEnabled();

    /**
    * Enable or disable the video adaptive jitter compensation. <br/>
    * <br/>
    * @param enable true to enable the video adaptive jitter compensation, false to<br/>
    * disable it. <br/>
    */
    public void enableVideoAdaptiveJittcomp(boolean enable);

    /**
    * Tells whether video capture is enabled. <br/>
    * <br/>
    * @return true if video capture is enabled, false if disabled. <br/>
    */
    public boolean videoCaptureEnabled();

    /**
    * Enable or disable video capture. <br/>
    * <br/>
    * This function does not have any effect during calls. It just indicates the<br/>
    * {@link Core} to initiate future calls with video capture or not. <br/>
    * @param enable true to enable video capture, false to disable it. <br/>
    */
    public void enableVideoCapture(boolean enable);

    /**
    * Returns the name of the currently active video device. <br/>
    * <br/>
    * @return The name of the currently active video device.   <br/>
    */
    @Nullable
    public String getVideoDevice();

    /**
    * Sets the active video device. <br/>
    * <br/>
    * @param id The name of the video device to use as returned by<br/>
    * linphone_core_get_video_devices   <br/>
    */
    public int setVideoDevice(@Nullable String id);

    /**
    * Gets the list of the available video capture devices. <br/>
    * <br/>
    * @return An unmodifiable array of strings contanining the names of the available<br/>
    * video capture devices that is null terminated.      <br/>
    */
    @NonNull
    public String[] getVideoDevicesList();

    /**
    * Tells whether video display is enabled. <br/>
    * <br/>
    * @return true if video display is enabled, false if disabled. <br/>
    */
    public boolean videoDisplayEnabled();

    /**
    * Enable or disable video display. <br/>
    * <br/>
    * This function does not have any effect during calls. It just indicates the<br/>
    * {@link Core} to initiate future calls with video display or not. <br/>
    * @param enable true to enable video display, false to disable it. <br/>
    */
    public void enableVideoDisplay(boolean enable);

    /**
    * Get the name of the mediastreamer2 filter used for rendering video. <br/>
    * <br/>
    * @return The currently selected video display filter.   <br/>
    */
    @Nullable
    public String getVideoDisplayFilter();

    /**
    * Set the name of the mediastreamer2 filter to be used for rendering video. <br/>
    * <br/>
    * This is for advanced users of the library, mainly to workaround hardware/driver<br/>
    * bugs. <br/>
    * @param filterName the filter name to use or null to use default.   <br/>
    */
    public void setVideoDisplayFilter(@Nullable String filterName);

    /**
    * Get the DSCP field for outgoing video streams. <br/>
    * <br/>
    * The DSCP defines the quality of service in IP packets. <br/>
    * @return The current DSCP value <br/>
    */
    public int getVideoDscp();

    /**
    * Set the DSCP field for outgoing video streams. <br/>
    * <br/>
    * The DSCP defines the quality of service in IP packets. <br/>
    * @param dscp The DSCP value to set <br/>
    */
    public void setVideoDscp(int dscp);

    /**
    * Returns true if either capture or display is enabled, false otherwise. <br/>
    * <br/>
    * same as ( {@link #videoCaptureEnabled} | {@link #videoDisplayEnabled} )<br/>
    * @return true if either capture or display is enabled, false otherwise. <br/>
    */
    public boolean videoEnabled();

    /**
    * Returns the nominal video jitter buffer size in milliseconds. <br/>
    * <br/>
    * @return The nominal video jitter buffer size in milliseconds <br/>
    */
    public int getVideoJittcomp();

    /**
    * Sets the nominal video jitter buffer size in milliseconds. <br/>
    * <br/>
    * The value takes effect immediately for all running and pending calls, if any. A<br/>
    * value of 0 disables the jitter buffer.<br/>
    * @param milliseconds the jitter buffer size in milliseconds <br/>
    */
    public void setVideoJittcomp(int milliseconds);

    /**
    * Use to get multicast address to be used for video stream. <br/>
    * <br/>
    * @return an ipv4/6 multicast address, or default value.   <br/>
    */
    @Nullable
    public String getVideoMulticastAddr();

    /**
    * Use to set multicast address to be used for video stream. <br/>
    * <br/>
    * @param ip an ipv4/6 multicast address.   <br/>
    * @return 0 in case of success <br/>
    */
    public int setVideoMulticastAddr(@Nullable String ip);

    /**
    * Use to get multicast state of video stream. <br/>
    * <br/>
    * @return true if subsequent calls will propose multicast ip set by {@link #setVideoMulticastAddr}<br/>
    */
    public boolean videoMulticastEnabled();

    /**
    * Use to enable multicast rtp for video stream. <br/>
    * <br/>
    * If enabled, outgoing calls put a multicast address from {@link #getVideoMulticastAddr}<br/>
    * into video cline. In case of outgoing call video stream is sent to this<br/>
    * multicast address. For incoming calls behavior is unchanged. <br/>
    * @param yesno if yes, subsequent outgoing calls will propose multicast ip set by<br/>
    * {@link #setVideoMulticastAddr} <br/>
    */
    public void enableVideoMulticast(boolean yesno);

    /**
    * Use to get multicast ttl to be used for video stream. <br/>
    * <br/>
    * @return a time to leave value <br/>
    */
    public int getVideoMulticastTtl();

    /**
    * Use to set multicast ttl to be used for video stream. <br/>
    * <br/>
    * @param ttl value or -1 if not used. [0..255] default value is 1 <br/>
    * @return 0 in case of success <br/>
    */
    public int setVideoMulticastTtl(int ttl);

    /**
    * Return the list of the available video payload types. <br/>
    * <br/>
    * @return A freshly allocated list of the available payload types. The list must<br/>
    * be destroyed with bctbx_list_free() after usage. The elements of the list<br/>
    * haven't to be unref.      <br/>
    */
    @NonNull
    public PayloadType[] getVideoPayloadTypes();

    /**
    * Redefine the list of the available video payload types. <br/>
    * <br/>
    * @param payloadTypes The new list of codecs. The core does not take ownership on<br/>
    * it.    <br/>
    */
    public void setVideoPayloadTypes(@Nullable PayloadType[] payloadTypes);

    /**
    * Gets the UDP port used for video streaming. <br/>
    * <br/>
    * @return The UDP port used for video streaming <br/>
    */
    public int getVideoPort();

    /**
    * Sets the UDP port used for video streaming. <br/>
    * <br/>
    * A value of -1 will request the system to allocate the local port randomly. This<br/>
    * is recommended in order to avoid firewall warnings. <br/>
    * @param port The UDP port to use for video streaming <br/>
    */
    public void setVideoPort(int port);

    /**
    * Get the video port range from which is randomly chosen the UDP port used for<br/>
    * video streaming. <br/>
    * <br/>
    * @return a {@link Range} object     <br/>
    */
    @NonNull
    public Range getVideoPortsRange();

    /**
    * Get the video preset used for video calls. <br/>
    * <br/>
    * @return The name of the video preset used for video calls (can be null if the<br/>
    * default video preset is used).   <br/>
    */
    @Nullable
    public String getVideoPreset();

    /**
    * Set the video preset to be used for video calls. <br/>
    * <br/>
    * @param preset The name of the video preset to be used (can be null to use the<br/>
    * default video preset).   <br/>
    */
    public void setVideoPreset(@Nullable String preset);

    /**
    * Tells whether video preview is enabled. <br/>
    * <br/>
    * @return A boolean value telling whether video preview is enabled <br/>
    */
    public boolean videoPreviewEnabled();

    /**
    * Controls video preview enablement. <br/>
    * <br/>
    * @param enable A boolean value telling whether the video preview is to be shown<br/>
    * Video preview refers to the action of displaying the local webcam image to the<br/>
    * user while not in call. <br/>
    */
    public void enableVideoPreview(boolean enable);

    /**
    * Enable or disable video source reuse when switching from preview to actual<br/>
    * video call. <br/>
    * <br/>
    * This source reuse is useful when you always display the preview, even before<br/>
    * calls are initiated. By keeping the video source for the transition to a real<br/>
    * video call, you will smooth out the source close/reopen cycle.<br/>
    * This function does not have any effect durfing calls. It just indicates the<br/>
    * {@link Core} to initiate future calls with video source reuse or not. Also, at<br/>
    * the end of a video call, the source will be closed whatsoever for now. <br/>
    * @param enable true to enable video source reuse. false to disable it for<br/>
    * subsequent calls. <br/>
    */
    public void enableVideoSourceReuse(boolean enable);

    /**
    * Tells whether Wifi only mode is enabled or not. <br/>
    * <br/>
    * warning: Only works for Android platform. <br/>
    * @return A boolean value telling whether Wifi only mode is enabled or not <br/>
    */
    public boolean wifiOnlyEnabled();

    /**
    * Turns Wifi only mode on or off. <br/>
    * <br/>
    * If enabled, app won't register when active network isn't WiFi or Ethernet.<br/>
    * warning: Only works for Android platform. <br/>
    * @param enable A boolean value telling whether to enable IPv6 support <br/>
    */
    public void enableWifiOnly(boolean enable);

    /**
    * Get the path to the file storing the zrtp secrets cache. <br/>
    * <br/>
    * @return The path to the file storing the zrtp secrets cache.   <br/>
    */
    @Nullable
    public String getZrtpSecretsFile();

    /**
    * Set the path to the file storing the zrtp secrets cache. <br/>
    * <br/>
    * @param file The path to the file to use to store the zrtp secrets cache.   <br/>
    */
    public void setZrtpSecretsFile(@Nullable String file);

    /**
    * Special function to indicate if the audio session is activated. <br/>
    * <br/>
    * Must be called when ProviderDelegate of the callkit notifies that the audio<br/>
    * session is activated or deactivated. <br/>
    */
    public void activateAudioSession(boolean actived);

    /**
    * Add all current calls into the conference. <br/>
    * <br/>
    * If no conference is running a new internal conference context is created and<br/>
    * all current calls are added to it. <br/>
    * @return 0 if succeeded. Negative number if failed <br/>
    */
    public int addAllToConference();

    /**
    * Adds authentication information to the {@link Core}. <br/>
    * <br/>
    * That piece of information will be used during all SIP transactions that require<br/>
    * authentication. <br/>
    * @param info The {@link AuthInfo} to add.   <br/>
    */
    public void addAuthInfo(@NonNull AuthInfo info);

    /**
    * Add support for the specified content type. <br/>
    * <br/>
    * It is the application responsibility to handle it correctly afterwards. <br/>
    * @param contentType The content type to add support for   <br/>
    */
    public void addContentTypeSupport(@NonNull String contentType);

    /**
    * Add a friend list. <br/>
    * <br/>
    * @param list {@link FriendList} object   <br/>
    */
    public void addFriendList(@NonNull FriendList list);

    /**
    * Add the given linphone specs to the list of functionalities the linphone client<br/>
    * supports. <br/>
    * <br/>
    * @param spec The spec to add   <br/>
    */
    public void addLinphoneSpec(@NonNull String spec);

    /**
    * Add a proxy configuration. <br/>
    * <br/>
    * This will start registration on the proxy, if registration is enabled. <br/>
    * @param config the {@link ProxyConfig} to add   <br/>
    * @return 0 if successful, -1 otherwise <br/>
    */
    public int addProxyConfig(@NonNull ProxyConfig config);

    /**
    * This function controls signaling features supported by the core. <br/>
    * <br/>
    * They are typically included in a SIP Supported header. <br/>
    * @param tag The feature tag name   <br/>
    */
    public void addSupportedTag(@NonNull String tag);

    /**
    * Add a participant to the conference. <br/>
    * <br/>
    * If no conference is going on a new internal conference context is created and<br/>
    * the participant is added to it. <br/>
    * @param call The current call with the participant to add   <br/>
    * @return 0 if succeeded. Negative number if failed <br/>
    */
    public int addToConference(@NonNull Call call);

    /**
    * Special function to indicate if the audio route is changed. <br/>
    * <br/>
    * Must be called in the callback of AVAudioSessionRouteChangeNotification. <br/>
    * @deprecated 07/01/2020 now handled in the linphone SDK directly <br/>
    */
    @Deprecated
    public void audioRouteChanged();

    /**
    * Checks if a new version of the application is available. <br/>
    * <br/>
    * @param currentVersion The current version of the application   <br/>
    */
    public void checkForUpdate(@NonNull String currentVersion);

    /**
    * Clear all authentication information. <br/>
    * <br/>
    */
    public void clearAllAuthInfo();

    /**
    * Erase the call log. <br/>
    * <br/>
    */
    public void clearCallLogs();

    /**
    * Erase all proxies from config. <br/>
    * <br/>
    */
    public void clearProxyConfig();

    /**
    * Special function to configure audio session with default settings. <br/>
    * <br/>
    * Must be called in ProviderDelegate's callbacks when answer an incoming call and<br/>
    * start an outgoing call. <br/>
    */
    public void configureAudioSession();

    /**
    * Create a {@link AccountCreator} and set Linphone Request callbacks. <br/>
    * <br/>
    * @param xmlrpcUrl The URL to the XML-RPC server.   <br/>
    * @return The new {@link AccountCreator} object.   <br/>
    */
    @NonNull
    public AccountCreator createAccountCreator(@Nullable String xmlrpcUrl);

    /**
    * Create a {@link Address} object by parsing the user supplied address, given as<br/>
    * a string. <br/>
    * <br/>
    * @param address String containing the user supplied address   <br/>
    * @return The create {@link Address} object   <br/>
    */
    @Nullable
    public Address createAddress(@Nullable String address);

    /**
    * Creates a fake {@link CallLog}. <br/>
    * <br/>
    * @param from {@link Address} of caller   <br/>
    * @param to {@link Address} of callee   <br/>
    * @param dir {@link Call#Dir} of call <br/>
    * @param duration call length in seconds <br/>
    * @param startTime timestamp of call start time <br/>
    * @param connectedTime timestamp of call connection <br/>
    * @param status {@link Call#Status} of call <br/>
    * @param videoEnabled whether video was enabled or not for this call <br/>
    * @param quality call quality <br/>
    * @return a {@link CallLog} object   <br/>
    */
    @NonNull
    public CallLog createCallLog(@NonNull Address from, @NonNull Address to, Call.Dir dir, int duration, long startTime, long connectedTime, Call.Status status, boolean videoEnabled, float quality);

    /**
    * Create a {@link CallParams} suitable for {@link #inviteWithParams},<br/>
    * linphone_core_accept_call_with_params,<br/>
    * linphone_core_accept_early_media_with_params or<br/>
    * linphone_core_accept_call_update. <br/>
    * <br/>
    * The parameters are initialized according to the current {@link Core}<br/>
    * configuration and the last used local {@link CallParams}, the ones passed<br/>
    * through {@link Call#update}, {@link Call#acceptWithParams} or<br/>
    * linphone_call_accept_update_with_params(). <br/>
    * @param call {@link Call} for which the parameters are to be build, or null in<br/>
    * the case where the parameters are to be used for a new outgoing call.   <br/>
    * @return A new {@link CallParams} object.   <br/>
    */
    @Nullable
    public CallParams createCallParams(@Nullable Call call);

    /**
    * Create a chat room. <br/>
    * <br/>
    * @param params The chat room creation parameters {@link ChatRoomParams}   <br/>
    * @param localAddr {@link Address} representing the local proxy configuration to<br/>
    * use for the chat room creation   <br/>
    * @param subject The subject of the group chat room   <br/>
    * @param participants The initial list of participants of the chat room    <br/>
    * @return The newly created chat room.   <br/>
    * @deprecated 02/07/2020, use {@link #createChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom createChatRoom(@NonNull ChatRoomParams params, @NonNull Address localAddr, @NonNull String subject, @NonNull Address[] participants);

    /**
    * Create a chat room. <br/>
    * <br/>
    * @param params The chat room creation parameters {@link ChatRoomParams}   <br/>
    * @param participants The initial list of participants of the chat room.    <br/>
    * @return The newly created chat room.   <br/>
    * @deprecated 02/07/2020, use {@link #createChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom createChatRoom(@NonNull ChatRoomParams params, String subject, @NonNull Address[] participants);

    /**
    * <br/>
    * @param params The chat room creation parameters {@link ChatRoomParams}   <br/>
    * @param localAddr {@link Address} representing the local proxy configuration to<br/>
    * use for the chat room creation   <br/>
    * @param participant {@link Address} representing the initial participant to add<br/>
    * to the chat room   <br/>
    * @return The newly created chat room.   <br/>
    * @deprecated 02/07/2020, use {@link #createChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom createChatRoom(@NonNull ChatRoomParams params, @NonNull Address localAddr, @NonNull Address participant);

    /**
    * <br/>
    * @param participant {@link Address} representing the initial participant to add<br/>
    * to the chat room   <br/>
    * @return The newly created chat room.   <br/>
    * @deprecated 02/07/2020, use {@link #createChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom createChatRoom(@NonNull Address participant);

    /**
    * <br/>
    * @param subject The subject of the group chat room   <br/>
    * @param participants The initial list of participants of the chat room.    <br/>
    * @return The newly created chat room.   <br/>
    * @deprecated 02/07/2020, use {@link #createChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom createChatRoom(@NonNull String subject, @NonNull Address[] participants);

    /**
    * Create a chat room. <br/>
    * <br/>
    * @param params The chat room creation parameters {@link ChatRoomParams}   <br/>
    * @param localAddr {@link Address} representing the local proxy configuration to<br/>
    * use for the chat room creation or null   <br/>
    * @param participants The initial list of participants of the chat room.    <br/>
    * @return The newly created chat room (can be an existing one if backend is<br/>
    * Basic) or null.   <br/>
    */
    @Nullable
    public ChatRoom createChatRoom(@Nullable ChatRoomParams params, @Nullable Address localAddr, @NonNull Address[] participants);

    /**
    * Create a client-side group chat room. <br/>
    * <br/>
    * When calling this function the chat room is only created at the client-side and<br/>
    * is empty. You need to call {@link ChatRoom#addParticipants} to create at the<br/>
    * server side and add participants to it. Also, the created chat room will not be<br/>
    * a one-to-one chat room even if {@link ChatRoom#addParticipants} is called with<br/>
    * only one participant.<br/>
    * @param subject The subject of the group chat room   <br/>
    * @param fallback Boolean value telling whether we should plan on being able to<br/>
    * fallback to a basic chat room if the client-side group chat room creation fails <br/>
    * @return The newly created client-side group chat room.   <br/>
    * @deprecated 02/07/2020, use {@link #createChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom createClientGroupChatRoom(@NonNull String subject, boolean fallback);

    /**
    * Create a client-side group chat room. <br/>
    * <br/>
    * When calling this function the chat room is only created at the client-side and<br/>
    * is empty. You need to call {@link ChatRoom#addParticipants} to create at the<br/>
    * server side and add participants to it. Also, the created chat room will not be<br/>
    * a one-to-one chat room even if {@link ChatRoom#addParticipants} is called with<br/>
    * only one participant.<br/>
    * @param subject The subject of the group chat room   <br/>
    * @param fallback Boolean value telling whether we should plan on being able to<br/>
    * fallback to a basic chat room if the client-side group chat room creation fails <br/>
    * @param encrypted Boolean value telling whether we should apply encryption or<br/>
    * not on chat messages sent and received on this room. <br/>
    * @return The newly created client-side group chat room.   <br/>
    * @deprecated 02/07/2020, use {@link #createChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom createClientGroupChatRoom(@NonNull String subject, boolean fallback, boolean encrypted);

    /**
    * Create some default conference parameters for instanciating a a conference with<br/>
    * {@link #createConferenceWithParams}. <br/>
    * <br/>
    * @return conference parameters.   <br/>
    */
    @NonNull
    public ConferenceParams createConferenceParams();

    /**
    * Create a conference. <br/>
    * <br/>
    * @param params Parameters of the conference. See {@link ConferenceParams}.   <br/>
    * @return A pointer on the freshly created conference. That object will be<br/>
    * automatically freed by the core after calling {@link #terminateConference}.   <br/>
    */
    @Nullable
    public Conference createConferenceWithParams(@NonNull ConferenceParams params);

    /**
    * Create a {@link Config} object from a user config file. <br/>
    * <br/>
    * @param filename The filename of the config file to read to fill the<br/>
    * instantiated {@link Config}   <br/>
    * @return a {@link Config} object.   <br/>
    */
    @NonNull
    public Config createConfig(@Nullable String filename);

    /**
    * Create a content with default values from Linphone core. <br/>
    * <br/>
    * @return {@link Content} object with default values set   <br/>
    */
    @NonNull
    public Content createContent();

    /**
    * Creates and returns the default chat room parameters. <br/>
    * <br/>
    * @return A {@link ChatRoomParams} object   <br/>
    */
    @NonNull
    public ChatRoomParams createDefaultChatRoomParams();

    /**
    * Create a default LinphoneFriend. <br/>
    * <br/>
    * @return The created {@link Friend} object   <br/>
    */
    @NonNull
    public Friend createFriend();

    /**
    * Create a new empty {@link FriendList} object. <br/>
    * <br/>
    * @return A new {@link FriendList} object.   <br/>
    */
    @NonNull
    public FriendList createFriendList();

    /**
    * Create a {@link Friend} from the given address. <br/>
    * <br/>
    * @param address A string containing the address to create the {@link Friend}<br/>
    * from   <br/>
    * @return The created {@link Friend} object.   <br/>
    */
    @Nullable
    public Friend createFriendWithAddress(@NonNull String address);

    /**
    * Creates an empty info message. <br/>
    * <br/>
    * @return a new LinphoneInfoMessage.  <br/>
    * The info message can later be filled with information using {@link InfoMessage#addHeader}<br/>
    * or {@link InfoMessage#setContent}, and finally sent with<br/>
    * linphone_core_send_info_message(). <br/>
    */
    @NonNull
    public InfoMessage createInfoMessage();

    /**
    * Create an independent media file player. <br/>
    * <br/>
    * This player support WAVE and MATROSKA formats. <br/>
    * @param soundCardName Playback sound card. If null, the ringer sound card set in<br/>
    * {@link Core} will be used   <br/>
    * @param videoDisplayName Video display. If null, the video display set in {@link Core}<br/>
    * will be used   <br/>
    * @param windowId Id of the drawing window. Depend of video out   <br/>
    * @return A pointer on the new instance. null if failed.   <br/>
    */
    @Nullable
    public Player createLocalPlayer(@Nullable String soundCardName, @Nullable String videoDisplayName, @Nullable Object windowId);

    /**
    * Create a {@link MagicSearch} object. <br/>
    * <br/>
    * @return The created {@link MagicSearch} object   <br/>
    */
    @NonNull
    public MagicSearch createMagicSearch();

    /**
    * Create a new {@link NatPolicy} object with every policies being disabled. <br/>
    * <br/>
    * @return A new {@link NatPolicy} object.   <br/>
    */
    @NonNull
    public NatPolicy createNatPolicy();

    /**
    * Create a new {@link NatPolicy} by reading the config of a {@link Core}<br/>
    * according to the passed ref. <br/>
    * <br/>
    * @param ref The reference of a NAT policy in the config of the {@link Core}   <br/>
    * @return A new {@link NatPolicy} object.   <br/>
    */
    @Nullable
    public NatPolicy createNatPolicyFromConfig(@NonNull String ref);

    /**
    * Create an out-of-dialog notification, specifying the destination resource, the<br/>
    * event name. <br/>
    * <br/>
    * The notification can be send with {@link Event#notify}. <br/>
    * @param resource the destination resource   <br/>
    * @param event the event name   <br/>
    * @return a {@link Event} holding the context of the notification.   <br/>
    */
    @NonNull
    public Event createNotify(@NonNull Address resource, @NonNull String event);

    /**
    * Create a publish context for a one-shot publish. <br/>
    * <br/>
    * After being created, the publish must be sent using {@link Event#sendPublish}.<br/>
    * The {@link Event} is automatically terminated when the publish transaction is<br/>
    * finished, either with success or failure. The application must not call {@link Event#terminate}<br/>
    * for such one-shot publish. <br/>
    * @param resource the resource uri for the event   <br/>
    * @param event the event name   <br/>
    * @return the {@link Event} holding the context of the publish.   <br/>
    */
    @NonNull
    public Event createOneShotPublish(@NonNull Address resource, @NonNull String event);

    /**
    * Create a {@link PresenceActivity} with the given type and description. <br/>
    * <br/>
    * @param acttype The {@link PresenceActivity#Type} to set for the activity. <br/>
    * @param description An additional description of the activity to set for the<br/>
    * activity. Can be null if no additional description is to be added.   <br/>
    * @return The created {@link PresenceActivity} object.   <br/>
    */
    @NonNull
    public PresenceActivity createPresenceActivity(PresenceActivity.Type acttype, @Nullable String description);

    /**
    * Create a default LinphonePresenceModel. <br/>
    * <br/>
    * @return The created {@link PresenceModel} object.   <br/>
    */
    @NonNull
    public PresenceModel createPresenceModel();

    /**
    * Create a {@link PresenceModel} with the given activity type and activity<br/>
    * description. <br/>
    * <br/>
    * @param acttype The {@link PresenceActivity#Type} to set for the activity of the<br/>
    * created model. <br/>
    * @param description An additional description of the activity to set for the<br/>
    * activity. Can be null if no additional description is to be added.   <br/>
    * @return The created {@link PresenceModel} object.   <br/>
    */
    @NonNull
    public PresenceModel createPresenceModelWithActivity(PresenceActivity.Type acttype, @Nullable String description);

    /**
    * Create a {@link PresenceModel} with the given activity type, activity<br/>
    * description, note content and note language. <br/>
    * <br/>
    * @param acttype The {@link PresenceActivity#Type} to set for the activity of the<br/>
    * created model. <br/>
    * @param description An additional description of the activity to set for the<br/>
    * activity. Can be null if no additional description is to be added.   <br/>
    * @param note The content of the note to be added to the created model.   <br/>
    * @param lang The language of the note to be added to the created model.   <br/>
    * @return The created {@link PresenceModel} object.   <br/>
    */
    @NonNull
    public PresenceModel createPresenceModelWithActivityAndNote(PresenceActivity.Type acttype, @Nullable String description, @NonNull String note, @Nullable String lang);

    /**
    * Create a {@link PresenceNote} with the given content and language. <br/>
    * <br/>
    * @param content The content of the note to be created.   <br/>
    * @param lang The language of the note to be created.   <br/>
    * @return The created {@link PresenceNote} object.   <br/>
    */
    @NonNull
    public PresenceNote createPresenceNote(@NonNull String content, @Nullable String lang);

    /**
    * Create a {@link PresencePerson} with the given id. <br/>
    * <br/>
    * @param id The id of the person to be created.   <br/>
    * @return The created {@link PresencePerson} object.   <br/>
    */
    @NonNull
    public PresencePerson createPresencePerson(@NonNull String id);

    /**
    * Create a {@link PresenceService} with the given id, basic status and contact. <br/>
    * <br/>
    * @param id The id of the service to be created.   <br/>
    * @param basicStatus The basic status of the service to be created. <br/>
    * @param contact A string containing a contact information corresponding to the<br/>
    * service to be created.   <br/>
    * @return The created {@link PresenceService} object.   <br/>
    */
    @NonNull
    public PresenceService createPresenceService(@NonNull String id, PresenceBasicStatus basicStatus, @NonNull String contact);

    /**
    * Same as {@link #getPrimaryContact} but the result is a {@link Address} object<br/>
    * instead of const char *. <br/>
    * <br/>
    * @return a {@link Address} object.   <br/>
    */
    @Nullable
    public Address createPrimaryContactParsed();

    /**
    * Create a proxy config with default values from Linphone core. <br/>
    * <br/>
    * @return {@link ProxyConfig} with default values set   <br/>
    */
    @NonNull
    public ProxyConfig createProxyConfig();

    /**
    * Create a publish context for an event state. <br/>
    * <br/>
    * After being created, the publish must be sent using {@link Event#sendPublish}.<br/>
    * After expiry, the publication is refreshed unless it is terminated before. <br/>
    * @param resource the resource uri for the event   <br/>
    * @param event the event name   <br/>
    * @param expires the lifetime of event being published, -1 if no associated<br/>
    * duration, in which case it will not be refreshed. <br/>
    * @return the {@link Event} holding the context of the publish.   <br/>
    */
    @NonNull
    public Event createPublish(@NonNull Address resource, @NonNull String event, int expires);

    /**
    * Create an outgoing subscription, specifying the destination resource, the event<br/>
    * name, and an optional content body. <br/>
    * <br/>
    * If accepted, the subscription runs for a finite period, but is automatically<br/>
    * renewed if not terminated before. Unlike {@link #subscribe} the subscription<br/>
    * isn't sent immediately. It will be send when calling {@link Event#sendSubscribe}<br/>
    * . <br/>
    * @param resource the destination resource   <br/>
    * @param event the event name   <br/>
    * @param expires the whished duration of the subscription <br/>
    * @return a {@link Event} holding the context of the created subcription.   <br/>
    */
    @NonNull
    public Event createSubscribe(@NonNull Address resource, @NonNull String event, int expires);

    /**
    * Create an outgoing subscription, specifying the destination resource, the event<br/>
    * name, and an optional content body. <br/>
    * <br/>
    * If accepted, the subscription runs for a finite period, but is automatically<br/>
    * renewed if not terminated before. Unlike {@link #subscribe} the subscription<br/>
    * isn't sent immediately. It will be send when calling {@link Event#sendSubscribe}<br/>
    * . <br/>
    * @param resource the destination resource   <br/>
    * @param proxy the proxy configuration to use   <br/>
    * @param event the event name   <br/>
    * @param expires the whished duration of the subscription <br/>
    * @return a {@link Event} holding the context of the created subcription.   <br/>
    */
    @NonNull
    public Event createSubscribe(@NonNull Address resource, @NonNull ProxyConfig proxy, @NonNull String event, int expires);

    /**
    * Create a {@link XmlRpcSession} for a given url. <br/>
    * <br/>
    * @param url The URL to the XML-RPC server. Must be NON null.   <br/>
    * @return The new {@link XmlRpcSession} object.   <br/>
    */
    @NonNull
    public XmlRpcSession createXmlRpcSession(@NonNull String url);

    /**
    * Removes a chatroom including all message history from the LinphoneCore. <br/>
    * <br/>
    * @param chatRoom A {@link ChatRoom} object   <br/>
    */
    public void deleteChatRoom(@NonNull ChatRoom chatRoom);

    /**
    * Inconditionnaly disable incoming chat messages. <br/>
    * <br/>
    * @param denyReason the deny reason ({@link Reason#None} has no effect). <br/>
    */
    public void disableChat(Reason denyReason);

    /**
    * Enable reception of incoming chat messages. <br/>
    * <br/>
    * By default it is enabled but it can be disabled with {@link #disableChat}. <br/>
    */
    public void enableChat();

    /**
    * Tells to {@link Core} to use Linphone Instant Messaging encryption. <br/>
    * <br/>
    * @param enable The new lime state<br/>
    * @deprecated 04/02/2019 Use linphone_core_enable_lime_x3dh instead. <br/>
    */
    @Deprecated
    public void enableLime(LimeState enable);

    /**
    * Call this method when you receive a push notification (if you handle push<br/>
    * notifications manually). <br/>
    * <br/>
    * It will ensure the proxy configs are correctly registered to the proxy server,<br/>
    * so the call or the message will be correctly delivered. <br/>
    */
    public void ensureRegistered();

    /**
    * This method is called by the application to notify the linphone core library<br/>
    * when it enters background mode. <br/>
    * <br/>
    */
    public void enterBackground();

    /**
    * Join the local participant to the running conference. <br/>
    * <br/>
    * @return 0 if succeeded. Negative number if failed <br/>
    */
    public int enterConference();

    /**
    * This method is called by the application to notify the linphone core library<br/>
    * when it enters foreground mode. <br/>
    * <br/>
    */
    public void enterForeground();

    /**
    * Returns whether a specific file format is supported. <br/>
    * <br/>
    * see: linphone_core_get_supported_file_formats <br/>
    * @param fmt The format extension (wav, mkv).   <br/>
    * @return true if the file format is supported, false otherwise <br/>
    */
    public boolean fileFormatSupported(@NonNull String fmt);

    /**
    * Find authentication info matching realm, username, domain criteria. <br/>
    * <br/>
    * First of all, (realm,username) pair are searched. If multiple results (which<br/>
    * should not happen because realm are supposed to be unique), then domain is<br/>
    * added to the search. <br/>
    * @param realm the authentication 'realm' (optional)   <br/>
    * @param username the SIP username to be authenticated (mandatory)   <br/>
    * @param sipDomain the SIP domain name (optional)   <br/>
    * @return a {@link AuthInfo} if found.   <br/>
    */
    @Nullable
    public AuthInfo findAuthInfo(@Nullable String realm, @NonNull String username, @Nullable String sipDomain);

    /**
    * Search from the list of current calls if a remote address match uri. <br/>
    * <br/>
    * @param uri which should match call remote uri   <br/>
    * @return {@link Call} or null is no match is found.   <br/>
    * @deprecated 27/10/2020. Use {@link #getCallByRemoteAddress2} instead. <br/>
    */
    @Deprecated
    @Nullable
    public Call findCallFromUri(@NonNull String uri);

    /**
    * Get the call log matching the call id, or null if can't be found. <br/>
    * <br/>
    * @param callId Call id of the call log to find   <br/>
    * @return A call log matching the call id if any.   <br/>
    */
    @Nullable
    public CallLog findCallLogFromCallId(@NonNull String callId);

    /**
    * Find a chat room. <br/>
    * <br/>
    * No reference is transfered to the application. The {@link Core} keeps a<br/>
    * reference on the chat room. <br/>
    * @param peerAddr a linphone address.   <br/>
    * @param localAddr a linphone address.   <br/>
    * @return {@link ChatRoom} where messaging can take place.   <br/>
    * @deprecated 02/07/2020, use {@link #searchChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom findChatRoom(@NonNull Address peerAddr, @NonNull Address localAddr);

    /**
    * Retrieves a list of {@link Address} sort and filter. <br/>
    * <br/>
    * @param filter Chars used for the filter*   <br/>
    * @param sipOnly Only sip address or not <br/>
    * @return A list of filtered {@link Address} + the {@link Address} created with<br/>
    * the filter.    <br/>
    */
    @NonNull
    public Address[] findContactsByChar(@NonNull String filter, boolean sipOnly);

    /**
    * Search a {@link Friend} by its address. <br/>
    * <br/>
    * @param address The {@link Address} to use to search the friend.   <br/>
    * @return The {@link Friend} object corresponding to the given address or null if<br/>
    * not found.   <br/>
    */
    @Nullable
    public Friend findFriend(@NonNull Address address);

    /**
    * Search a {@link Friend} by its phone number. <br/>
    * <br/>
    * @param phoneNumber The phone number to use to search the friend.   <br/>
    * @return The {@link Friend} object corresponding to the given phone number or<br/>
    * null if not found.   <br/>
    */
    @Nullable
    public Friend findFriendByPhoneNumber(@NonNull String phoneNumber);

    /**
    * Search all {@link Friend} matching an address. <br/>
    * <br/>
    * @param address The address to use to search the friends.   <br/>
    * @return A list of {@link Friend} corresponding to the given address.      <br/>
    */
    @NonNull
    public Friend[] findFriends(@NonNull Address address);

    /**
    * Find a one to one chat room. <br/>
    * <br/>
    * No reference is transfered to the application. The {@link Core} keeps a<br/>
    * reference on the chat room. <br/>
    * @param localAddr a linphone address.   <br/>
    * @param participantAddr a linphone address.   <br/>
    * @return {@link ChatRoom} where messaging can take place.   <br/>
    * @deprecated 12/12/2018, use linphone_core_find_one_to_one_chat_room_2 instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom findOneToOneChatRoom(@NonNull Address localAddr, @NonNull Address participantAddr);

    /**
    * Find a one to one chat room. <br/>
    * <br/>
    * No reference is transfered to the application. The {@link Core} keeps a<br/>
    * reference on the chat room. <br/>
    * @param localAddr a linphone address.   <br/>
    * @param participantAddr a linphone address.   <br/>
    * @param encrypted whether to look for an encrypted chat room or not <br/>
    * @return {@link ChatRoom} where messaging can take place.   <br/>
    * @deprecated 02/07/2020, use {@link #searchChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom findOneToOneChatRoom(@NonNull Address localAddr, @NonNull Address participantAddr, boolean encrypted);

    /**
    * Get the call by callid. <br/>
    * <br/>
    * @return call {@link Call}, return null if there is no call find.   <br/>
    */
    @Nullable
    public Call getCallByCallid(String callId);

    /**
    * Get the call with the remote_address specified. <br/>
    * <br/>
    * @param remoteAddress The remote address of the call that we want to get   <br/>
    * @return The call if it has been found, null otherwise.  <br/>
    * @deprecated 08/07/2020 use {@link #getCallByRemoteAddress2} instead <br/>
    */
    @Deprecated
    @Nullable
    public Call getCallByRemoteAddress(@NonNull String remoteAddress);

    /**
    * Get the call with the specified {@link Address}. <br/>
    * <br/>
    * @param remoteAddress the {@link Address} for which the call remote address must<br/>
    * match   <br/>
    * @return the {@link Call} of the call if found.   <br/>
    */
    @Nullable
    public Call getCallByRemoteAddress2(@NonNull Address remoteAddress);

    /**
    * Get the list of call logs (past calls). <br/>
    * <br/>
    * At the contrary of linphone_core_get_call_logs, it is your responsibility to<br/>
    * unref the logs and free this list once you are done using it. <br/>
    * @param peerAddress The remote {@link Address} object.   <br/>
    * @param localAddress The local {@link Address} object   <br/>
    * @return A list of {@link CallLog}.      <br/>
    */
    @NonNull
    public CallLog[] getCallHistory(@NonNull Address peerAddress, @NonNull Address localAddress);

    /**
    * Get the list of call logs (past calls) that matches the given {@link Address}. <br/>
    * <br/>
    * At the contrary of linphone_core_get_call_logs, it is your responsibility to<br/>
    * unref the logs and free this list once you are done using it. <br/>
    * @param address {@link Address} object   <br/>
    * @return A list of {@link CallLog}.      <br/>
    * @deprecated 29/10/2018 Use {@link #getCallHistory} instead.<br/>
    */
    @Deprecated
    @NonNull
    public CallLog[] getCallHistoryForAddress(@NonNull Address address);

    /**
    * Get a chat room. <br/>
    * <br/>
    * If it does not exist yet, it will be created as a basic chat room. No reference<br/>
    * is transfered to the application. The {@link Core} keeps a reference on the<br/>
    * chat room. warning: This method is prone to errors, use {@link #searchChatRoom}<br/>
    * instead <br/>
    * @param peerAddr a linphone address.   <br/>
    * @param localAddr a linphone address.   <br/>
    * @return {@link ChatRoom} where messaging can take place.   <br/>
    * @deprecated 02/07/2020, use {@link #searchChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom getChatRoom(@NonNull Address peerAddr, @NonNull Address localAddr);

    /**
    * Get a chat room whose peer is the supplied address. <br/>
    * <br/>
    * If it does not exist yet, it will be created as a basic chat room. No reference<br/>
    * is transfered to the application. The {@link Core} keeps a reference on the<br/>
    * chat room. warning: This method is prone to errors, use {@link #searchChatRoom}<br/>
    * instead <br/>
    * @param addr a linphone address.   <br/>
    * @return {@link ChatRoom} where messaging can take place.   <br/>
    * @deprecated 02/07/2020, use {@link #searchChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom getChatRoom(@NonNull Address addr);

    /**
    * Get a chat room for messaging from a sip uri like sip:joe@sip.linphone.org. <br/>
    * <br/>
    * If it does not exist yet, it will be created as a basic chat room. No reference<br/>
    * is transfered to the application. The {@link Core} keeps a reference on the<br/>
    * chat room. warning: This method is prone to errors, use {@link #searchChatRoom}<br/>
    * instead <br/>
    * @param to The destination address for messages.   <br/>
    * @return {@link ChatRoom} where messaging can take place.   <br/>
    * @deprecated 02/07/2020, use {@link #searchChatRoom} instead<br/>
    */
    @Deprecated
    @Nullable
    public ChatRoom getChatRoomFromUri(@NonNull String to);

    /**
    * Search a {@link Friend} by its reference key. <br/>
    * <br/>
    * @param key The reference key to use to search the friend.   <br/>
    * @return The {@link Friend} object corresponding to the given reference key.   <br/>
    */
    @Nullable
    public Friend getFriendByRefKey(@NonNull String key);

    /**
    * Retrieves the list of {@link Friend} from the core that has the given display<br/>
    * name. <br/>
    * <br/>
    * @param name the name of the list   <br/>
    * @return the first {@link FriendList} object or null.   <br/>
    */
    @Nullable
    public FriendList getFriendListByName(@NonNull String name);

    /**
    * Get payload type from mime type and clock rate. <br/>
    * <br/>
    * This function searches in audio and video codecs for the given payload type<br/>
    * name and clockrate. <br/>
    * @param type payload mime type (I.E SPEEX, PCMU, VP8)   <br/>
    * @param rate can be LINPHONE_FIND_PAYLOAD_IGNORE_RATE <br/>
    * @param channels number of channels, can be<br/>
    * LINPHONE_FIND_PAYLOAD_IGNORE_CHANNELS <br/>
    * @return Returns null if not found. If a {@link PayloadType} is returned, it<br/>
    * must be released with linphone_payload_type_unref after using it.     <br/>
    * warning: The returned payload type is allocated as a floating reference i.e.<br/>
    * the reference counter is initialized to 0. <br/>
    */
    @Nullable
    public PayloadType getPayloadType(@NonNull String type, int rate, int channels);

    /**
    * Search for a {@link ProxyConfig} by it's idkey. <br/>
    * <br/>
    * @param idkey An arbitrary idkey string associated to a proxy configuration <br/>
    * @return the {@link ProxyConfig} object for the given idkey value, or null if<br/>
    * none found   <br/>
    */
    @Nullable
    public ProxyConfig getProxyConfigByIdkey(String idkey);

    /**
    * Return the unread chat message count for a given local address. <br/>
    * <br/>
    * @param address {@link Address} object.   <br/>
    * @return The unread chat message count. <br/>
    */
    public int getUnreadChatMessageCountFromLocal(@NonNull Address address);

    /**
    * Get the zrtp sas validation status for a peer uri. <br/>
    * <br/>
    * Once the SAS has been validated or rejected, the status will never return to<br/>
    * Unknown (unless you delete your cache) <br/>
    * @param addr the peer uri  <br/>
    * @return - LinphoneZrtpPeerStatusUnknown: this uri is not present in cache OR<br/>
    * during calls with the active device, SAS never was validated or rejected<br/>
    */
    public ZrtpPeerStatus getZrtpStatus(@NonNull String addr);

    /**
    * Check whether the device has a hardware echo canceller. <br/>
    * <br/>
    * @return true if it does, false otherwise <br/>
    */
    public boolean hasBuiltinEchoCanceller();

    /**
    * Check whether the device is flagged has crappy opengl. <br/>
    * <br/>
    * @return true if crappy opengl flag is set, false otherwise <br/>
    */
    public boolean hasCrappyOpengl();

    /**
    * Tells whether there is a call running. <br/>
    * <br/>
    * @return A boolean value telling whether a call is currently running or not <br/>
    */
    public boolean inCall();

    /**
    * Constructs a {@link Address} from the given string if possible. <br/>
    * <br/>
    * In case of just a username, characters will be unescaped. If a phone number is<br/>
    * detected, it will be flattened. sip: or sips: prefix will be added if not<br/>
    * present. Finally,  will be added if not present using default proxy config.<br/>
    * see: {@link ProxyConfig#normalizeSipUri} for documentation. <br/>
    * @param url the url to parse   <br/>
    * @return the {@link Address} matching the url or null in case of failure.     <br/>
    */
    @Nullable
    public Address interpretUrl(@NonNull String url);

    /**
    * Initiates an outgoing call. <br/>
    * <br/>
    * The application doesn't own a reference to the returned LinphoneCall object.<br/>
    * Use linphone_call_ref to safely keep the LinphoneCall pointer valid within your<br/>
    * application.<br/>
    * @param url The destination of the call (sip address, or phone number).   <br/>
    * @return A {@link Call} object or null in case of failure.   <br/>
    */
    @Nullable
    public Call invite(@NonNull String url);

    /**
    * Initiates an outgoing call given a destination {@link Address} The {@link Address}<br/>
    * can be constructed directly using linphone_address_new, or created by {@link #interpretUrl}<br/>
    * . <br/>
    * <br/>
    * The application doesn't own a reference to the returned {@link Call} object.<br/>
    * Use linphone_call_ref to safely keep the {@link Call} pointer valid within your<br/>
    * application. <br/>
    * @param addr The destination of the call (sip address).   <br/>
    * @return A {@link Call} object or null in case of failure.   <br/>
    */
    @Nullable
    public Call inviteAddress(@NonNull Address addr);

    /**
    * Initiates an outgoing call given a destination {@link Address} The {@link Address}<br/>
    * can be constructed directly using linphone_address_new, or created by {@link #interpretUrl}<br/>
    * . <br/>
    * <br/>
    * The application doesn't own a reference to the returned {@link Call} object.<br/>
    * Use linphone_call_ref to safely keep the {@link Call} pointer valid within your<br/>
    * application. If the proxy is not specified in parameters, the caller proxy will<br/>
    * be automatically selected by finding what is the best to reach the destination<br/>
    * of the call. <br/>
    * @param addr The destination of the call (sip address).   <br/>
    * @param params Call parameters   <br/>
    * @return A {@link Call} object or null in case of failure.   <br/>
    */
    @Nullable
    public Call inviteAddressWithParams(@NonNull Address addr, @NonNull CallParams params);

    /**
    * Initiates an outgoing call according to supplied call parameters The<br/>
    * application doesn't own a reference to the returned {@link Call} object. <br/>
    * <br/>
    * Use linphone_call_ref to safely keep the {@link Call} pointer valid within your<br/>
    * application. <br/>
    * @param url The destination of the call (sip address, or phone number).   <br/>
    * @param params the {@link CallParams} call parameters   <br/>
    * @return A {@link Call} object or null in case of failure.   <br/>
    */
    @Nullable
    public Call inviteWithParams(@NonNull String url, @NonNull CallParams params);

    /**
    * Tells whether a content type is supported. <br/>
    * <br/>
    * @param contentType The content type to check   <br/>
    * @return A boolean value telling whether the specified content type is supported<br/>
    * or not. <br/>
    */
    public boolean isContentTypeSupported(@NonNull String contentType);

    /**
    * Checks if the given media filter is loaded and usable. <br/>
    * <br/>
    * This is for advanced users of the library, mainly to expose mediastreamer video<br/>
    * filter status. <br/>
    * @param filtername the filter name   <br/>
    * @return true if the filter is loaded and usable, false otherwise <br/>
    */
    public boolean isMediaFilterSupported(@NonNull String filtername);

    /**
    * Main loop function. <br/>
    * <br/>
    * It is crucial that your application call it periodically.<br/>
    * {@link #iterate} performs various backgrounds tasks:<br/>
    */
    public void iterate();

    /**
    * Make the local participant leave the running conference. <br/>
    * <br/>
    * @return 0 if succeeded. Negative number if failed <br/>
    */
    public int leaveConference();

    /**
    * Tells if lime is available. <br/>
    * <br/>
    * @deprecated 04/02/2019 Use linphone_core_lime_x3dh_available instead. <br/>
    */
    @Deprecated
    public boolean limeAvailable();

    /**
    * Returns the lime state. <br/>
    * <br/>
    * @return The current lime state<br/>
    * @deprecated 04/02/2019 Use linphone_core_lime_x3dh_enabled instead. <br/>
    */
    @Deprecated
    public LimeState limeEnabled();

    /**
    * Tells if LIME X3DH is available. <br/>
    * <br/>
    */
    public boolean limeX3DhAvailable();

    /**
    * Update current config with the content of a xml config file. <br/>
    * <br/>
    * @param xmlUri the path to the xml file   <br/>
    */
    public void loadConfigFromXml(@NonNull String xmlUri);

    /**
    * Check if a media encryption type is supported. <br/>
    * <br/>
    * @param menc {@link MediaEncryption} <br/>
    * @return whether a media encryption scheme is supported by the {@link Core}<br/>
    * engine <br/>
    */
    public boolean mediaEncryptionSupported(MediaEncryption menc);

    /**
    * Migrates the call logs from the linphonerc to the database if not done yet. <br/>
    * <br/>
    */
    public void migrateLogsFromRcToDb();

    /**
    * Migrate configuration so that all SIP transports are enabled. <br/>
    * <br/>
    * Versions of linphone < 3.7 did not support using multiple SIP transport<br/>
    * simultaneously. This function helps application to migrate the configuration so<br/>
    * that all transports are enabled. Existing proxy configuration are added a<br/>
    * transport parameter so that they continue using the unique transport that was<br/>
    * set previously. This function must be used just after creating the core, before<br/>
    * any call to {@link #iterate} <br/>
    * @return 1 if migration was done, 0 if not done because unnecessary or already<br/>
    * done, -1 in case of error. <br/>
    */
    public int migrateToMultiTransport();

    /**
    * Notify all friends that have subscribed. <br/>
    * <br/>
    * @param presence {@link PresenceModel} to notify   <br/>
    */
    public void notifyAllFriends(@NonNull PresenceModel presence);

    /**
    * Notifies the upper layer that a presence status has been received by calling<br/>
    * the appropriate callback if one has been set. <br/>
    * <br/>
    * This method is for advanced usage, where customization of the liblinphone's<br/>
    * internal behavior is required. <br/>
    * @param linphoneFriend the {@link Friend} whose presence information has been<br/>
    * received.   <br/>
    */
    public void notifyNotifyPresenceReceived(@NonNull Friend linphoneFriend);

    /**
    * Notifies the upper layer that a presence model change has been received for the<br/>
    * uri or telephone number given as a parameter, by calling the appropriate<br/>
    * callback if one has been set. <br/>
    * <br/>
    * This method is for advanced usage, where customization of the liblinphone's<br/>
    * internal behavior is required. <br/>
    * @param linphoneFriend the {@link Friend} whose presence information has been<br/>
    * received.   <br/>
    * @param uriOrTel telephone number or sip uri   <br/>
    * @param presenceModel the {@link PresenceModel} that has been modified   <br/>
    */
    public void notifyNotifyPresenceReceivedForUriOrTel(@NonNull Friend linphoneFriend, @NonNull String uriOrTel, @NonNull PresenceModel presenceModel);

    /**
    * Pause all currently running calls. <br/>
    * <br/>
    * @return 0 <br/>
    */
    public int pauseAllCalls();

    /**
    * Plays a dtmf sound to the local user. <br/>
    * <br/>
    * @param dtmf DTMF to play ['0'..'16'] | '#' | '#' <br/>
    * @param durationMs Duration in ms, -1 means play until next further call to<br/>
    * {@link #stopDtmf} <br/>
    */
    public void playDtmf(char dtmf, int durationMs);

    /**
    * Plays an audio file to the local user. <br/>
    * <br/>
    * This function works at any time, during calls, or when no calls are running. It<br/>
    * doesn't request the underlying audio system to support multiple playback<br/>
    * streams. <br/>
    * @param audiofile The path to an audio file in wav PCM 16 bit format   <br/>
    * @return 0 on success, -1 on error <br/>
    */
    public int playLocal(@NonNull String audiofile);

    /**
    * Empties sound resources to allow a new call to be accepted. <br/>
    * <br/>
    * This function is autyomatically called by the core if the media resource mode<br/>
    * is set to unique. <br/>
    * @return An integer returning the exit value. If it is 0, sound resources have<br/>
    * been emptied. Otherwise, sound resources are busy and cannot be freed<br/>
    * immediately. <br/>
    */
    public int preemptSoundResources();

    /**
    * Call generic OpenGL render for a given core. <br/>
    * <br/>
    */
    public void previewOglRender();

    /**
    * Publish an event state. <br/>
    * <br/>
    * This first create a {@link Event} with {@link #createPublish} and calls {@link Event#sendPublish}<br/>
    * to actually send it. After expiry, the publication is refreshed unless it is<br/>
    * terminated before. <br/>
    * @param resource the resource uri for the event   <br/>
    * @param event the event name   <br/>
    * @param expires the lifetime of event being published, -1 if no associated<br/>
    * duration, in which case it will not be refreshed. <br/>
    * @param body the actual published data   <br/>
    * @return the {@link Event} holding the context of the publish.   <br/>
    */
    @Nullable
    public Event publish(@NonNull Address resource, @NonNull String event, int expires, @NonNull Content body);

    /**
    * Gets keep alive interval of real time text. <br/>
    * <br/>
    * @return keep alive interval of real time text. <br/>
    */
    public int realtimeTextGetKeepaliveInterval();

    /**
    * Set keep alive interval for real time text. <br/>
    * <br/>
    * @param interval The keep alive interval of real time text, 25000 by default. <br/>
    */
    public void realtimeTextSetKeepaliveInterval(int interval);

    /**
    * force registration refresh to be initiated upon next iterate <br/>
    * <br/>
    */
    public void refreshRegisters();

    /**
    * Black list a friend. <br/>
    * <br/>
    * same as {@link Friend#setIncSubscribePolicy} with {@link SubscribePolicy#SPDeny}<br/>
    * policy; <br/>
    * @param linphoneFriend {@link Friend} to reject   <br/>
    */
    public void rejectSubscriber(@NonNull Friend linphoneFriend);

    /**
    * Reload mediastreamer2 plugins from specified directory. <br/>
    * <br/>
    * @param path the path from where plugins are to be loaded, pass null to use<br/>
    * default (compile-time determined) plugin directory.   <br/>
    */
    public void reloadMsPlugins(@Nullable String path);

    /**
    * Update detection of sound devices. <br/>
    * <br/>
    * Use this function when the application is notified of USB plug events, so that<br/>
    * list of available hardwares for sound playback and capture is updated. <br/>
    */
    public void reloadSoundDevices();

    /**
    * Update detection of camera devices. <br/>
    * <br/>
    * Use this function when the application is notified of USB plug events, so that<br/>
    * list of available hardwares for video capture is updated. <br/>
    */
    public void reloadVideoDevices();

    /**
    * Removes an authentication information object. <br/>
    * <br/>
    * @param info The {@link AuthInfo} to remove.   <br/>
    */
    public void removeAuthInfo(@NonNull AuthInfo info);

    /**
    * Remove a specific call log from call history list. <br/>
    * <br/>
    * This function destroys the call log object. It must not be accessed anymore by<br/>
    * the application after calling this function. <br/>
    * @param callLog {@link CallLog} object to remove.   <br/>
    */
    public void removeCallLog(@NonNull CallLog callLog);

    /**
    * Remove support for the specified content type. <br/>
    * <br/>
    * It is the application responsibility to handle it correctly afterwards. <br/>
    * @param contentType The content type to remove support for   <br/>
    */
    public void removeContentTypeSupport(@NonNull String contentType);

    /**
    * Removes a friend list. <br/>
    * <br/>
    * @param list {@link FriendList} object   <br/>
    */
    public void removeFriendList(@NonNull FriendList list);

    /**
    * Remove a call from the conference. <br/>
    * <br/>
    * @param call a call that has been previously merged into the conference.  <br/>
    * After removing the remote participant belonging to the supplied call, the call<br/>
    * becomes a normal call in paused state. If one single remote participant is left<br/>
    * alone together with the local user in the conference after the removal, then<br/>
    * the conference is automatically transformed into a simple call in<br/>
    * StreamsRunning state. The conference's resources are then automatically<br/>
    * destroyed.<br/>
    * In other words, unless {@link #leaveConference} is explicitly called, the last<br/>
    * remote participant of a conference is automatically put in a simple call in<br/>
    * running state.<br/>
    * @return 0 if successful, -1 otherwise. <br/>
    */
    public int removeFromConference(@NonNull Call call);

    /**
    * Remove the given linphone specs from the list of functionalities the linphone<br/>
    * client supports. <br/>
    * <br/>
    * @param spec The spec to remove   <br/>
    */
    public void removeLinphoneSpec(@NonNull String spec);

    /**
    * Removes a proxy configuration. <br/>
    * <br/>
    * {@link Core} will then automatically unregister and place the proxy<br/>
    * configuration on a deleted list. For that reason, a removed proxy does NOT need<br/>
    * to be freed. <br/>
    * @param config the {@link ProxyConfig} to remove   <br/>
    */
    public void removeProxyConfig(@NonNull ProxyConfig config);

    /**
    * Remove a supported tag. <br/>
    * <br/>
    * @param tag The tag to remove  <br/>
    * see: {@link #addSupportedTag} <br/>
    */
    public void removeSupportedTag(@NonNull String tag);

    /**
    * Reset the counter of missed calls. <br/>
    * <br/>
    */
    public void resetMissedCallsCount();

    /**
    * Find a chat room. <br/>
    * <br/>
    * @param params The chat room parameters to match {@link ChatRoomParams} or null <br/>
    *  <br/>
    * @param localAddr {@link Address} representing the local proxy configuration or<br/>
    * null   <br/>
    * @param remoteAddr {@link Address} to search for or null   <br/>
    * @param participants The participants that must be present in the chat room to<br/>
    * find.    <br/>
    * @return A matching chat room or null if none matches.   <br/>
    */
    @Nullable
    public ChatRoom searchChatRoom(@Nullable ChatRoomParams params, @Nullable Address localAddr, @Nullable Address remoteAddr, @Nullable Address[] participants);

    /**
    * Sets the UDP port range from which to randomly select the port used for audio<br/>
    * streaming. <br/>
    * <br/>
    * @param minPort The lower bound of the audio port range to use <br/>
    * @param maxPort The upper bound of the audio port range to use <br/>
    */
    public void setAudioPortRange(int minPort, int maxPort);

    /**
    * Assign an audio file to be played locally upon call failure, for a given<br/>
    * reason. <br/>
    * <br/>
    * @param reason the {@link Reason} representing the failure error code. <br/>
    * @param audiofile a wav file to be played when such call failure happens.   <br/>
    */
    public void setCallErrorTone(Reason reason, @Nullable String audiofile);

    /**
    * Set the rectangle where the decoder will search a QRCode. <br/>
    * <br/>
    * @param x axis <br/>
    * @param y axis <br/>
    * @param w width <br/>
    * @param h height <br/>
    */
    public void setQrcodeDecodeRect(int x, int y, int w, int h);

    /**
    * Sets the UDP port range from which to randomly select the port used for text<br/>
    * streaming. <br/>
    * <br/>
    * @param minPort The lower bound of the text port range to use <br/>
    * @param maxPort The upper bound of the text port range to use <br/>
    */
    public void setTextPortRange(int minPort, int maxPort);

    /**
    * Assign an audio file to be played as a specific tone id. <br/>
    * <br/>
    * This function typically allows to customize telephony tones per country. <br/>
    * @param toneId the #LinphoneToneId <br/>
    * @param audiofile a wav file to be played.   <br/>
    */
    public void setTone(ToneID toneId, @NonNull String audiofile);

    /**
    * Set the user agent string used in SIP messages. <br/>
    * <br/>
    * Set the user agent string used in SIP messages as "[ua_name]/[version]". No<br/>
    * slash character will be printed if null is given to "version". If null is given<br/>
    * to "ua_name" and "version" both, the User-agent header will be empty.<br/>
    * This function should be called just after linphone_factory_create_core ideally. <br/>
    * @param name Name of the user agent.   <br/>
    * @param version Version of the user agent.   <br/>
    */
    public void setUserAgent(@Nullable String name, @Nullable String version);

    /**
    * Sets the UDP port range from which to randomly select the port used for video<br/>
    * streaming. <br/>
    * <br/>
    * @param minPort The lower bound of the video port range to use <br/>
    * @param maxPort The upper bound of the video port range to use <br/>
    */
    public void setVideoPortRange(int minPort, int maxPort);

    /**
    * Tells whether a specified sound device can capture sound. <br/>
    * <br/>
    * @param device the device name as returned by linphone_core_get_sound_devices   <br/>
    * @return A boolean value telling whether the specified sound device can capture<br/>
    * sound <br/>
    * @deprecated 08/07/2020 use {@link AudioDevice} API instead()<br/>
    */
    @Deprecated
    public boolean soundDeviceCanCapture(@NonNull String device);

    /**
    * Tells whether a specified sound device can play sound. <br/>
    * <br/>
    * @param device the device name as returned by linphone_core_get_sound_devices   <br/>
    * @return A boolean value telling whether the specified sound device can play<br/>
    * sound <br/>
    * @deprecated 08/07/2020 use {@link AudioDevice} API instead()<br/>
    */
    @Deprecated
    public boolean soundDeviceCanPlayback(@NonNull String device);

    /**
    * Check if a call will need the sound resources in near future (typically an<br/>
    * outgoing call that is awaiting response). <br/>
    * <br/>
    * In liblinphone, it is not possible to have two independant calls using sound<br/>
    * device or camera at the same time. In order to prevent this situation, an<br/>
    * application can use {@link #soundResourcesLocked} to know whether it is<br/>
    * possible at a given time to start a new outgoing call. When the function<br/>
    * returns true, an application should not allow the user to start an outgoing<br/>
    * call. <br/>
    * @return A boolean value telling whether a call will need the sound resources in<br/>
    * near future <br/>
    */
    public boolean soundResourcesLocked();

    /**
    * Start a {@link Core} object after it has been instantiated and not<br/>
    * automatically started. <br/>
    * <br/>
    * Also re-initialize a {@link Core} object that has been stopped using {@link #stop}<br/>
    * . Must be called only if {@link GlobalState} is either Ready of Off. State will<br/>
    * changed to Startup, Configuring and then On.<br/>
    * @return 0: success, -1: global failure, -2: could not connect database <br/>
    */
    public int start();

    /**
    * Start recording the running conference. <br/>
    * <br/>
    * @param path Path to the file where the recording will be written   <br/>
    * @return 0 if succeeded. Negative number if failed <br/>
    */
    public int startConferenceRecording(@NonNull String path);

    /**
    * Starts an echo calibration of the sound devices, in order to find adequate<br/>
    * settings for the echo canceler automatically. <br/>
    * <br/>
    * @return LinphoneStatus whether calibration has started or not. <br/>
    */
    public int startEchoCancellerCalibration();

    /**
    * Start the simulation of call to test the latency with an external device. <br/>
    * <br/>
    * @param rate Sound sample rate. <br/>
    * @return -1 in case of failure, 1 otherwise. <br/>
    */
    public int startEchoTester(int rate);

    /**
    * Stop a {@link Core} object after it has been instantiated and started. <br/>
    * <br/>
    * If stopped, it can be started again using {@link #start}. Must be called only<br/>
    * if {@link GlobalState} is either On. State will changed to Shutdown and then<br/>
    * Off.<br/>
    */
    public void stop();

    /**
    * Stop asynchronously a {@link Core} object after it has been instantiated and<br/>
    * started. <br/>
    * <br/>
    * State changes to Shutdown then {@link #iterate} must be called to allow the<br/>
    * Core to end asynchronous tasks (terminate call, etc.). When all tasks are<br/>
    * finished, State will change to Off. Must be called only if {@link GlobalState}<br/>
    * is On. When {@link GlobalState} is Off {@link Core} can be started again using<br/>
    * {@link #start}.<br/>
    */
    public void stopAsync();

    /**
    * Stop recording the running conference. <br/>
    * <br/>
    * @return 0 if succeeded. Negative number if failed <br/>
    */
    public int stopConferenceRecording();

    /**
    * Stops playing a dtmf started by {@link #playDtmf}. <br/>
    * <br/>
    */
    public void stopDtmf();

    /**
    * Stop the simulation of call. <br/>
    * <br/>
    */
    public int stopEchoTester();

    /**
    * Whenever the liblinphone is playing a ring to advertise an incoming call or<br/>
    * ringback of an outgoing call, this function stops the ringing. <br/>
    * <br/>
    * Typical use is to stop ringing when the user requests to ignore the call. <br/>
    */
    public void stopRinging();

    /**
    * Create an outgoing subscription, specifying the destination resource, the event<br/>
    * name, and an optional content body. <br/>
    * <br/>
    * If accepted, the subscription runs for a finite period, but is automatically<br/>
    * renewed if not terminated before. <br/>
    * @param resource the destination resource   <br/>
    * @param event the event name   <br/>
    * @param expires the whished duration of the subscription <br/>
    * @param body an optional body, may be null.   <br/>
    * @return a {@link Event} holding the context of the created subcription.   <br/>
    */
    @NonNull
    public Event subscribe(@NonNull Address resource, @NonNull String event, int expires, @Nullable Content body);

    /**
    * Take a photo of currently from capture device and write it into a jpeg file. <br/>
    * <br/>
    * Note that the snapshot is asynchronous, an application shall not assume that<br/>
    * the file is created when the function returns.<br/>
    * @param file a path where to write the jpeg content.   <br/>
    * @return 0 if successfull, -1 otherwise (typically if jpeg format is not<br/>
    * supported). <br/>
    */
    public int takePreviewSnapshot(@NonNull String file);

    /**
    * Terminates all the calls. <br/>
    * <br/>
    * @return 0 <br/>
    */
    public int terminateAllCalls();

    /**
    * Terminate the running conference. <br/>
    * <br/>
    * If it is a local conference, all calls inside it will become back separate<br/>
    * calls and will be put in #LinphoneCallPaused state. If it is a conference<br/>
    * involving a focus server, all calls inside the conference will be terminated. <br/>
    * @return 0 if succeeded. Negative number if failed <br/>
    */
    public int terminateConference();

    /**
    * Upload the log collection to the configured server url. <br/>
    * <br/>
    */
    public void uploadLogCollection();

    /**
    * Tells the core to use a separate window for local camera preview video, instead<br/>
    * of inserting local view within the remote video window. <br/>
    * <br/>
    * @param yesno true to use a separate window, false to insert the preview in the<br/>
    * remote video window. <br/>
    */
    public void usePreviewWindow(boolean yesno);

    /**
    * Specify whether the tls server certificate must be verified when connecting to<br/>
    * a SIP/TLS server. <br/>
    * <br/>
    * @param yesno A boolean value telling whether the tls server certificate must be<br/>
    * verified <br/>
    */
    public void verifyServerCertificates(boolean yesno);

    /**
    * Specify whether the tls server certificate common name must be verified when<br/>
    * connecting to a SIP/TLS server. <br/>
    * <br/>
    * @param yesno A boolean value telling whether the tls server certificate common<br/>
    * name must be verified <br/>
    */
    public void verifyServerCn(boolean yesno);

    /**
    * Test if video is supported. <br/>
    * <br/>
    * @return true if the library was built with video support, false otherwise <br/>
    */
    public boolean videoSupported();

    /**
    * Compress the log collection in a single file. <br/>
    * <br/>
    * @return The path of the compressed log collection file (to be freed calling<br/>
    * ms_free()).     <br/>
    */
    @NonNull
    public String compressLogCollection();

    /**
    * Enable the linphone core log collection to upload logs on a server. <br/>
    * <br/>
    * @param state {@link LogCollectionState} value telling whether to enable log<br/>
    * collection or not. <br/>
    */
    public void enableLogCollection(LogCollectionState state);

    /**
    * Get the max file size in bytes of the files used for log collection. <br/>
    * <br/>
    * @return The max file size in bytes of the files used for log collection.   <br/>
    */
    @NonNull
    public int getLogCollectionMaxFileSize();

    /**
    * Get the path where the log files will be written for log collection. <br/>
    * <br/>
    * @return The path where the log files will be written.   <br/>
    */
    @NonNull
    public String getLogCollectionPath();

    /**
    * Get the prefix of the filenames that will be used for log collection. <br/>
    * <br/>
    * @return The prefix of the filenames used for log collection.   <br/>
    */
    @NonNull
    public String getLogCollectionPrefix();

    /**
    * Returns liblinphone's version as a string. <br/>
    * <br/>
    * @return the current version of the {@link Core}   <br/>
    */
    @NonNull
    public String getVersion();

    /**
    * Tells whether the linphone core log collection is enabled. <br/>
    * <br/>
    * @return The {@link LogCollectionState} of the {@link Core} log collection. <br/>
    */
    public LogCollectionState logCollectionEnabled();

    /**
    * Reset the log collection by removing the log files. <br/>
    * <br/>
    */
    public void resetLogCollection();

    /**
    * Enable logs serialization (output logs from either the thread that creates the<br/>
    * linphone core or the thread that calls {@link #iterate}). <br/>
    * <br/>
    * Must be called before creating the linphone core. <br/>
    */
    public void serializeLogs();

    /**
    * Set the max file size in bytes of the files used for log collection. <br/>
    * <br/>
    * Warning: this function should only not be used to change size dynamically but<br/>
    * instead only before calling {@link #enableLogCollection}. If you increase max<br/>
    * size on runtime, logs chronological order COULD be broken. <br/>
    * @param size The max file size in bytes of the files used for log collection. <br/>
    */
    public void setLogCollectionMaxFileSize(int size);

    /**
    * Set the path of a directory where the log files will be written for log<br/>
    * collection. <br/>
    * <br/>
    * @param path The path where the log files will be written.   <br/>
    */
    public void setLogCollectionPath(@NonNull String path);

    /**
    * Set the prefix of the filenames that will be used for log collection. <br/>
    * <br/>
    * @param prefix The prefix to use for the filenames for log collection.   <br/>
    */
    public void setLogCollectionPrefix(@NonNull String prefix);

    /**
    * True if tunnel support was compiled. <br/>
    * <br/>
    * @return true if library was built with tunnel, false otherwise <br/>
    */
    public boolean tunnelAvailable();

    /**
    * Return the availability of uPnP. <br/>
    * <br/>
    * @return true if uPnP is available otherwise return false. <br/>
    */
    public boolean upnpAvailable();

    /**
    * Tells whether VCARD support is builtin. <br/>
    * <br/>
    * @return true if VCARD is supported, false otherwise. <br/>
    */
    public boolean vcardSupported();

    /**
    */
    public void addListener(CoreListener listener);

    /**
    */
    public void removeListener(CoreListener listener);

    /**
      * Sets the object to store in this object user's data
      * @param data the object to store
      */
    public void setUserData(Object data);

    /**
      * Gets the object stored in this object user's data
      * @return the object store if any, null otherwise
      */
    public Object getUserData();

    /**
      * Gets the native pointer used by this class to make native method calls.
      * @return the nativer pointer, as long
      */
    public long getNativePointer();

    public String toString();
}

class CoreImpl implements Core {

    protected long nativePtr = 0;
    protected transient Object userData = null;

    protected CoreImpl(long ptr) {
        nativePtr = ptr;
    }

    public long getNativePointer() {
        return nativePtr;
    }


    private native String getAdaptiveRateAlgorithm(long nativePtr);
    @Override @NonNull
    synchronized public String getAdaptiveRateAlgorithm()  {
        
        return getAdaptiveRateAlgorithm(nativePtr);
    }

    private native void setAdaptiveRateAlgorithm(long nativePtr, String algorithm);
    @Override
    synchronized public void setAdaptiveRateAlgorithm(@NonNull String algorithm)  {
        
        setAdaptiveRateAlgorithm(nativePtr, algorithm);
    }

    private native boolean adaptiveRateControlEnabled(long nativePtr);
    @Override
    synchronized public boolean adaptiveRateControlEnabled()  {
        
        return adaptiveRateControlEnabled(nativePtr);
    }

    private native void enableAdaptiveRateControl(long nativePtr, boolean enabled);
    @Override
    synchronized public void enableAdaptiveRateControl(boolean enabled)  {
        
        enableAdaptiveRateControl(nativePtr, enabled);
    }

    private native boolean audioAdaptiveJittcompEnabled(long nativePtr);
    @Override
    synchronized public boolean audioAdaptiveJittcompEnabled()  {
        
        return audioAdaptiveJittcompEnabled(nativePtr);
    }

    private native void enableAudioAdaptiveJittcomp(long nativePtr, boolean enable);
    @Override
    synchronized public void enableAudioAdaptiveJittcomp(boolean enable)  {
        
        enableAudioAdaptiveJittcomp(nativePtr, enable);
    }

    private native AudioDevice[] getAudioDevices(long nativePtr);
    @Override @NonNull
    synchronized public AudioDevice[] getAudioDevices()  {
        
        return getAudioDevices(nativePtr);
    }

    private native int getAudioDscp(long nativePtr);
    @Override
    synchronized public int getAudioDscp()  {
        
        return getAudioDscp(nativePtr);
    }

    private native void setAudioDscp(long nativePtr, int dscp);
    @Override
    synchronized public void setAudioDscp(int dscp)  {
        
        setAudioDscp(nativePtr, dscp);
    }

    private native int getAudioJittcomp(long nativePtr);
    @Override
    synchronized public int getAudioJittcomp()  {
        
        return getAudioJittcomp(nativePtr);
    }

    private native void setAudioJittcomp(long nativePtr, int milliseconds);
    @Override
    synchronized public void setAudioJittcomp(int milliseconds)  {
        
        setAudioJittcomp(nativePtr, milliseconds);
    }

    private native String getAudioMulticastAddr(long nativePtr);
    @Override @Nullable
    synchronized public String getAudioMulticastAddr()  {
        
        return getAudioMulticastAddr(nativePtr);
    }

    private native int setAudioMulticastAddr(long nativePtr, String ip);
    @Override
    synchronized public int setAudioMulticastAddr(@Nullable String ip)  {
        
        return setAudioMulticastAddr(nativePtr, ip);
    }

    private native boolean audioMulticastEnabled(long nativePtr);
    @Override
    synchronized public boolean audioMulticastEnabled()  {
        
        return audioMulticastEnabled(nativePtr);
    }

    private native void enableAudioMulticast(long nativePtr, boolean yesno);
    @Override
    synchronized public void enableAudioMulticast(boolean yesno)  {
        
        enableAudioMulticast(nativePtr, yesno);
    }

    private native int getAudioMulticastTtl(long nativePtr);
    @Override
    synchronized public int getAudioMulticastTtl()  {
        
        return getAudioMulticastTtl(nativePtr);
    }

    private native int setAudioMulticastTtl(long nativePtr, int ttl);
    @Override
    synchronized public int setAudioMulticastTtl(int ttl)  {
        
        return setAudioMulticastTtl(nativePtr, ttl);
    }

    private native PayloadType[] getAudioPayloadTypes(long nativePtr);
    @Override @NonNull
    synchronized public PayloadType[] getAudioPayloadTypes()  {
        
        return getAudioPayloadTypes(nativePtr);
    }

    private native void setAudioPayloadTypes(long nativePtr, PayloadType[] payloadTypes);
    @Override
    synchronized public void setAudioPayloadTypes(@Nullable PayloadType[] payloadTypes)  {
        
        setAudioPayloadTypes(nativePtr, payloadTypes);
    }

    private native int getAudioPort(long nativePtr);
    @Override
    synchronized public int getAudioPort()  {
        
        return getAudioPort(nativePtr);
    }

    private native void setAudioPort(long nativePtr, int port);
    @Override
    synchronized public void setAudioPort(int port)  {
        
        setAudioPort(nativePtr, port);
    }

    private native Range getAudioPortsRange(long nativePtr);
    @Override @NonNull
    synchronized public Range getAudioPortsRange()  {
        
        return (Range)getAudioPortsRange(nativePtr);
    }

    private native AuthInfo[] getAuthInfoList(long nativePtr);
    @Override @NonNull
    synchronized public AuthInfo[] getAuthInfoList()  {
        
        return getAuthInfoList(nativePtr);
    }

    private native void setAutoIterateEnabled(long nativePtr, boolean enable);
    @Override
    synchronized public void setAutoIterateEnabled(boolean enable)  {
        
        setAutoIterateEnabled(nativePtr, enable);
    }

    private native int getAvpfMode(long nativePtr);
    @Override
    synchronized public AVPFMode getAvpfMode()  {
        
        return AVPFMode.fromInt(getAvpfMode(nativePtr));
    }

    private native void setAvpfMode(long nativePtr, int mode);
    @Override
    synchronized public void setAvpfMode(AVPFMode mode)  {
        
        setAvpfMode(nativePtr, mode.toInt());
    }

    private native int getAvpfRrInterval(long nativePtr);
    @Override
    synchronized public int getAvpfRrInterval()  {
        
        return getAvpfRrInterval(nativePtr);
    }

    private native void setAvpfRrInterval(long nativePtr, int interval);
    @Override
    synchronized public void setAvpfRrInterval(int interval)  {
        
        setAvpfRrInterval(nativePtr, interval);
    }

    private native CallLog[] getCallLogs(long nativePtr);
    @Override @NonNull
    synchronized public CallLog[] getCallLogs()  {
        
        return getCallLogs(nativePtr);
    }

    private native String getCallLogsDatabasePath(long nativePtr);
    @Override @Nullable
    synchronized public String getCallLogsDatabasePath()  {
        
        return getCallLogsDatabasePath(nativePtr);
    }

    private native void setCallLogsDatabasePath(long nativePtr, String path);
    @Override
    synchronized public void setCallLogsDatabasePath(@Nullable String path)  {
        
        setCallLogsDatabasePath(nativePtr, path);
    }

    private native boolean callkitEnabled(long nativePtr);
    @Override
    synchronized public boolean callkitEnabled()  {
        
        return callkitEnabled(nativePtr);
    }

    private native void enableCallkit(long nativePtr, boolean enabled);
    @Override
    synchronized public void enableCallkit(boolean enabled)  {
        
        enableCallkit(nativePtr, enabled);
    }

    private native Call[] getCalls(long nativePtr);
    @Override @NonNull
    synchronized public Call[] getCalls()  {
        
        return getCalls(nativePtr);
    }

    private native int getCallsNb(long nativePtr);
    @Override
    synchronized public int getCallsNb()  {
        
        return getCallsNb(nativePtr);
    }

    private native int getCameraSensorRotation(long nativePtr);
    @Override
    synchronized public int getCameraSensorRotation()  {
        
        return getCameraSensorRotation(nativePtr);
    }

    private native String getCaptureDevice(long nativePtr);
    @Override @Nullable
    synchronized public String getCaptureDevice()  {
        
        return getCaptureDevice(nativePtr);
    }

    private native int setCaptureDevice(long nativePtr, String devid);
    @Override
    synchronized public int setCaptureDevice(@Nullable String devid)  {
        
        return setCaptureDevice(nativePtr, devid);
    }

    private native boolean chatEnabled(long nativePtr);
    @Override
    synchronized public boolean chatEnabled()  {
        
        return chatEnabled(nativePtr);
    }

    private native ChatRoom[] getChatRooms(long nativePtr);
    @Override @NonNull
    synchronized public ChatRoom[] getChatRooms()  {
        
        return getChatRooms(nativePtr);
    }

    private native Conference getConference(long nativePtr);
    @Override @Nullable
    synchronized public Conference getConference()  {
        
        return (Conference)getConference(nativePtr);
    }

    private native float getConferenceLocalInputVolume(long nativePtr);
    @Override
    synchronized public float getConferenceLocalInputVolume()  {
        
        return getConferenceLocalInputVolume(nativePtr);
    }

    private native boolean conferenceServerEnabled(long nativePtr);
    @Override
    synchronized public boolean conferenceServerEnabled()  {
        
        return conferenceServerEnabled(nativePtr);
    }

    private native void enableConferenceServer(long nativePtr, boolean enable);
    @Override
    synchronized public void enableConferenceServer(boolean enable)  {
        
        enableConferenceServer(nativePtr, enable);
    }

    private native int getConferenceSize(long nativePtr);
    @Override
    synchronized public int getConferenceSize()  {
        
        return getConferenceSize(nativePtr);
    }

    private native Config getConfig(long nativePtr);
    @Override @NonNull
    synchronized public Config getConfig()  {
        
        return (Config)getConfig(nativePtr);
    }

    private native int getConsolidatedPresence(long nativePtr);
    @Override
    synchronized public ConsolidatedPresence getConsolidatedPresence()  {
        
        return ConsolidatedPresence.fromInt(getConsolidatedPresence(nativePtr));
    }

    private native void setConsolidatedPresence(long nativePtr, int presence);
    @Override
    synchronized public void setConsolidatedPresence(ConsolidatedPresence presence)  {
        
        setConsolidatedPresence(nativePtr, presence.toInt());
    }

    private native Call getCurrentCall(long nativePtr);
    @Override @Nullable
    synchronized public Call getCurrentCall()  {
        
        return (Call)getCurrentCall(nativePtr);
    }

    private native Address getCurrentCallRemoteAddress(long nativePtr);
    @Override @Nullable
    synchronized public Address getCurrentCallRemoteAddress()  {
        
        return (Address)getCurrentCallRemoteAddress(nativePtr);
    }

    private native VideoDefinition getCurrentPreviewVideoDefinition(long nativePtr);
    @Override @NonNull
    synchronized public VideoDefinition getCurrentPreviewVideoDefinition()  {
        
        return (VideoDefinition)getCurrentPreviewVideoDefinition(nativePtr);
    }

    private native FriendList getDefaultFriendList(long nativePtr);
    @Override @Nullable
    synchronized public FriendList getDefaultFriendList()  {
        
        return (FriendList)getDefaultFriendList(nativePtr);
    }

    private native AudioDevice getDefaultInputAudioDevice(long nativePtr);
    @Override @NonNull
    synchronized public AudioDevice getDefaultInputAudioDevice()  {
        
        return (AudioDevice)getDefaultInputAudioDevice(nativePtr);
    }

    private native void setDefaultInputAudioDevice(long nativePtr, AudioDevice audioDevice);
    @Override
    synchronized public void setDefaultInputAudioDevice(@NonNull AudioDevice audioDevice)  {
        
        setDefaultInputAudioDevice(nativePtr, audioDevice);
    }

    private native AudioDevice getDefaultOutputAudioDevice(long nativePtr);
    @Override @NonNull
    synchronized public AudioDevice getDefaultOutputAudioDevice()  {
        
        return (AudioDevice)getDefaultOutputAudioDevice(nativePtr);
    }

    private native void setDefaultOutputAudioDevice(long nativePtr, AudioDevice audioDevice);
    @Override
    synchronized public void setDefaultOutputAudioDevice(@NonNull AudioDevice audioDevice)  {
        
        setDefaultOutputAudioDevice(nativePtr, audioDevice);
    }

    private native ProxyConfig getDefaultProxyConfig(long nativePtr);
    @Override @Nullable
    synchronized public ProxyConfig getDefaultProxyConfig()  {
        
        return (ProxyConfig)getDefaultProxyConfig(nativePtr);
    }

    private native void setDefaultProxyConfig(long nativePtr, ProxyConfig config);
    @Override
    synchronized public void setDefaultProxyConfig(@Nullable ProxyConfig config)  {
        
        setDefaultProxyConfig(nativePtr, config);
    }

    private native String getDefaultVideoDisplayFilter(long nativePtr);
    @Override @NonNull
    synchronized public String getDefaultVideoDisplayFilter()  {
        
        return getDefaultVideoDisplayFilter(nativePtr);
    }

    private native int getDelayedTimeout(long nativePtr);
    @Override
    synchronized public int getDelayedTimeout()  {
        
        return getDelayedTimeout(nativePtr);
    }

    private native void setDelayedTimeout(long nativePtr, int seconds);
    @Override
    synchronized public void setDelayedTimeout(int seconds)  {
        
        setDelayedTimeout(nativePtr, seconds);
    }

    private native int getDeviceRotation(long nativePtr);
    @Override
    synchronized public int getDeviceRotation()  {
        
        return getDeviceRotation(nativePtr);
    }

    private native void setDeviceRotation(long nativePtr, int rotation);
    @Override
    synchronized public void setDeviceRotation(int rotation)  {
        
        setDeviceRotation(nativePtr, rotation);
    }

    private native boolean getDisableRecordOnMute(long nativePtr);
    @Override
    synchronized public boolean getDisableRecordOnMute()  {
        
        return getDisableRecordOnMute(nativePtr);
    }

    private native void setDisableRecordOnMute(long nativePtr, boolean disable);
    @Override
    synchronized public void setDisableRecordOnMute(boolean disable)  {
        
        setDisableRecordOnMute(nativePtr, disable);
    }

    private native boolean dnsSearchEnabled(long nativePtr);
    @Override
    synchronized public boolean dnsSearchEnabled()  {
        
        return dnsSearchEnabled(nativePtr);
    }

    private native void enableDnsSearch(long nativePtr, boolean enable);
    @Override
    synchronized public void enableDnsSearch(boolean enable)  {
        
        enableDnsSearch(nativePtr, enable);
    }

    private native void setDnsServers(long nativePtr, String[] servers);
    @Override
    synchronized public void setDnsServers(@Nullable String[] servers)  {
        
        setDnsServers(nativePtr, servers);
    }

    private native void setDnsServersApp(long nativePtr, String[] servers);
    @Override
    synchronized public void setDnsServersApp(@Nullable String[] servers)  {
        
        setDnsServersApp(nativePtr, servers);
    }

    private native boolean getDnsSetByApp(long nativePtr);
    @Override
    synchronized public boolean getDnsSetByApp()  {
        
        return getDnsSetByApp(nativePtr);
    }

    private native boolean dnsSrvEnabled(long nativePtr);
    @Override
    synchronized public boolean dnsSrvEnabled()  {
        
        return dnsSrvEnabled(nativePtr);
    }

    private native void enableDnsSrv(long nativePtr, boolean enable);
    @Override
    synchronized public void enableDnsSrv(boolean enable)  {
        
        enableDnsSrv(nativePtr, enable);
    }

    private native int getDownloadBandwidth(long nativePtr);
    @Override
    synchronized public int getDownloadBandwidth()  {
        
        return getDownloadBandwidth(nativePtr);
    }

    private native void setDownloadBandwidth(long nativePtr, int bandwidth);
    @Override
    synchronized public void setDownloadBandwidth(int bandwidth)  {
        
        setDownloadBandwidth(nativePtr, bandwidth);
    }

    private native int getDownloadPtime(long nativePtr);
    @Override
    synchronized public int getDownloadPtime()  {
        
        return getDownloadPtime(nativePtr);
    }

    private native void setDownloadPtime(long nativePtr, int ptime);
    @Override
    synchronized public void setDownloadPtime(int ptime)  {
        
        setDownloadPtime(nativePtr, ptime);
    }

    private native boolean echoCancellationEnabled(long nativePtr);
    @Override
    synchronized public boolean echoCancellationEnabled()  {
        
        return echoCancellationEnabled(nativePtr);
    }

    private native void enableEchoCancellation(long nativePtr, boolean enable);
    @Override
    synchronized public void enableEchoCancellation(boolean enable)  {
        
        enableEchoCancellation(nativePtr, enable);
    }

    private native String getEchoCancellerFilterName(long nativePtr);
    @Override @Nullable
    synchronized public String getEchoCancellerFilterName()  {
        
        return getEchoCancellerFilterName(nativePtr);
    }

    private native void setEchoCancellerFilterName(long nativePtr, String filtername);
    @Override
    synchronized public void setEchoCancellerFilterName(@Nullable String filtername)  {
        
        setEchoCancellerFilterName(nativePtr, filtername);
    }

    private native boolean echoLimiterEnabled(long nativePtr);
    @Override
    synchronized public boolean echoLimiterEnabled()  {
        
        return echoLimiterEnabled(nativePtr);
    }

    private native void enableEchoLimiter(long nativePtr, boolean enable);
    @Override
    synchronized public void enableEchoLimiter(boolean enable)  {
        
        enableEchoLimiter(nativePtr, enable);
    }

    private native void setEnableSipUpdate(long nativePtr, int value);
    @Override
    synchronized public void setEnableSipUpdate(int value)  {
        
        setEnableSipUpdate(nativePtr, value);
    }

    private native void setExpectedBandwidth(long nativePtr, int bandwidth);
    @Override
    synchronized public void setExpectedBandwidth(int bandwidth)  {
        
        setExpectedBandwidth(nativePtr, bandwidth);
    }

    private native AudioDevice[] getExtendedAudioDevices(long nativePtr);
    @Override @NonNull
    synchronized public AudioDevice[] getExtendedAudioDevices()  {
        
        return getExtendedAudioDevices(nativePtr);
    }

    private native String getFileTransferServer(long nativePtr);
    @Override @Nullable
    synchronized public String getFileTransferServer()  {
        
        return getFileTransferServer(nativePtr);
    }

    private native void setFileTransferServer(long nativePtr, String serverUrl);
    @Override
    synchronized public void setFileTransferServer(@Nullable String serverUrl)  {
        
        setFileTransferServer(nativePtr, serverUrl);
    }

    private native boolean forcedIceRelayEnabled(long nativePtr);
    @Override
    synchronized public boolean forcedIceRelayEnabled()  {
        
        return forcedIceRelayEnabled(nativePtr);
    }

    private native void enableForcedIceRelay(long nativePtr, boolean enable);
    @Override
    synchronized public void enableForcedIceRelay(boolean enable)  {
        
        enableForcedIceRelay(nativePtr, enable);
    }

    private native void enableFriendListSubscription(long nativePtr, boolean enable);
    @Override
    synchronized public void enableFriendListSubscription(boolean enable)  {
        
        enableFriendListSubscription(nativePtr, enable);
    }

    private native String getFriendsDatabasePath(long nativePtr);
    @Override @Nullable
    synchronized public String getFriendsDatabasePath()  {
        
        return getFriendsDatabasePath(nativePtr);
    }

    private native void setFriendsDatabasePath(long nativePtr, String path);
    @Override
    synchronized public void setFriendsDatabasePath(@Nullable String path)  {
        
        setFriendsDatabasePath(nativePtr, path);
    }

    private native FriendList[] getFriendsLists(long nativePtr);
    @Override @NonNull
    synchronized public FriendList[] getFriendsLists()  {
        
        return getFriendsLists(nativePtr);
    }

    private native int getGlobalState(long nativePtr);
    @Override @NonNull
    synchronized public GlobalState getGlobalState()  {
        
        return GlobalState.fromInt(getGlobalState(nativePtr));
    }

    private native boolean getGuessHostname(long nativePtr);
    @Override
    synchronized public boolean getGuessHostname()  {
        
        return getGuessHostname(nativePtr);
    }

    private native void setGuessHostname(long nativePtr, boolean enable);
    @Override
    synchronized public void setGuessHostname(boolean enable)  {
        
        setGuessHostname(nativePtr, enable);
    }

    private native String getHttpProxyHost(long nativePtr);
    @Override @Nullable
    synchronized public String getHttpProxyHost()  {
        
        return getHttpProxyHost(nativePtr);
    }

    private native void setHttpProxyHost(long nativePtr, String host);
    @Override
    synchronized public void setHttpProxyHost(@Nullable String host)  {
        
        setHttpProxyHost(nativePtr, host);
    }

    private native int getHttpProxyPort(long nativePtr);
    @Override
    synchronized public int getHttpProxyPort()  {
        
        return getHttpProxyPort(nativePtr);
    }

    private native void setHttpProxyPort(long nativePtr, int port);
    @Override
    synchronized public void setHttpProxyPort(int port)  {
        
        setHttpProxyPort(nativePtr, port);
    }

    private native String getIdentity(long nativePtr);
    @Override @NonNull
    synchronized public String getIdentity()  {
        
        return getIdentity(nativePtr);
    }

    private native ImNotifPolicy getImNotifPolicy(long nativePtr);
    @Override @Nullable
    synchronized public ImNotifPolicy getImNotifPolicy()  {
        
        return (ImNotifPolicy)getImNotifPolicy(nativePtr);
    }

    private native int getInCallTimeout(long nativePtr);
    @Override
    synchronized public int getInCallTimeout()  {
        
        return getInCallTimeout(nativePtr);
    }

    private native void setInCallTimeout(long nativePtr, int seconds);
    @Override
    synchronized public void setInCallTimeout(int seconds)  {
        
        setInCallTimeout(nativePtr, seconds);
    }

    private native int getIncTimeout(long nativePtr);
    @Override
    synchronized public int getIncTimeout()  {
        
        return getIncTimeout(nativePtr);
    }

    private native void setIncTimeout(long nativePtr, int seconds);
    @Override
    synchronized public void setIncTimeout(int seconds)  {
        
        setIncTimeout(nativePtr, seconds);
    }

    private native AudioDevice getInputAudioDevice(long nativePtr);
    @Override @Nullable
    synchronized public AudioDevice getInputAudioDevice()  {
        
        return (AudioDevice)getInputAudioDevice(nativePtr);
    }

    private native void setInputAudioDevice(long nativePtr, AudioDevice audioDevice);
    @Override
    synchronized public void setInputAudioDevice(@Nullable AudioDevice audioDevice)  {
        
        setInputAudioDevice(nativePtr, audioDevice);
    }

    private native boolean ipv6Enabled(long nativePtr);
    @Override
    synchronized public boolean ipv6Enabled()  {
        
        return ipv6Enabled(nativePtr);
    }

    private native void enableIpv66(long nativePtr, boolean enable);
    @Override
    synchronized public void enableIpv6(boolean enable)  {
        
        enableIpv66(nativePtr, enable);
    }

    private native boolean isAutoIterateEnabled(long nativePtr);
    @Override
    synchronized public boolean isAutoIterateEnabled()  {
        
        return isAutoIterateEnabled(nativePtr);
    }

    private native boolean isEchoCancellerCalibrationRequired(long nativePtr);
    @Override
    synchronized public boolean isEchoCancellerCalibrationRequired()  {
        
        return isEchoCancellerCalibrationRequired(nativePtr);
    }

    private native boolean isFriendListSubscriptionEnabled(long nativePtr);
    @Override
    synchronized public boolean isFriendListSubscriptionEnabled()  {
        
        return isFriendListSubscriptionEnabled(nativePtr);
    }

    private native boolean isInBackground(long nativePtr);
    @Override
    synchronized public boolean isInBackground()  {
        
        return isInBackground(nativePtr);
    }

    private native boolean isInConference(long nativePtr);
    @Override
    synchronized public boolean isInConference()  {
        
        return isInConference(nativePtr);
    }

    private native boolean isIncomingInvitePending(long nativePtr);
    @Override
    synchronized public boolean isIncomingInvitePending()  {
        
        return isIncomingInvitePending(nativePtr);
    }

    private native boolean isMediaEncryptionMandatory(long nativePtr);
    @Override
    synchronized public boolean isMediaEncryptionMandatory()  {
        
        return isMediaEncryptionMandatory(nativePtr);
    }

    private native boolean isNativeRingingEnabled(long nativePtr);
    @Override
    synchronized public boolean isNativeRingingEnabled()  {
        
        return isNativeRingingEnabled(nativePtr);
    }

    private native boolean isNetworkReachable(long nativePtr);
    @Override
    synchronized public boolean isNetworkReachable()  {
        
        return isNetworkReachable(nativePtr);
    }

    private native boolean isPushNotificationAvailable(long nativePtr);
    @Override
    synchronized public boolean isPushNotificationAvailable()  {
        
        return isPushNotificationAvailable(nativePtr);
    }

    private native boolean isPushNotificationEnabled(long nativePtr);
    @Override
    synchronized public boolean isPushNotificationEnabled()  {
        
        return isPushNotificationEnabled(nativePtr);
    }

    private native boolean isSenderNameHiddenInForwardMessage(long nativePtr);
    @Override
    synchronized public boolean isSenderNameHiddenInForwardMessage()  {
        
        return isSenderNameHiddenInForwardMessage(nativePtr);
    }

    private native boolean isVibrationOnIncomingCallEnabled(long nativePtr);
    @Override
    synchronized public boolean isVibrationOnIncomingCallEnabled()  {
        
        return isVibrationOnIncomingCallEnabled(nativePtr);
    }

    private native boolean keepAliveEnabled(long nativePtr);
    @Override
    synchronized public boolean keepAliveEnabled()  {
        
        return keepAliveEnabled(nativePtr);
    }

    private native void enableKeepAlive(long nativePtr, boolean enable);
    @Override
    synchronized public void enableKeepAlive(boolean enable)  {
        
        enableKeepAlive(nativePtr, enable);
    }

    private native CallLog getLastOutgoingCallLog(long nativePtr);
    @Override @Nullable
    synchronized public CallLog getLastOutgoingCallLog()  {
        
        return (CallLog)getLastOutgoingCallLog(nativePtr);
    }

    private native boolean limeX3DhEnabled(long nativePtr);
    @Override
    synchronized public boolean limeX3DhEnabled()  {
        
        return limeX3DhEnabled(nativePtr);
    }

    private native void enableLimeX3Dh(long nativePtr, boolean enable);
    @Override
    synchronized public void enableLimeX3Dh(boolean enable)  {
        
        enableLimeX3Dh(nativePtr, enable);
    }

    private native String getLimeX3DhServerUrl(long nativePtr);
    @Override @Nullable
    synchronized public String getLimeX3DhServerUrl()  {
        
        return getLimeX3DhServerUrl(nativePtr);
    }

    private native void setLimeX3DhServerUrl(long nativePtr, String url);
    @Override
    synchronized public void setLimeX3DhServerUrl(@Nullable String url)  {
        
        setLimeX3DhServerUrl(nativePtr, url);
    }

    private native String getLinphoneSpecs(long nativePtr);
    @Override @Nullable
    synchronized public String getLinphoneSpecs()  {
        
        return getLinphoneSpecs(nativePtr);
    }

    private native void setLinphoneSpecs(long nativePtr, String specs);
    @Override
    synchronized public void setLinphoneSpecs(@Nullable String specs)  {
        
        setLinphoneSpecs(nativePtr, specs);
    }

    private native String[] getLinphoneSpecsList(long nativePtr);
    @Override @NonNull
    synchronized public String[] getLinphoneSpecsList()  {
        
        return getLinphoneSpecsList(nativePtr);
    }

    private native void setLinphoneSpecsList(long nativePtr, String[] specs);
    @Override
    synchronized public void setLinphoneSpecsList(@Nullable String[] specs)  {
        
        setLinphoneSpecsList(nativePtr, specs);
    }

    private native String getLogCollectionUploadServerUrl(long nativePtr);
    @Override @Nullable
    synchronized public String getLogCollectionUploadServerUrl()  {
        
        return getLogCollectionUploadServerUrl(nativePtr);
    }

    private native void setLogCollectionUploadServerUrl(long nativePtr, String serverUrl);
    @Override
    synchronized public void setLogCollectionUploadServerUrl(@Nullable String serverUrl)  {
        
        setLogCollectionUploadServerUrl(nativePtr, serverUrl);
    }

    private native int getMaxCalls(long nativePtr);
    @Override
    synchronized public int getMaxCalls()  {
        
        return getMaxCalls(nativePtr);
    }

    private native void setMaxCalls(long nativePtr, int max);
    @Override
    synchronized public void setMaxCalls(int max)  {
        
        setMaxCalls(nativePtr, max);
    }

    private native int getMaxSizeForAutoDownloadIncomingFiles(long nativePtr);
    @Override
    synchronized public int getMaxSizeForAutoDownloadIncomingFiles()  {
        
        return getMaxSizeForAutoDownloadIncomingFiles(nativePtr);
    }

    private native void setMaxSizeForAutoDownloadIncomingFiles(long nativePtr, int size);
    @Override
    synchronized public void setMaxSizeForAutoDownloadIncomingFiles(int size)  {
        
        setMaxSizeForAutoDownloadIncomingFiles(nativePtr, size);
    }

    private native String getMediaDevice(long nativePtr);
    @Override @Nullable
    synchronized public String getMediaDevice()  {
        
        return getMediaDevice(nativePtr);
    }

    private native int setMediaDevice(long nativePtr, String devid);
    @Override
    synchronized public int setMediaDevice(@Nullable String devid)  {
        
        return setMediaDevice(nativePtr, devid);
    }

    private native int getMediaEncryption(long nativePtr);
    @Override
    synchronized public MediaEncryption getMediaEncryption()  {
        
        return MediaEncryption.fromInt(getMediaEncryption(nativePtr));
    }

    private native int setMediaEncryption(long nativePtr, int menc);
    @Override
    synchronized public int setMediaEncryption(MediaEncryption menc)  {
        
        return setMediaEncryption(nativePtr, menc.toInt());
    }

    private native void setMediaEncryptionMandatory(long nativePtr, boolean mandatory);
    @Override
    synchronized public void setMediaEncryptionMandatory(boolean mandatory)  {
        
        setMediaEncryptionMandatory(nativePtr, mandatory);
    }

    private native void setMediaNetworkReachable(long nativePtr, boolean reachable);
    @Override
    synchronized public void setMediaNetworkReachable(boolean reachable)  {
        
        setMediaNetworkReachable(nativePtr, reachable);
    }

    private native int getMediaResourceMode(long nativePtr);
    @Override
    synchronized public MediaResourceMode getMediaResourceMode()  {
        
        return MediaResourceMode.fromInt(getMediaResourceMode(nativePtr));
    }

    private native void setMediaResourceMode(long nativePtr, int mode);
    @Override
    synchronized public void setMediaResourceMode(MediaResourceMode mode)  {
        
        setMediaResourceMode(nativePtr, mode.toInt());
    }

    private native boolean micEnabled(long nativePtr);
    @Override
    synchronized public boolean micEnabled()  {
        
        return micEnabled(nativePtr);
    }

    private native void enableMic(long nativePtr, boolean enable);
    @Override
    synchronized public void enableMic(boolean enable)  {
        
        enableMic(nativePtr, enable);
    }

    private native float getMicGainDb(long nativePtr);
    @Override
    synchronized public float getMicGainDb()  {
        
        return getMicGainDb(nativePtr);
    }

    private native void setMicGainDb(long nativePtr, float level);
    @Override
    synchronized public void setMicGainDb(float level)  {
        
        setMicGainDb(nativePtr, level);
    }

    private native int getMissedCallsCount(long nativePtr);
    @Override
    synchronized public int getMissedCallsCount()  {
        
        return getMissedCallsCount(nativePtr);
    }

    private native int getMtu(long nativePtr);
    @Override
    synchronized public int getMtu()  {
        
        return getMtu(nativePtr);
    }

    private native void setMtu(long nativePtr, int mtu);
    @Override
    synchronized public void setMtu(int mtu)  {
        
        setMtu(nativePtr, mtu);
    }

    private native String getNatAddress(long nativePtr);
    @Override @Nullable
    synchronized public String getNatAddress()  {
        
        return getNatAddress(nativePtr);
    }

    private native void setNatAddress(long nativePtr, String addr);
    @Override
    synchronized public void setNatAddress(@Nullable String addr)  {
        
        setNatAddress(nativePtr, addr);
    }

    private native NatPolicy getNatPolicy(long nativePtr);
    @Override @Nullable
    synchronized public NatPolicy getNatPolicy()  {
        
        return (NatPolicy)getNatPolicy(nativePtr);
    }

    private native void setNatPolicy(long nativePtr, NatPolicy policy);
    @Override
    synchronized public void setNatPolicy(@Nullable NatPolicy policy)  {
        
        setNatPolicy(nativePtr, policy);
    }

    private native Object getNativePreviewWindowId(long nativePtr);
    @Override @Nullable
    synchronized public Object getNativePreviewWindowId()  {
        
        return getNativePreviewWindowId(nativePtr);
    }

    private native void setNativePreviewWindowId(long nativePtr, Object windowId);
    @Override
    synchronized public void setNativePreviewWindowId(@Nullable Object windowId)  {
        
        setNativePreviewWindowId(nativePtr, windowId);
    }

    private native void setNativeRingingEnabled(long nativePtr, boolean enable);
    @Override
    synchronized public void setNativeRingingEnabled(boolean enable)  {
        
        setNativeRingingEnabled(nativePtr, enable);
    }

    private native Object getNativeVideoWindowId(long nativePtr);
    @Override @Nullable
    synchronized public Object getNativeVideoWindowId()  {
        
        return getNativeVideoWindowId(nativePtr);
    }

    private native void setNativeVideoWindowId(long nativePtr, Object windowId);
    @Override
    synchronized public void setNativeVideoWindowId(@Nullable Object windowId)  {
        
        setNativeVideoWindowId(nativePtr, windowId);
    }

    private native void setNetworkReachable(long nativePtr, boolean reachable);
    @Override
    synchronized public void setNetworkReachable(boolean reachable)  {
        
        setNetworkReachable(nativePtr, reachable);
    }

    private native int getNortpTimeout(long nativePtr);
    @Override
    synchronized public int getNortpTimeout()  {
        
        return getNortpTimeout(nativePtr);
    }

    private native void setNortpTimeout(long nativePtr, int seconds);
    @Override
    synchronized public void setNortpTimeout(int seconds)  {
        
        setNortpTimeout(nativePtr, seconds);
    }

    private native AudioDevice getOutputAudioDevice(long nativePtr);
    @Override @Nullable
    synchronized public AudioDevice getOutputAudioDevice()  {
        
        return (AudioDevice)getOutputAudioDevice(nativePtr);
    }

    private native void setOutputAudioDevice(long nativePtr, AudioDevice audioDevice);
    @Override
    synchronized public void setOutputAudioDevice(@Nullable AudioDevice audioDevice)  {
        
        setOutputAudioDevice(nativePtr, audioDevice);
    }

    private native String getPlayFile(long nativePtr);
    @Override @Nullable
    synchronized public String getPlayFile()  {
        
        return getPlayFile(nativePtr);
    }

    private native void setPlayFile(long nativePtr, String file);
    @Override
    synchronized public void setPlayFile(@Nullable String file)  {
        
        setPlayFile(nativePtr, file);
    }

    private native String getPlaybackDevice(long nativePtr);
    @Override @Nullable
    synchronized public String getPlaybackDevice()  {
        
        return getPlaybackDevice(nativePtr);
    }

    private native int setPlaybackDevice(long nativePtr, String devid);
    @Override
    synchronized public int setPlaybackDevice(@Nullable String devid)  {
        
        return setPlaybackDevice(nativePtr, devid);
    }

    private native float getPlaybackGainDb(long nativePtr);
    @Override
    synchronized public float getPlaybackGainDb()  {
        
        return getPlaybackGainDb(nativePtr);
    }

    private native void setPlaybackGainDb(long nativePtr, float level);
    @Override
    synchronized public void setPlaybackGainDb(float level)  {
        
        setPlaybackGainDb(nativePtr, level);
    }

    private native float getPreferredFramerate(long nativePtr);
    @Override
    synchronized public float getPreferredFramerate()  {
        
        return getPreferredFramerate(nativePtr);
    }

    private native void setPreferredFramerate(long nativePtr, float fps);
    @Override
    synchronized public void setPreferredFramerate(float fps)  {
        
        setPreferredFramerate(nativePtr, fps);
    }

    private native VideoDefinition getPreferredVideoDefinition(long nativePtr);
    @Override @NonNull
    synchronized public VideoDefinition getPreferredVideoDefinition()  {
        
        return (VideoDefinition)getPreferredVideoDefinition(nativePtr);
    }

    private native void setPreferredVideoDefinition(long nativePtr, VideoDefinition videoDefinition);
    @Override
    synchronized public void setPreferredVideoDefinition(@NonNull VideoDefinition videoDefinition)  {
        
        setPreferredVideoDefinition(nativePtr, videoDefinition);
    }

    private native void setPreferredVideoDefinitionByName(long nativePtr, String name);
    @Override
    synchronized public void setPreferredVideoDefinitionByName(@NonNull String name)  {
        
        setPreferredVideoDefinitionByName(nativePtr, name);
    }

    private native PresenceModel getPresenceModel(long nativePtr);
    @Override @Nullable
    synchronized public PresenceModel getPresenceModel()  {
        
        return (PresenceModel)getPresenceModel(nativePtr);
    }

    private native void setPresenceModel(long nativePtr, PresenceModel presence);
    @Override
    synchronized public void setPresenceModel(@Nullable PresenceModel presence)  {
        
        setPresenceModel(nativePtr, presence);
    }

    private native VideoDefinition getPreviewVideoDefinition(long nativePtr);
    @Override @Nullable
    synchronized public VideoDefinition getPreviewVideoDefinition()  {
        
        return (VideoDefinition)getPreviewVideoDefinition(nativePtr);
    }

    private native void setPreviewVideoDefinition(long nativePtr, VideoDefinition videoDefinition);
    @Override
    synchronized public void setPreviewVideoDefinition(@Nullable VideoDefinition videoDefinition)  {
        
        setPreviewVideoDefinition(nativePtr, videoDefinition);
    }

    private native String getPrimaryContact(long nativePtr);
    @Override @NonNull
    synchronized public String getPrimaryContact()  {
        
        return getPrimaryContact(nativePtr);
    }

    private native int setPrimaryContact(long nativePtr, String contact);
    @Override
    synchronized public int setPrimaryContact(@NonNull String contact)  {
        
        return setPrimaryContact(nativePtr, contact);
    }

    private native Address getPrimaryContactParsed(long nativePtr);
    @Override @Nullable
    synchronized public Address getPrimaryContactParsed()  {
        
        return (Address)getPrimaryContactParsed(nativePtr);
    }

    private native String getProvisioningUri(long nativePtr);
    @Override @Nullable
    synchronized public String getProvisioningUri()  {
        
        return getProvisioningUri(nativePtr);
    }

    private native int setProvisioningUri(long nativePtr, String uri);
    @Override
    synchronized public int setProvisioningUri(@Nullable String uri)  {
        
        return setProvisioningUri(nativePtr, uri);
    }

    private native ProxyConfig[] getProxyConfigList(long nativePtr);
    @Override @NonNull
    synchronized public ProxyConfig[] getProxyConfigList()  {
        
        return getProxyConfigList(nativePtr);
    }

    private native int getPushIncomingCallTimeout(long nativePtr);
    @Override
    synchronized public int getPushIncomingCallTimeout()  {
        
        return getPushIncomingCallTimeout(nativePtr);
    }

    private native void setPushIncomingCallTimeout(long nativePtr, int seconds);
    @Override
    synchronized public void setPushIncomingCallTimeout(int seconds)  {
        
        setPushIncomingCallTimeout(nativePtr, seconds);
    }

    private native void setPushNotificationEnabled(long nativePtr, boolean enable);
    @Override
    synchronized public void setPushNotificationEnabled(boolean enable)  {
        
        setPushNotificationEnabled(nativePtr, enable);
    }

    private native boolean qrcodeVideoPreviewEnabled(long nativePtr);
    @Override
    synchronized public boolean qrcodeVideoPreviewEnabled()  {
        
        return qrcodeVideoPreviewEnabled(nativePtr);
    }

    private native void enableQrcodeVideoPreview(long nativePtr, boolean enable);
    @Override
    synchronized public void enableQrcodeVideoPreview(boolean enable)  {
        
        enableQrcodeVideoPreview(nativePtr, enable);
    }

    private native boolean realtimeTextEnabled(long nativePtr);
    @Override
    synchronized public boolean realtimeTextEnabled()  {
        
        return realtimeTextEnabled(nativePtr);
    }

    private native String getRecordFile(long nativePtr);
    @Override @Nullable
    synchronized public String getRecordFile()  {
        
        return getRecordFile(nativePtr);
    }

    private native void setRecordFile(long nativePtr, String file);
    @Override
    synchronized public void setRecordFile(@Nullable String file)  {
        
        setRecordFile(nativePtr, file);
    }

    private native String getRemoteRingbackTone(long nativePtr);
    @Override @Nullable
    synchronized public String getRemoteRingbackTone()  {
        
        return getRemoteRingbackTone(nativePtr);
    }

    private native void setRemoteRingbackTone(long nativePtr, String ring);
    @Override
    synchronized public void setRemoteRingbackTone(@Nullable String ring)  {
        
        setRemoteRingbackTone(nativePtr, ring);
    }

    private native boolean retransmissionOnNackEnabled(long nativePtr);
    @Override
    synchronized public boolean retransmissionOnNackEnabled()  {
        
        return retransmissionOnNackEnabled(nativePtr);
    }

    private native void enableRetransmissionOnNack(long nativePtr, boolean enable);
    @Override
    synchronized public void enableRetransmissionOnNack(boolean enable)  {
        
        enableRetransmissionOnNack(nativePtr, enable);
    }

    private native String getRing(long nativePtr);
    @Override @Nullable
    synchronized public String getRing()  {
        
        return getRing(nativePtr);
    }

    private native void setRing(long nativePtr, String path);
    @Override
    synchronized public void setRing(@Nullable String path)  {
        
        setRing(nativePtr, path);
    }

    private native boolean getRingDuringIncomingEarlyMedia(long nativePtr);
    @Override
    synchronized public boolean getRingDuringIncomingEarlyMedia()  {
        
        return getRingDuringIncomingEarlyMedia(nativePtr);
    }

    private native void setRingDuringIncomingEarlyMedia(long nativePtr, boolean enable);
    @Override
    synchronized public void setRingDuringIncomingEarlyMedia(boolean enable)  {
        
        setRingDuringIncomingEarlyMedia(nativePtr, enable);
    }

    private native String getRingback(long nativePtr);
    @Override @Nullable
    synchronized public String getRingback()  {
        
        return getRingback(nativePtr);
    }

    private native void setRingback(long nativePtr, String path);
    @Override
    synchronized public void setRingback(@Nullable String path)  {
        
        setRingback(nativePtr, path);
    }

    private native String getRingerDevice(long nativePtr);
    @Override @Nullable
    synchronized public String getRingerDevice()  {
        
        return getRingerDevice(nativePtr);
    }

    private native int setRingerDevice(long nativePtr, String devid);
    @Override
    synchronized public int setRingerDevice(@Nullable String devid)  {
        
        return setRingerDevice(nativePtr, devid);
    }

    private native String getRootCa(long nativePtr);
    @Override @Nullable
    synchronized public String getRootCa()  {
        
        return getRootCa(nativePtr);
    }

    private native void setRootCa(long nativePtr, String path);
    @Override
    synchronized public void setRootCa(@Nullable String path)  {
        
        setRootCa(nativePtr, path);
    }

    private native void setRootCaData(long nativePtr, String data);
    @Override
    synchronized public void setRootCaData(@Nullable String data)  {
        
        setRootCaData(nativePtr, data);
    }

    private native boolean rtpBundleEnabled(long nativePtr);
    @Override
    synchronized public boolean rtpBundleEnabled()  {
        
        return rtpBundleEnabled(nativePtr);
    }

    private native void enableRtpBundle(long nativePtr, boolean value);
    @Override
    synchronized public void enableRtpBundle(boolean value)  {
        
        enableRtpBundle(nativePtr, value);
    }

    private native boolean sdp200AckEnabled(long nativePtr);
    @Override
    synchronized public boolean sdp200AckEnabled()  {
        
        return sdp200AckEnabled(nativePtr);
    }

    private native void enableSdp200Ack(long nativePtr, boolean enable);
    @Override
    synchronized public void enableSdp200Ack(boolean enable)  {
        
        enableSdp200Ack(nativePtr, enable);
    }

    private native boolean selfViewEnabled(long nativePtr);
    @Override
    synchronized public boolean selfViewEnabled()  {
        
        return selfViewEnabled(nativePtr);
    }

    private native void enableSelfView(long nativePtr, boolean enable);
    @Override
    synchronized public void enableSelfView(boolean enable)  {
        
        enableSelfView(nativePtr, enable);
    }

    private native void enableSenderNameHiddenInForwardMessage(long nativePtr, boolean enable);
    @Override
    synchronized public void enableSenderNameHiddenInForwardMessage(boolean enable)  {
        
        enableSenderNameHiddenInForwardMessage(nativePtr, enable);
    }

    private native boolean getSessionExpiresEnabled(long nativePtr);
    @Override
    synchronized public boolean getSessionExpiresEnabled()  {
        
        return getSessionExpiresEnabled(nativePtr);
    }

    private native void setSessionExpiresEnabled(long nativePtr, boolean enabled);
    @Override
    synchronized public void setSessionExpiresEnabled(boolean enabled)  {
        
        setSessionExpiresEnabled(nativePtr, enabled);
    }

    private native int getSessionExpiresMinValue(long nativePtr);
    @Override
    synchronized public int getSessionExpiresMinValue()  {
        
        return getSessionExpiresMinValue(nativePtr);
    }

    private native void setSessionExpiresMinValue(long nativePtr, int min);
    @Override
    synchronized public void setSessionExpiresMinValue(int min)  {
        
        setSessionExpiresMinValue(nativePtr, min);
    }

    private native int getSessionExpiresRefresherValue(long nativePtr);
    @Override
    synchronized public SessionExpiresRefresher getSessionExpiresRefresherValue()  {
        
        return SessionExpiresRefresher.fromInt(getSessionExpiresRefresherValue(nativePtr));
    }

    private native void setSessionExpiresRefresherValue(long nativePtr, int refresher);
    @Override
    synchronized public void setSessionExpiresRefresherValue(SessionExpiresRefresher refresher)  {
        
        setSessionExpiresRefresherValue(nativePtr, refresher.toInt());
    }

    private native int getSessionExpiresValue(long nativePtr);
    @Override
    synchronized public int getSessionExpiresValue()  {
        
        return getSessionExpiresValue(nativePtr);
    }

    private native void setSessionExpiresValue(long nativePtr, int expires);
    @Override
    synchronized public void setSessionExpiresValue(int expires)  {
        
        setSessionExpiresValue(nativePtr, expires);
    }

    private native int getSipDscp(long nativePtr);
    @Override
    synchronized public int getSipDscp()  {
        
        return getSipDscp(nativePtr);
    }

    private native void setSipDscp(long nativePtr, int dscp);
    @Override
    synchronized public void setSipDscp(int dscp)  {
        
        setSipDscp(nativePtr, dscp);
    }

    private native void setSipNetworkReachable(long nativePtr, boolean reachable);
    @Override
    synchronized public void setSipNetworkReachable(boolean reachable)  {
        
        setSipNetworkReachable(nativePtr, reachable);
    }

    private native int getSipTransportTimeout(long nativePtr);
    @Override
    synchronized public int getSipTransportTimeout()  {
        
        return getSipTransportTimeout(nativePtr);
    }

    private native void setSipTransportTimeout(long nativePtr, int timeoutMs);
    @Override
    synchronized public void setSipTransportTimeout(int timeoutMs)  {
        
        setSipTransportTimeout(nativePtr, timeoutMs);
    }

    private native String[] getSoundDevicesList(long nativePtr);
    @Override @NonNull
    synchronized public String[] getSoundDevicesList()  {
        
        return getSoundDevicesList(nativePtr);
    }

    private native String getStaticPicture(long nativePtr);
    @Override @Nullable
    synchronized public String getStaticPicture()  {
        
        return getStaticPicture(nativePtr);
    }

    private native int setStaticPicture(long nativePtr, String path);
    @Override
    synchronized public int setStaticPicture(@Nullable String path)  {
        
        return setStaticPicture(nativePtr, path);
    }

    private native float getStaticPictureFps(long nativePtr);
    @Override
    synchronized public float getStaticPictureFps()  {
        
        return getStaticPictureFps(nativePtr);
    }

    private native int setStaticPictureFps(long nativePtr, float fps);
    @Override
    synchronized public int setStaticPictureFps(float fps)  {
        
        return setStaticPictureFps(nativePtr, fps);
    }

    private native String getStunServer(long nativePtr);
    @Override @Nullable
    synchronized public String getStunServer()  {
        
        return getStunServer(nativePtr);
    }

    private native void setStunServer(long nativePtr, String server);
    @Override
    synchronized public void setStunServer(@Nullable String server)  {
        
        setStunServer(nativePtr, server);
    }

    private native String[] getSupportedFileFormatsList(long nativePtr);
    @Override @NonNull
    synchronized public String[] getSupportedFileFormatsList()  {
        
        return getSupportedFileFormatsList(nativePtr);
    }

    private native void setSupportedTag(long nativePtr, String tags);
    @Override
    synchronized public void setSupportedTag(String tags)  {
        
        setSupportedTag(nativePtr, tags);
    }

    private native PayloadType[] getTextPayloadTypes(long nativePtr);
    @Override @NonNull
    synchronized public PayloadType[] getTextPayloadTypes()  {
        
        return getTextPayloadTypes(nativePtr);
    }

    private native void setTextPayloadTypes(long nativePtr, PayloadType[] payloadTypes);
    @Override
    synchronized public void setTextPayloadTypes(@Nullable PayloadType[] payloadTypes)  {
        
        setTextPayloadTypes(nativePtr, payloadTypes);
    }

    private native int getTextPort(long nativePtr);
    @Override
    synchronized public int getTextPort()  {
        
        return getTextPort(nativePtr);
    }

    private native void setTextPort(long nativePtr, int port);
    @Override
    synchronized public void setTextPort(int port)  {
        
        setTextPort(nativePtr, port);
    }

    private native Range getTextPortsRange(long nativePtr);
    @Override @NonNull
    synchronized public Range getTextPortsRange()  {
        
        return (Range)getTextPortsRange(nativePtr);
    }

    private native String getTlsCert(long nativePtr);
    @Override @Nullable
    synchronized public String getTlsCert()  {
        
        return getTlsCert(nativePtr);
    }

    private native void setTlsCert(long nativePtr, String tlsCert);
    @Override
    synchronized public void setTlsCert(@Nullable String tlsCert)  {
        
        setTlsCert(nativePtr, tlsCert);
    }

    private native String getTlsCertPath(long nativePtr);
    @Override @Nullable
    synchronized public String getTlsCertPath()  {
        
        return getTlsCertPath(nativePtr);
    }

    private native void setTlsCertPath(long nativePtr, String tlsCertPath);
    @Override
    synchronized public void setTlsCertPath(@Nullable String tlsCertPath)  {
        
        setTlsCertPath(nativePtr, tlsCertPath);
    }

    private native String getTlsKey(long nativePtr);
    @Override @Nullable
    synchronized public String getTlsKey()  {
        
        return getTlsKey(nativePtr);
    }

    private native void setTlsKey(long nativePtr, String tlsKey);
    @Override
    synchronized public void setTlsKey(@Nullable String tlsKey)  {
        
        setTlsKey(nativePtr, tlsKey);
    }

    private native String getTlsKeyPath(long nativePtr);
    @Override @Nullable
    synchronized public String getTlsKeyPath()  {
        
        return getTlsKeyPath(nativePtr);
    }

    private native void setTlsKeyPath(long nativePtr, String tlsKeyPath);
    @Override
    synchronized public void setTlsKeyPath(@Nullable String tlsKeyPath)  {
        
        setTlsKeyPath(nativePtr, tlsKeyPath);
    }

    private native Transports getTransports(long nativePtr);
    @Override @NonNull
    synchronized public Transports getTransports()  {
        
        return (Transports)getTransports(nativePtr);
    }

    private native int setTransports(long nativePtr, Transports transports);
    @Override
    synchronized public int setTransports(@NonNull Transports transports)  {
        
        return setTransports(nativePtr, transports);
    }

    private native Transports getTransportsUsed(long nativePtr);
    @Override @NonNull
    synchronized public Transports getTransportsUsed()  {
        
        return (Transports)getTransportsUsed(nativePtr);
    }

    private native Tunnel getTunnel(long nativePtr);
    @Override @Nullable
    synchronized public Tunnel getTunnel()  {
        
        return (Tunnel)getTunnel(nativePtr);
    }

    private native int getUnreadChatMessageCount(long nativePtr);
    @Override
    synchronized public int getUnreadChatMessageCount()  {
        
        return getUnreadChatMessageCount(nativePtr);
    }

    private native int getUnreadChatMessageCountFromActiveLocals(long nativePtr);
    @Override
    synchronized public int getUnreadChatMessageCountFromActiveLocals()  {
        
        return getUnreadChatMessageCountFromActiveLocals(nativePtr);
    }

    private native int getUploadBandwidth(long nativePtr);
    @Override
    synchronized public int getUploadBandwidth()  {
        
        return getUploadBandwidth(nativePtr);
    }

    private native void setUploadBandwidth(long nativePtr, int bandwidth);
    @Override
    synchronized public void setUploadBandwidth(int bandwidth)  {
        
        setUploadBandwidth(nativePtr, bandwidth);
    }

    private native int getUploadPtime(long nativePtr);
    @Override
    synchronized public int getUploadPtime()  {
        
        return getUploadPtime(nativePtr);
    }

    private native void setUploadPtime(long nativePtr, int ptime);
    @Override
    synchronized public void setUploadPtime(int ptime)  {
        
        setUploadPtime(nativePtr, ptime);
    }

    private native String getUpnpExternalIpaddress(long nativePtr);
    @Override @Nullable
    synchronized public String getUpnpExternalIpaddress()  {
        
        return getUpnpExternalIpaddress(nativePtr);
    }

    private native int getUpnpState(long nativePtr);
    @Override
    synchronized public UpnpState getUpnpState()  {
        
        return UpnpState.fromInt(getUpnpState(nativePtr));
    }

    private native boolean getUseFiles(long nativePtr);
    @Override
    synchronized public boolean getUseFiles()  {
        
        return getUseFiles(nativePtr);
    }

    private native void setUseFiles(long nativePtr, boolean yesno);
    @Override
    synchronized public void setUseFiles(boolean yesno)  {
        
        setUseFiles(nativePtr, yesno);
    }

    private native boolean getUseInfoForDtmf(long nativePtr);
    @Override
    synchronized public boolean getUseInfoForDtmf()  {
        
        return getUseInfoForDtmf(nativePtr);
    }

    private native void setUseInfoForDtmf(long nativePtr, boolean useInfo);
    @Override
    synchronized public void setUseInfoForDtmf(boolean useInfo)  {
        
        setUseInfoForDtmf(nativePtr, useInfo);
    }

    private native boolean getUseRfc2833ForDtmf(long nativePtr);
    @Override
    synchronized public boolean getUseRfc2833ForDtmf()  {
        
        return getUseRfc2833ForDtmf(nativePtr);
    }

    private native void setUseRfc2833ForDtmf(long nativePtr, boolean useRfc2833);
    @Override
    synchronized public void setUseRfc2833ForDtmf(boolean useRfc2833)  {
        
        setUseRfc2833ForDtmf(nativePtr, useRfc2833);
    }

    private native String getUserAgent(long nativePtr);
    @Override @NonNull
    synchronized public String getUserAgent()  {
        
        return getUserAgent(nativePtr);
    }

    private native String getUserCertificatesPath(long nativePtr);
    @Override @Nullable
    synchronized public String getUserCertificatesPath()  {
        
        return getUserCertificatesPath(nativePtr);
    }

    private native void setUserCertificatesPath(long nativePtr, String path);
    @Override
    synchronized public void setUserCertificatesPath(@Nullable String path)  {
        
        setUserCertificatesPath(nativePtr, path);
    }

    private native void setVibrationOnIncomingCallEnabled(long nativePtr, boolean enable);
    @Override
    synchronized public void setVibrationOnIncomingCallEnabled(boolean enable)  {
        
        setVibrationOnIncomingCallEnabled(nativePtr, enable);
    }

    private native VideoActivationPolicy getVideoActivationPolicy(long nativePtr);
    @Override @NonNull
    synchronized public VideoActivationPolicy getVideoActivationPolicy()  {
        
        return (VideoActivationPolicy)getVideoActivationPolicy(nativePtr);
    }

    private native void setVideoActivationPolicy(long nativePtr, VideoActivationPolicy policy);
    @Override
    synchronized public void setVideoActivationPolicy(@NonNull VideoActivationPolicy policy)  {
        
        setVideoActivationPolicy(nativePtr, policy);
    }

    private native boolean videoAdaptiveJittcompEnabled(long nativePtr);
    @Override
    synchronized public boolean videoAdaptiveJittcompEnabled()  {
        
        return videoAdaptiveJittcompEnabled(nativePtr);
    }

    private native void enableVideoAdaptiveJittcomp(long nativePtr, boolean enable);
    @Override
    synchronized public void enableVideoAdaptiveJittcomp(boolean enable)  {
        
        enableVideoAdaptiveJittcomp(nativePtr, enable);
    }

    private native boolean videoCaptureEnabled(long nativePtr);
    @Override
    synchronized public boolean videoCaptureEnabled()  {
        
        return videoCaptureEnabled(nativePtr);
    }

    private native void enableVideoCapture(long nativePtr, boolean enable);
    @Override
    synchronized public void enableVideoCapture(boolean enable)  {
        
        enableVideoCapture(nativePtr, enable);
    }

    private native String getVideoDevice(long nativePtr);
    @Override @Nullable
    synchronized public String getVideoDevice()  {
        
        return getVideoDevice(nativePtr);
    }

    private native int setVideoDevice(long nativePtr, String id);
    @Override
    synchronized public int setVideoDevice(@Nullable String id)  {
        
        return setVideoDevice(nativePtr, id);
    }

    private native String[] getVideoDevicesList(long nativePtr);
    @Override @NonNull
    synchronized public String[] getVideoDevicesList()  {
        
        return getVideoDevicesList(nativePtr);
    }

    private native boolean videoDisplayEnabled(long nativePtr);
    @Override
    synchronized public boolean videoDisplayEnabled()  {
        
        return videoDisplayEnabled(nativePtr);
    }

    private native void enableVideoDisplay(long nativePtr, boolean enable);
    @Override
    synchronized public void enableVideoDisplay(boolean enable)  {
        
        enableVideoDisplay(nativePtr, enable);
    }

    private native String getVideoDisplayFilter(long nativePtr);
    @Override @Nullable
    synchronized public String getVideoDisplayFilter()  {
        
        return getVideoDisplayFilter(nativePtr);
    }

    private native void setVideoDisplayFilter(long nativePtr, String filterName);
    @Override
    synchronized public void setVideoDisplayFilter(@Nullable String filterName)  {
        
        setVideoDisplayFilter(nativePtr, filterName);
    }

    private native int getVideoDscp(long nativePtr);
    @Override
    synchronized public int getVideoDscp()  {
        
        return getVideoDscp(nativePtr);
    }

    private native void setVideoDscp(long nativePtr, int dscp);
    @Override
    synchronized public void setVideoDscp(int dscp)  {
        
        setVideoDscp(nativePtr, dscp);
    }

    private native boolean videoEnabled(long nativePtr);
    @Override
    synchronized public boolean videoEnabled()  {
        
        return videoEnabled(nativePtr);
    }

    private native int getVideoJittcomp(long nativePtr);
    @Override
    synchronized public int getVideoJittcomp()  {
        
        return getVideoJittcomp(nativePtr);
    }

    private native void setVideoJittcomp(long nativePtr, int milliseconds);
    @Override
    synchronized public void setVideoJittcomp(int milliseconds)  {
        
        setVideoJittcomp(nativePtr, milliseconds);
    }

    private native String getVideoMulticastAddr(long nativePtr);
    @Override @Nullable
    synchronized public String getVideoMulticastAddr()  {
        
        return getVideoMulticastAddr(nativePtr);
    }

    private native int setVideoMulticastAddr(long nativePtr, String ip);
    @Override
    synchronized public int setVideoMulticastAddr(@Nullable String ip)  {
        
        return setVideoMulticastAddr(nativePtr, ip);
    }

    private native boolean videoMulticastEnabled(long nativePtr);
    @Override
    synchronized public boolean videoMulticastEnabled()  {
        
        return videoMulticastEnabled(nativePtr);
    }

    private native void enableVideoMulticast(long nativePtr, boolean yesno);
    @Override
    synchronized public void enableVideoMulticast(boolean yesno)  {
        
        enableVideoMulticast(nativePtr, yesno);
    }

    private native int getVideoMulticastTtl(long nativePtr);
    @Override
    synchronized public int getVideoMulticastTtl()  {
        
        return getVideoMulticastTtl(nativePtr);
    }

    private native int setVideoMulticastTtl(long nativePtr, int ttl);
    @Override
    synchronized public int setVideoMulticastTtl(int ttl)  {
        
        return setVideoMulticastTtl(nativePtr, ttl);
    }

    private native PayloadType[] getVideoPayloadTypes(long nativePtr);
    @Override @NonNull
    synchronized public PayloadType[] getVideoPayloadTypes()  {
        
        return getVideoPayloadTypes(nativePtr);
    }

    private native void setVideoPayloadTypes(long nativePtr, PayloadType[] payloadTypes);
    @Override
    synchronized public void setVideoPayloadTypes(@Nullable PayloadType[] payloadTypes)  {
        
        setVideoPayloadTypes(nativePtr, payloadTypes);
    }

    private native int getVideoPort(long nativePtr);
    @Override
    synchronized public int getVideoPort()  {
        
        return getVideoPort(nativePtr);
    }

    private native void setVideoPort(long nativePtr, int port);
    @Override
    synchronized public void setVideoPort(int port)  {
        
        setVideoPort(nativePtr, port);
    }

    private native Range getVideoPortsRange(long nativePtr);
    @Override @NonNull
    synchronized public Range getVideoPortsRange()  {
        
        return (Range)getVideoPortsRange(nativePtr);
    }

    private native String getVideoPreset(long nativePtr);
    @Override @Nullable
    synchronized public String getVideoPreset()  {
        
        return getVideoPreset(nativePtr);
    }

    private native void setVideoPreset(long nativePtr, String preset);
    @Override
    synchronized public void setVideoPreset(@Nullable String preset)  {
        
        setVideoPreset(nativePtr, preset);
    }

    private native boolean videoPreviewEnabled(long nativePtr);
    @Override
    synchronized public boolean videoPreviewEnabled()  {
        
        return videoPreviewEnabled(nativePtr);
    }

    private native void enableVideoPreview(long nativePtr, boolean enable);
    @Override
    synchronized public void enableVideoPreview(boolean enable)  {
        
        enableVideoPreview(nativePtr, enable);
    }

    private native void enableVideoSourceReuse(long nativePtr, boolean enable);
    @Override
    synchronized public void enableVideoSourceReuse(boolean enable)  {
        
        enableVideoSourceReuse(nativePtr, enable);
    }

    private native boolean wifiOnlyEnabled(long nativePtr);
    @Override
    synchronized public boolean wifiOnlyEnabled()  {
        
        return wifiOnlyEnabled(nativePtr);
    }

    private native void enableWifiOnly(long nativePtr, boolean enable);
    @Override
    synchronized public void enableWifiOnly(boolean enable)  {
        
        enableWifiOnly(nativePtr, enable);
    }

    private native String getZrtpSecretsFile(long nativePtr);
    @Override @Nullable
    synchronized public String getZrtpSecretsFile()  {
        
        return getZrtpSecretsFile(nativePtr);
    }

    private native void setZrtpSecretsFile(long nativePtr, String file);
    @Override
    synchronized public void setZrtpSecretsFile(@Nullable String file)  {
        
        setZrtpSecretsFile(nativePtr, file);
    }

    private native void activateAudioSession(long nativePtr, boolean actived);
    @Override
    synchronized public void activateAudioSession(boolean actived)  {
        
        activateAudioSession(nativePtr, actived);
    }

    private native int addAllToConference(long nativePtr);
    @Override
    synchronized public int addAllToConference()  {
        
        return addAllToConference(nativePtr);
    }

    private native void addAuthInfo(long nativePtr, AuthInfo info);
    @Override
    synchronized public void addAuthInfo(@NonNull AuthInfo info)  {
        
        addAuthInfo(nativePtr, info);
    }

    private native void addContentTypeSupport(long nativePtr, String contentType);
    @Override
    synchronized public void addContentTypeSupport(@NonNull String contentType)  {
        
        addContentTypeSupport(nativePtr, contentType);
    }

    private native void addFriendList(long nativePtr, FriendList list);
    @Override
    synchronized public void addFriendList(@NonNull FriendList list)  {
        
        addFriendList(nativePtr, list);
    }

    private native void addLinphoneSpec(long nativePtr, String spec);
    @Override
    synchronized public void addLinphoneSpec(@NonNull String spec)  {
        
        addLinphoneSpec(nativePtr, spec);
    }

    private native int addProxyConfig(long nativePtr, ProxyConfig config);
    @Override
    synchronized public int addProxyConfig(@NonNull ProxyConfig config)  {
        
        return addProxyConfig(nativePtr, config);
    }

    private native void addSupportedTag(long nativePtr, String tag);
    @Override
    synchronized public void addSupportedTag(@NonNull String tag)  {
        
        addSupportedTag(nativePtr, tag);
    }

    private native int addToConference(long nativePtr, Call call);
    @Override
    synchronized public int addToConference(@NonNull Call call)  {
        
        return addToConference(nativePtr, call);
    }

    private native void audioRouteChanged(long nativePtr);
    @Override
    synchronized public void audioRouteChanged()  {
        
        audioRouteChanged(nativePtr);
    }

    private native void checkForUpdate(long nativePtr, String currentVersion);
    @Override
    synchronized public void checkForUpdate(@NonNull String currentVersion)  {
        
        checkForUpdate(nativePtr, currentVersion);
    }

    private native void clearAllAuthInfo(long nativePtr);
    @Override
    synchronized public void clearAllAuthInfo()  {
        
        clearAllAuthInfo(nativePtr);
    }

    private native void clearCallLogs(long nativePtr);
    @Override
    synchronized public void clearCallLogs()  {
        
        clearCallLogs(nativePtr);
    }

    private native void clearProxyConfig(long nativePtr);
    @Override
    synchronized public void clearProxyConfig()  {
        
        clearProxyConfig(nativePtr);
    }

    private native void configureAudioSession(long nativePtr);
    @Override
    synchronized public void configureAudioSession()  {
        
        configureAudioSession(nativePtr);
    }

    private native AccountCreator createAccountCreator(long nativePtr, String xmlrpcUrl);
    @Override @NonNull
    synchronized public AccountCreator createAccountCreator(@Nullable String xmlrpcUrl)  {
        
        return (AccountCreator)createAccountCreator(nativePtr, xmlrpcUrl);
    }

    private native Address createAddress(long nativePtr, String address);
    @Override @Nullable
    synchronized public Address createAddress(@Nullable String address)  {
        
        return (Address)createAddress(nativePtr, address);
    }

    private native CallLog createCallLog(long nativePtr, Address from, Address to, int dir, int duration, long startTime, long connectedTime, int status, boolean videoEnabled, float quality);
    @Override @NonNull
    synchronized public CallLog createCallLog(@NonNull Address from, @NonNull Address to, Call.Dir dir, int duration, long startTime, long connectedTime, Call.Status status, boolean videoEnabled, float quality)  {
        
        return (CallLog)createCallLog(nativePtr, from, to, dir.toInt(), duration, startTime, connectedTime, status.toInt(), videoEnabled, quality);
    }

    private native CallParams createCallParams(long nativePtr, Call call);
    @Override @Nullable
    synchronized public CallParams createCallParams(@Nullable Call call)  {
        
        return (CallParams)createCallParams(nativePtr, call);
    }

    private native ChatRoom createChatRoom(long nativePtr, ChatRoomParams params, Address localAddr, String subject, Address[] participants);
    @Override @Nullable
    synchronized public ChatRoom createChatRoom(@NonNull ChatRoomParams params, @NonNull Address localAddr, @NonNull String subject, @NonNull Address[] participants)  {
        
        return (ChatRoom)createChatRoom(nativePtr, params, localAddr, subject, participants);
    }

    private native ChatRoom createChatRoom2(long nativePtr, ChatRoomParams params, String subject, Address[] participants);
    @Override @Nullable
    synchronized public ChatRoom createChatRoom(@NonNull ChatRoomParams params, String subject, @NonNull Address[] participants)  {
        
        return (ChatRoom)createChatRoom2(nativePtr, params, subject, participants);
    }

    private native ChatRoom createChatRoom4(long nativePtr, ChatRoomParams params, Address localAddr, Address participant);
    @Override @Nullable
    synchronized public ChatRoom createChatRoom(@NonNull ChatRoomParams params, @NonNull Address localAddr, @NonNull Address participant)  {
        
        return (ChatRoom)createChatRoom4(nativePtr, params, localAddr, participant);
    }

    private native ChatRoom createChatRoom5(long nativePtr, Address participant);
    @Override @Nullable
    synchronized public ChatRoom createChatRoom(@NonNull Address participant)  {
        
        return (ChatRoom)createChatRoom5(nativePtr, participant);
    }

    private native ChatRoom createChatRoom3(long nativePtr, String subject, Address[] participants);
    @Override @Nullable
    synchronized public ChatRoom createChatRoom(@NonNull String subject, @NonNull Address[] participants)  {
        
        return (ChatRoom)createChatRoom3(nativePtr, subject, participants);
    }

    private native ChatRoom createChatRoom6(long nativePtr, ChatRoomParams params, Address localAddr, Address[] participants);
    @Override @Nullable
    synchronized public ChatRoom createChatRoom(@Nullable ChatRoomParams params, @Nullable Address localAddr, @NonNull Address[] participants)  {
        
        return (ChatRoom)createChatRoom6(nativePtr, params, localAddr, participants);
    }

    private native ChatRoom createClientGroupChatRoom(long nativePtr, String subject, boolean fallback);
    @Override @Nullable
    synchronized public ChatRoom createClientGroupChatRoom(@NonNull String subject, boolean fallback)  {
        
        return (ChatRoom)createClientGroupChatRoom(nativePtr, subject, fallback);
    }

    private native ChatRoom createClientGroupChatRoom2(long nativePtr, String subject, boolean fallback, boolean encrypted);
    @Override @Nullable
    synchronized public ChatRoom createClientGroupChatRoom(@NonNull String subject, boolean fallback, boolean encrypted)  {
        
        return (ChatRoom)createClientGroupChatRoom2(nativePtr, subject, fallback, encrypted);
    }

    private native ConferenceParams createConferenceParams(long nativePtr);
    @Override @NonNull
    synchronized public ConferenceParams createConferenceParams()  {
        
        return (ConferenceParams)createConferenceParams(nativePtr);
    }

    private native Conference createConferenceWithParams(long nativePtr, ConferenceParams params);
    @Override @Nullable
    synchronized public Conference createConferenceWithParams(@NonNull ConferenceParams params)  {
        
        return (Conference)createConferenceWithParams(nativePtr, params);
    }

    private native Config createConfig(long nativePtr, String filename);
    @Override @NonNull
    synchronized public Config createConfig(@Nullable String filename)  {
        
        return (Config)createConfig(nativePtr, filename);
    }

    private native Content createContent(long nativePtr);
    @Override @NonNull
    synchronized public Content createContent()  {
        
        return (Content)createContent(nativePtr);
    }

    private native ChatRoomParams createDefaultChatRoomParams(long nativePtr);
    @Override @NonNull
    synchronized public ChatRoomParams createDefaultChatRoomParams()  {
        
        return (ChatRoomParams)createDefaultChatRoomParams(nativePtr);
    }

    private native Friend createFriend(long nativePtr);
    @Override @NonNull
    synchronized public Friend createFriend()  {
        
        return (Friend)createFriend(nativePtr);
    }

    private native FriendList createFriendList(long nativePtr);
    @Override @NonNull
    synchronized public FriendList createFriendList()  {
        
        return (FriendList)createFriendList(nativePtr);
    }

    private native Friend createFriendWithAddress(long nativePtr, String address);
    @Override @Nullable
    synchronized public Friend createFriendWithAddress(@NonNull String address)  {
        
        return (Friend)createFriendWithAddress(nativePtr, address);
    }

    private native InfoMessage createInfoMessage(long nativePtr);
    @Override @NonNull
    synchronized public InfoMessage createInfoMessage()  {
        
        return (InfoMessage)createInfoMessage(nativePtr);
    }

    private native Player createLocalPlayer(long nativePtr, String soundCardName, String videoDisplayName, Object windowId);
    @Override @Nullable
    synchronized public Player createLocalPlayer(@Nullable String soundCardName, @Nullable String videoDisplayName, @Nullable Object windowId)  {
        
        return (Player)createLocalPlayer(nativePtr, soundCardName, videoDisplayName, windowId);
    }

    private native MagicSearch createMagicSearch(long nativePtr);
    @Override @NonNull
    synchronized public MagicSearch createMagicSearch()  {
        
        return (MagicSearch)createMagicSearch(nativePtr);
    }

    private native NatPolicy createNatPolicy(long nativePtr);
    @Override @NonNull
    synchronized public NatPolicy createNatPolicy()  {
        
        return (NatPolicy)createNatPolicy(nativePtr);
    }

    private native NatPolicy createNatPolicyFromConfig(long nativePtr, String ref);
    @Override @Nullable
    synchronized public NatPolicy createNatPolicyFromConfig(@NonNull String ref)  {
        
        return (NatPolicy)createNatPolicyFromConfig(nativePtr, ref);
    }

    private native Event createNotify(long nativePtr, Address resource, String event);
    @Override @NonNull
    synchronized public Event createNotify(@NonNull Address resource, @NonNull String event)  {
        
        return (Event)createNotify(nativePtr, resource, event);
    }

    private native Event createOneShotPublish(long nativePtr, Address resource, String event);
    @Override @NonNull
    synchronized public Event createOneShotPublish(@NonNull Address resource, @NonNull String event)  {
        
        return (Event)createOneShotPublish(nativePtr, resource, event);
    }

    private native PresenceActivity createPresenceActivity(long nativePtr, int acttype, String description);
    @Override @NonNull
    synchronized public PresenceActivity createPresenceActivity(PresenceActivity.Type acttype, @Nullable String description)  {
        
        return (PresenceActivity)createPresenceActivity(nativePtr, acttype.toInt(), description);
    }

    private native PresenceModel createPresenceModel(long nativePtr);
    @Override @NonNull
    synchronized public PresenceModel createPresenceModel()  {
        
        return (PresenceModel)createPresenceModel(nativePtr);
    }

    private native PresenceModel createPresenceModelWithActivity(long nativePtr, int acttype, String description);
    @Override @NonNull
    synchronized public PresenceModel createPresenceModelWithActivity(PresenceActivity.Type acttype, @Nullable String description)  {
        
        return (PresenceModel)createPresenceModelWithActivity(nativePtr, acttype.toInt(), description);
    }

    private native PresenceModel createPresenceModelWithActivityAndNote(long nativePtr, int acttype, String description, String note, String lang);
    @Override @NonNull
    synchronized public PresenceModel createPresenceModelWithActivityAndNote(PresenceActivity.Type acttype, @Nullable String description, @NonNull String note, @Nullable String lang)  {
        
        return (PresenceModel)createPresenceModelWithActivityAndNote(nativePtr, acttype.toInt(), description, note, lang);
    }

    private native PresenceNote createPresenceNote(long nativePtr, String content, String lang);
    @Override @NonNull
    synchronized public PresenceNote createPresenceNote(@NonNull String content, @Nullable String lang)  {
        
        return (PresenceNote)createPresenceNote(nativePtr, content, lang);
    }

    private native PresencePerson createPresencePerson(long nativePtr, String id);
    @Override @NonNull
    synchronized public PresencePerson createPresencePerson(@NonNull String id)  {
        
        return (PresencePerson)createPresencePerson(nativePtr, id);
    }

    private native PresenceService createPresenceService(long nativePtr, String id, int basicStatus, String contact);
    @Override @NonNull
    synchronized public PresenceService createPresenceService(@NonNull String id, PresenceBasicStatus basicStatus, @NonNull String contact)  {
        
        return (PresenceService)createPresenceService(nativePtr, id, basicStatus.toInt(), contact);
    }

    private native Address createPrimaryContactParsed(long nativePtr);
    @Override @Nullable
    synchronized public Address createPrimaryContactParsed()  {
        
        return (Address)createPrimaryContactParsed(nativePtr);
    }

    private native ProxyConfig createProxyConfig(long nativePtr);
    @Override @NonNull
    synchronized public ProxyConfig createProxyConfig()  {
        
        return (ProxyConfig)createProxyConfig(nativePtr);
    }

    private native Event createPublish(long nativePtr, Address resource, String event, int expires);
    @Override @NonNull
    synchronized public Event createPublish(@NonNull Address resource, @NonNull String event, int expires)  {
        
        return (Event)createPublish(nativePtr, resource, event, expires);
    }

    private native Event createSubscribe(long nativePtr, Address resource, String event, int expires);
    @Override @NonNull
    synchronized public Event createSubscribe(@NonNull Address resource, @NonNull String event, int expires)  {
        
        return (Event)createSubscribe(nativePtr, resource, event, expires);
    }

    private native Event createSubscribe2(long nativePtr, Address resource, ProxyConfig proxy, String event, int expires);
    @Override @NonNull
    synchronized public Event createSubscribe(@NonNull Address resource, @NonNull ProxyConfig proxy, @NonNull String event, int expires)  {
        
        return (Event)createSubscribe2(nativePtr, resource, proxy, event, expires);
    }

    private native XmlRpcSession createXmlRpcSession(long nativePtr, String url);
    @Override @NonNull
    synchronized public XmlRpcSession createXmlRpcSession(@NonNull String url)  {
        
        return (XmlRpcSession)createXmlRpcSession(nativePtr, url);
    }

    private native void deleteChatRoom(long nativePtr, ChatRoom chatRoom);
    @Override
    synchronized public void deleteChatRoom(@NonNull ChatRoom chatRoom)  {
        
        deleteChatRoom(nativePtr, chatRoom);
    }

    private native void disableChat(long nativePtr, int denyReason);
    @Override
    synchronized public void disableChat(Reason denyReason)  {
        
        disableChat(nativePtr, denyReason.toInt());
    }

    private native void enableChat(long nativePtr);
    @Override
    synchronized public void enableChat()  {
        
        enableChat(nativePtr);
    }

    private native void enableLime(long nativePtr, int enable);
    @Override
    synchronized public void enableLime(LimeState enable)  {
        
        enableLime(nativePtr, enable.toInt());
    }

    private native void ensureRegistered(long nativePtr);
    @Override
    synchronized public void ensureRegistered()  {
        
        ensureRegistered(nativePtr);
    }

    private native void enterBackground(long nativePtr);
    @Override
    synchronized public void enterBackground()  {
        
        enterBackground(nativePtr);
    }

    private native int enterConference(long nativePtr);
    @Override
    synchronized public int enterConference()  {
        
        return enterConference(nativePtr);
    }

    private native void enterForeground(long nativePtr);
    @Override
    synchronized public void enterForeground()  {
        
        enterForeground(nativePtr);
    }

    private native boolean fileFormatSupported(long nativePtr, String fmt);
    @Override
    synchronized public boolean fileFormatSupported(@NonNull String fmt)  {
        
        return fileFormatSupported(nativePtr, fmt);
    }

    private native AuthInfo findAuthInfo(long nativePtr, String realm, String username, String sipDomain);
    @Override @Nullable
    synchronized public AuthInfo findAuthInfo(@Nullable String realm, @NonNull String username, @Nullable String sipDomain)  {
        
        return (AuthInfo)findAuthInfo(nativePtr, realm, username, sipDomain);
    }

    private native Call findCallFromUri(long nativePtr, String uri);
    @Override @Nullable
    synchronized public Call findCallFromUri(@NonNull String uri)  {
        
        return (Call)findCallFromUri(nativePtr, uri);
    }

    private native CallLog findCallLogFromCallId(long nativePtr, String callId);
    @Override @Nullable
    synchronized public CallLog findCallLogFromCallId(@NonNull String callId)  {
        
        return (CallLog)findCallLogFromCallId(nativePtr, callId);
    }

    private native ChatRoom findChatRoom(long nativePtr, Address peerAddr, Address localAddr);
    @Override @Nullable
    synchronized public ChatRoom findChatRoom(@NonNull Address peerAddr, @NonNull Address localAddr)  {
        
        return (ChatRoom)findChatRoom(nativePtr, peerAddr, localAddr);
    }

    private native Address[] findContactsByChar(long nativePtr, String filter, boolean sipOnly);
    @Override @NonNull
    synchronized public Address[] findContactsByChar(@NonNull String filter, boolean sipOnly)  {
        
        return findContactsByChar(nativePtr, filter, sipOnly);
    }

    private native Friend findFriend(long nativePtr, Address address);
    @Override @Nullable
    synchronized public Friend findFriend(@NonNull Address address)  {
        
        return (Friend)findFriend(nativePtr, address);
    }

    private native Friend findFriendByPhoneNumber(long nativePtr, String phoneNumber);
    @Override @Nullable
    synchronized public Friend findFriendByPhoneNumber(@NonNull String phoneNumber)  {
        
        return (Friend)findFriendByPhoneNumber(nativePtr, phoneNumber);
    }

    private native Friend[] findFriends(long nativePtr, Address address);
    @Override @NonNull
    synchronized public Friend[] findFriends(@NonNull Address address)  {
        
        return findFriends(nativePtr, address);
    }

    private native ChatRoom findOneToOneChatRoom(long nativePtr, Address localAddr, Address participantAddr);
    @Override @Nullable
    synchronized public ChatRoom findOneToOneChatRoom(@NonNull Address localAddr, @NonNull Address participantAddr)  {
        
        return (ChatRoom)findOneToOneChatRoom(nativePtr, localAddr, participantAddr);
    }

    private native ChatRoom findOneToOneChatRoom2(long nativePtr, Address localAddr, Address participantAddr, boolean encrypted);
    @Override @Nullable
    synchronized public ChatRoom findOneToOneChatRoom(@NonNull Address localAddr, @NonNull Address participantAddr, boolean encrypted)  {
        
        return (ChatRoom)findOneToOneChatRoom2(nativePtr, localAddr, participantAddr, encrypted);
    }

    private native Call getCallByCallid(long nativePtr, String callId);
    @Override @Nullable
    synchronized public Call getCallByCallid(String callId)  {
        
        return (Call)getCallByCallid(nativePtr, callId);
    }

    private native Call getCallByRemoteAddress(long nativePtr, String remoteAddress);
    @Override @Nullable
    synchronized public Call getCallByRemoteAddress(@NonNull String remoteAddress)  {
        
        return (Call)getCallByRemoteAddress(nativePtr, remoteAddress);
    }

    private native Call getCallByRemoteAddress22(long nativePtr, Address remoteAddress);
    @Override @Nullable
    synchronized public Call getCallByRemoteAddress2(@NonNull Address remoteAddress)  {
        
        return (Call)getCallByRemoteAddress22(nativePtr, remoteAddress);
    }

    private native CallLog[] getCallHistory2(long nativePtr, Address peerAddress, Address localAddress);
    @Override @NonNull
    synchronized public CallLog[] getCallHistory(@NonNull Address peerAddress, @NonNull Address localAddress)  {
        
        return getCallHistory2(nativePtr, peerAddress, localAddress);
    }

    private native CallLog[] getCallHistoryForAddress(long nativePtr, Address address);
    @Override @NonNull
    synchronized public CallLog[] getCallHistoryForAddress(@NonNull Address address)  {
        
        return getCallHistoryForAddress(nativePtr, address);
    }

    private native ChatRoom getChatRoom2(long nativePtr, Address peerAddr, Address localAddr);
    @Override @Nullable
    synchronized public ChatRoom getChatRoom(@NonNull Address peerAddr, @NonNull Address localAddr)  {
        
        return (ChatRoom)getChatRoom2(nativePtr, peerAddr, localAddr);
    }

    private native ChatRoom getChatRoom(long nativePtr, Address addr);
    @Override @Nullable
    synchronized public ChatRoom getChatRoom(@NonNull Address addr)  {
        
        return (ChatRoom)getChatRoom(nativePtr, addr);
    }

    private native ChatRoom getChatRoomFromUri(long nativePtr, String to);
    @Override @Nullable
    synchronized public ChatRoom getChatRoomFromUri(@NonNull String to)  {
        
        return (ChatRoom)getChatRoomFromUri(nativePtr, to);
    }

    private native Friend getFriendByRefKey(long nativePtr, String key);
    @Override @Nullable
    synchronized public Friend getFriendByRefKey(@NonNull String key)  {
        
        return (Friend)getFriendByRefKey(nativePtr, key);
    }

    private native FriendList getFriendListByName(long nativePtr, String name);
    @Override @Nullable
    synchronized public FriendList getFriendListByName(@NonNull String name)  {
        
        return (FriendList)getFriendListByName(nativePtr, name);
    }

    private native PayloadType getPayloadType(long nativePtr, String type, int rate, int channels);
    @Override @Nullable
    synchronized public PayloadType getPayloadType(@NonNull String type, int rate, int channels)  {
        
        return (PayloadType)getPayloadType(nativePtr, type, rate, channels);
    }

    private native ProxyConfig getProxyConfigByIdkey(long nativePtr, String idkey);
    @Override @Nullable
    synchronized public ProxyConfig getProxyConfigByIdkey(String idkey)  {
        
        return (ProxyConfig)getProxyConfigByIdkey(nativePtr, idkey);
    }

    private native int getUnreadChatMessageCountFromLocal(long nativePtr, Address address);
    @Override
    synchronized public int getUnreadChatMessageCountFromLocal(@NonNull Address address)  {
        
        return getUnreadChatMessageCountFromLocal(nativePtr, address);
    }

    private native int getZrtpStatus(long nativePtr, String addr);
    @Override
    synchronized public ZrtpPeerStatus getZrtpStatus(@NonNull String addr)  {
        
        return ZrtpPeerStatus.fromInt(getZrtpStatus(nativePtr, addr));
    }

    private native boolean hasBuiltinEchoCanceller(long nativePtr);
    @Override
    synchronized public boolean hasBuiltinEchoCanceller()  {
        
        return hasBuiltinEchoCanceller(nativePtr);
    }

    private native boolean hasCrappyOpengl(long nativePtr);
    @Override
    synchronized public boolean hasCrappyOpengl()  {
        
        return hasCrappyOpengl(nativePtr);
    }

    private native boolean inCall(long nativePtr);
    @Override
    synchronized public boolean inCall()  {
        
        return inCall(nativePtr);
    }

    private native Address interpretUrl(long nativePtr, String url);
    @Override @Nullable
    synchronized public Address interpretUrl(@NonNull String url)  {
        
        return (Address)interpretUrl(nativePtr, url);
    }

    private native Call invite(long nativePtr, String url);
    @Override @Nullable
    synchronized public Call invite(@NonNull String url)  {
        
        return (Call)invite(nativePtr, url);
    }

    private native Call inviteAddress(long nativePtr, Address addr);
    @Override @Nullable
    synchronized public Call inviteAddress(@NonNull Address addr)  {
        
        return (Call)inviteAddress(nativePtr, addr);
    }

    private native Call inviteAddressWithParams(long nativePtr, Address addr, CallParams params);
    @Override @Nullable
    synchronized public Call inviteAddressWithParams(@NonNull Address addr, @NonNull CallParams params)  {
        
        return (Call)inviteAddressWithParams(nativePtr, addr, params);
    }

    private native Call inviteWithParams(long nativePtr, String url, CallParams params);
    @Override @Nullable
    synchronized public Call inviteWithParams(@NonNull String url, @NonNull CallParams params)  {
        
        return (Call)inviteWithParams(nativePtr, url, params);
    }

    private native boolean isContentTypeSupported(long nativePtr, String contentType);
    @Override
    synchronized public boolean isContentTypeSupported(@NonNull String contentType)  {
        
        return isContentTypeSupported(nativePtr, contentType);
    }

    private native boolean isMediaFilterSupported(long nativePtr, String filtername);
    @Override
    synchronized public boolean isMediaFilterSupported(@NonNull String filtername)  {
        
        return isMediaFilterSupported(nativePtr, filtername);
    }

    private native void iterate(long nativePtr);
    @Override
    synchronized public void iterate()  {
        
        iterate(nativePtr);
    }

    private native int leaveConference(long nativePtr);
    @Override
    synchronized public int leaveConference()  {
        
        return leaveConference(nativePtr);
    }

    private native boolean limeAvailable(long nativePtr);
    @Override
    synchronized public boolean limeAvailable()  {
        
        return limeAvailable(nativePtr);
    }

    private native int limeEnabled(long nativePtr);
    @Override
    synchronized public LimeState limeEnabled()  {
        
        return LimeState.fromInt(limeEnabled(nativePtr));
    }

    private native boolean limeX3DhAvailable(long nativePtr);
    @Override
    synchronized public boolean limeX3DhAvailable()  {
        
        return limeX3DhAvailable(nativePtr);
    }

    private native void loadConfigFromXml(long nativePtr, String xmlUri);
    @Override
    synchronized public void loadConfigFromXml(@NonNull String xmlUri)  {
        
        loadConfigFromXml(nativePtr, xmlUri);
    }

    private native boolean mediaEncryptionSupported(long nativePtr, int menc);
    @Override
    synchronized public boolean mediaEncryptionSupported(MediaEncryption menc)  {
        
        return mediaEncryptionSupported(nativePtr, menc.toInt());
    }

    private native void migrateLogsFromRcToDb(long nativePtr);
    @Override
    synchronized public void migrateLogsFromRcToDb()  {
        
        migrateLogsFromRcToDb(nativePtr);
    }

    private native int migrateToMultiTransport(long nativePtr);
    @Override
    synchronized public int migrateToMultiTransport()  {
        
        return migrateToMultiTransport(nativePtr);
    }

    private native void notifyAllFriends(long nativePtr, PresenceModel presence);
    @Override
    synchronized public void notifyAllFriends(@NonNull PresenceModel presence)  {
        
        notifyAllFriends(nativePtr, presence);
    }

    private native void notifyNotifyPresenceReceived(long nativePtr, Friend linphoneFriend);
    @Override
    synchronized public void notifyNotifyPresenceReceived(@NonNull Friend linphoneFriend)  {
        
        notifyNotifyPresenceReceived(nativePtr, linphoneFriend);
    }

    private native void notifyNotifyPresenceReceivedForUriOrTel(long nativePtr, Friend linphoneFriend, String uriOrTel, PresenceModel presenceModel);
    @Override
    synchronized public void notifyNotifyPresenceReceivedForUriOrTel(@NonNull Friend linphoneFriend, @NonNull String uriOrTel, @NonNull PresenceModel presenceModel)  {
        
        notifyNotifyPresenceReceivedForUriOrTel(nativePtr, linphoneFriend, uriOrTel, presenceModel);
    }

    private native int pauseAllCalls(long nativePtr);
    @Override
    synchronized public int pauseAllCalls()  {
        
        return pauseAllCalls(nativePtr);
    }

    private native void playDtmf(long nativePtr, char dtmf, int durationMs);
    @Override
    synchronized public void playDtmf(char dtmf, int durationMs)  {
        
        playDtmf(nativePtr, dtmf, durationMs);
    }

    private native int playLocal(long nativePtr, String audiofile);
    @Override
    synchronized public int playLocal(@NonNull String audiofile)  {
        
        return playLocal(nativePtr, audiofile);
    }

    private native int preemptSoundResources(long nativePtr);
    @Override
    synchronized public int preemptSoundResources()  {
        
        return preemptSoundResources(nativePtr);
    }

    private native void previewOglRender(long nativePtr);
    @Override
    synchronized public void previewOglRender()  {
        
        previewOglRender(nativePtr);
    }

    private native Event publish(long nativePtr, Address resource, String event, int expires, Content body);
    @Override @Nullable
    synchronized public Event publish(@NonNull Address resource, @NonNull String event, int expires, @NonNull Content body)  {
        
        return (Event)publish(nativePtr, resource, event, expires, body);
    }

    private native int realtimeTextGetKeepaliveInterval(long nativePtr);
    @Override
    synchronized public int realtimeTextGetKeepaliveInterval()  {
        
        return realtimeTextGetKeepaliveInterval(nativePtr);
    }

    private native void realtimeTextSetKeepaliveInterval(long nativePtr, int interval);
    @Override
    synchronized public void realtimeTextSetKeepaliveInterval(int interval)  {
        
        realtimeTextSetKeepaliveInterval(nativePtr, interval);
    }

    private native void refreshRegisters(long nativePtr);
    @Override
    synchronized public void refreshRegisters()  {
        
        refreshRegisters(nativePtr);
    }

    private native void rejectSubscriber(long nativePtr, Friend linphoneFriend);
    @Override
    synchronized public void rejectSubscriber(@NonNull Friend linphoneFriend)  {
        
        rejectSubscriber(nativePtr, linphoneFriend);
    }

    private native void reloadMsPlugins(long nativePtr, String path);
    @Override
    synchronized public void reloadMsPlugins(@Nullable String path)  {
        
        reloadMsPlugins(nativePtr, path);
    }

    private native void reloadSoundDevices(long nativePtr);
    @Override
    synchronized public void reloadSoundDevices()  {
        
        reloadSoundDevices(nativePtr);
    }

    private native void reloadVideoDevices(long nativePtr);
    @Override
    synchronized public void reloadVideoDevices()  {
        
        reloadVideoDevices(nativePtr);
    }

    private native void removeAuthInfo(long nativePtr, AuthInfo info);
    @Override
    synchronized public void removeAuthInfo(@NonNull AuthInfo info)  {
        
        removeAuthInfo(nativePtr, info);
    }

    private native void removeCallLog(long nativePtr, CallLog callLog);
    @Override
    synchronized public void removeCallLog(@NonNull CallLog callLog)  {
        
        removeCallLog(nativePtr, callLog);
    }

    private native void removeContentTypeSupport(long nativePtr, String contentType);
    @Override
    synchronized public void removeContentTypeSupport(@NonNull String contentType)  {
        
        removeContentTypeSupport(nativePtr, contentType);
    }

    private native void removeFriendList(long nativePtr, FriendList list);
    @Override
    synchronized public void removeFriendList(@NonNull FriendList list)  {
        
        removeFriendList(nativePtr, list);
    }

    private native int removeFromConference(long nativePtr, Call call);
    @Override
    synchronized public int removeFromConference(@NonNull Call call)  {
        
        return removeFromConference(nativePtr, call);
    }

    private native void removeLinphoneSpec(long nativePtr, String spec);
    @Override
    synchronized public void removeLinphoneSpec(@NonNull String spec)  {
        
        removeLinphoneSpec(nativePtr, spec);
    }

    private native void removeProxyConfig(long nativePtr, ProxyConfig config);
    @Override
    synchronized public void removeProxyConfig(@NonNull ProxyConfig config)  {
        
        removeProxyConfig(nativePtr, config);
    }

    private native void removeSupportedTag(long nativePtr, String tag);
    @Override
    synchronized public void removeSupportedTag(@NonNull String tag)  {
        
        removeSupportedTag(nativePtr, tag);
    }

    private native void resetMissedCallsCount(long nativePtr);
    @Override
    synchronized public void resetMissedCallsCount()  {
        
        resetMissedCallsCount(nativePtr);
    }

    private native ChatRoom searchChatRoom(long nativePtr, ChatRoomParams params, Address localAddr, Address remoteAddr, Address[] participants);
    @Override @Nullable
    synchronized public ChatRoom searchChatRoom(@Nullable ChatRoomParams params, @Nullable Address localAddr, @Nullable Address remoteAddr, @Nullable Address[] participants)  {
        
        return (ChatRoom)searchChatRoom(nativePtr, params, localAddr, remoteAddr, participants);
    }

    private native void setAudioPortRange(long nativePtr, int minPort, int maxPort);
    @Override
    synchronized public void setAudioPortRange(int minPort, int maxPort)  {
        
        setAudioPortRange(nativePtr, minPort, maxPort);
    }

    private native void setCallErrorTone(long nativePtr, int reason, String audiofile);
    @Override
    synchronized public void setCallErrorTone(Reason reason, @Nullable String audiofile)  {
        
        setCallErrorTone(nativePtr, reason.toInt(), audiofile);
    }

    private native void setQrcodeDecodeRect(long nativePtr, int x, int y, int w, int h);
    @Override
    synchronized public void setQrcodeDecodeRect(int x, int y, int w, int h)  {
        
        setQrcodeDecodeRect(nativePtr, x, y, w, h);
    }

    private native void setTextPortRange(long nativePtr, int minPort, int maxPort);
    @Override
    synchronized public void setTextPortRange(int minPort, int maxPort)  {
        
        setTextPortRange(nativePtr, minPort, maxPort);
    }

    private native void setTone(long nativePtr, int toneId, String audiofile);
    @Override
    synchronized public void setTone(ToneID toneId, @NonNull String audiofile)  {
        
        setTone(nativePtr, toneId.toInt(), audiofile);
    }

    private native void setUserAgent(long nativePtr, String name, String version);
    @Override
    synchronized public void setUserAgent(@Nullable String name, @Nullable String version)  {
        
        setUserAgent(nativePtr, name, version);
    }

    private native void setVideoPortRange(long nativePtr, int minPort, int maxPort);
    @Override
    synchronized public void setVideoPortRange(int minPort, int maxPort)  {
        
        setVideoPortRange(nativePtr, minPort, maxPort);
    }

    private native boolean soundDeviceCanCapture(long nativePtr, String device);
    @Override
    synchronized public boolean soundDeviceCanCapture(@NonNull String device)  {
        
        return soundDeviceCanCapture(nativePtr, device);
    }

    private native boolean soundDeviceCanPlayback(long nativePtr, String device);
    @Override
    synchronized public boolean soundDeviceCanPlayback(@NonNull String device)  {
        
        return soundDeviceCanPlayback(nativePtr, device);
    }

    private native boolean soundResourcesLocked(long nativePtr);
    @Override
    synchronized public boolean soundResourcesLocked()  {
        
        return soundResourcesLocked(nativePtr);
    }

    private native int start(long nativePtr);
    @Override
    synchronized public int start()  {
        
        return start(nativePtr);
    }

    private native int startConferenceRecording(long nativePtr, String path);
    @Override
    synchronized public int startConferenceRecording(@NonNull String path)  {
        
        return startConferenceRecording(nativePtr, path);
    }

    private native int startEchoCancellerCalibration(long nativePtr);
    @Override
    synchronized public int startEchoCancellerCalibration()  {
        
        return startEchoCancellerCalibration(nativePtr);
    }

    private native int startEchoTester(long nativePtr, int rate);
    @Override
    synchronized public int startEchoTester(int rate)  {
        
        return startEchoTester(nativePtr, rate);
    }

    private native void stop(long nativePtr);
    @Override
    synchronized public void stop()  {
        
        stop(nativePtr);
    }

    private native void stopAsync(long nativePtr);
    @Override
    synchronized public void stopAsync()  {
        
        stopAsync(nativePtr);
    }

    private native int stopConferenceRecording(long nativePtr);
    @Override
    synchronized public int stopConferenceRecording()  {
        
        return stopConferenceRecording(nativePtr);
    }

    private native void stopDtmf(long nativePtr);
    @Override
    synchronized public void stopDtmf()  {
        
        stopDtmf(nativePtr);
    }

    private native int stopEchoTester(long nativePtr);
    @Override
    synchronized public int stopEchoTester()  {
        
        return stopEchoTester(nativePtr);
    }

    private native void stopRinging(long nativePtr);
    @Override
    synchronized public void stopRinging()  {
        
        stopRinging(nativePtr);
    }

    private native Event subscribe(long nativePtr, Address resource, String event, int expires, Content body);
    @Override @NonNull
    synchronized public Event subscribe(@NonNull Address resource, @NonNull String event, int expires, @Nullable Content body)  {
        
        return (Event)subscribe(nativePtr, resource, event, expires, body);
    }

    private native int takePreviewSnapshot(long nativePtr, String file);
    @Override
    synchronized public int takePreviewSnapshot(@NonNull String file)  {
        
        return takePreviewSnapshot(nativePtr, file);
    }

    private native int terminateAllCalls(long nativePtr);
    @Override
    synchronized public int terminateAllCalls()  {
        
        return terminateAllCalls(nativePtr);
    }

    private native int terminateConference(long nativePtr);
    @Override
    synchronized public int terminateConference()  {
        
        return terminateConference(nativePtr);
    }

    private native void uploadLogCollection(long nativePtr);
    @Override
    synchronized public void uploadLogCollection()  {
        
        uploadLogCollection(nativePtr);
    }

    private native void usePreviewWindow(long nativePtr, boolean yesno);
    @Override
    synchronized public void usePreviewWindow(boolean yesno)  {
        
        usePreviewWindow(nativePtr, yesno);
    }

    private native void verifyServerCertificates(long nativePtr, boolean yesno);
    @Override
    synchronized public void verifyServerCertificates(boolean yesno)  {
        
        verifyServerCertificates(nativePtr, yesno);
    }

    private native void verifyServerCn(long nativePtr, boolean yesno);
    @Override
    synchronized public void verifyServerCn(boolean yesno)  {
        
        verifyServerCn(nativePtr, yesno);
    }

    private native boolean videoSupported(long nativePtr);
    @Override
    synchronized public boolean videoSupported()  {
        
        return videoSupported(nativePtr);
    }

    private native String compressLogCollection(long nativePtr);
    @Override @NonNull
    synchronized public String compressLogCollection()  {
        
        return compressLogCollection(nativePtr);
    }

    private native void enableLogCollection(long nativePtr, int state);
    @Override
    synchronized public void enableLogCollection(LogCollectionState state)  {
        
        enableLogCollection(nativePtr, state.toInt());
    }

    private native int getLogCollectionMaxFileSize(long nativePtr);
    @Override @NonNull
    synchronized public int getLogCollectionMaxFileSize()  {
        
        return getLogCollectionMaxFileSize(nativePtr);
    }

    private native String getLogCollectionPath(long nativePtr);
    @Override @NonNull
    synchronized public String getLogCollectionPath()  {
        
        return getLogCollectionPath(nativePtr);
    }

    private native String getLogCollectionPrefix(long nativePtr);
    @Override @NonNull
    synchronized public String getLogCollectionPrefix()  {
        
        return getLogCollectionPrefix(nativePtr);
    }

    private native String getVersion(long nativePtr);
    @Override @NonNull
    synchronized public String getVersion()  {
        
        return getVersion(nativePtr);
    }

    private native int logCollectionEnabled(long nativePtr);
    @Override
    synchronized public LogCollectionState logCollectionEnabled()  {
        
        return LogCollectionState.fromInt(logCollectionEnabled(nativePtr));
    }

    private native void resetLogCollection(long nativePtr);
    @Override
    synchronized public void resetLogCollection()  {
        
        resetLogCollection(nativePtr);
    }

    private native void serializeLogs(long nativePtr);
    @Override
    synchronized public void serializeLogs()  {
        
        serializeLogs(nativePtr);
    }

    private native void setLogCollectionMaxFileSize(long nativePtr, int size);
    @Override
    synchronized public void setLogCollectionMaxFileSize(int size)  {
        
        setLogCollectionMaxFileSize(nativePtr, size);
    }

    private native void setLogCollectionPath(long nativePtr, String path);
    @Override
    synchronized public void setLogCollectionPath(@NonNull String path)  {
        
        setLogCollectionPath(nativePtr, path);
    }

    private native void setLogCollectionPrefix(long nativePtr, String prefix);
    @Override
    synchronized public void setLogCollectionPrefix(@NonNull String prefix)  {
        
        setLogCollectionPrefix(nativePtr, prefix);
    }

    private native boolean tunnelAvailable(long nativePtr);
    @Override
    synchronized public boolean tunnelAvailable()  {
        
        return tunnelAvailable(nativePtr);
    }

    private native boolean upnpAvailable(long nativePtr);
    @Override
    synchronized public boolean upnpAvailable()  {
        
        return upnpAvailable(nativePtr);
    }

    private native boolean vcardSupported(long nativePtr);
    @Override
    synchronized public boolean vcardSupported()  {
        
        return vcardSupported(nativePtr);
    }

    private native void addListener(long nativePtr, CoreListener listener);
    @Override
    synchronized public void addListener(CoreListener listener)  {
        
        addListener(nativePtr, listener);
    }

    private native void removeListener(long nativePtr, CoreListener listener);
    @Override
    synchronized public void removeListener(CoreListener listener)  {
        
        removeListener(nativePtr, listener);
    }

    private native org.linphone.mediastream.Factory getMediastreamerFactory(long nativePtr);
    public org.linphone.mediastream.Factory getMediastreamerFactory() {
        return getMediastreamerFactory(nativePtr);
    }

    private native boolean unref(long ptr);
    protected void finalize() throws Throwable {
        /* Considering the following scenario:
        User scrolls fast in a chat message list that contains a lot of images, 
        so the VM will garbage collect often due to the images taking much space.
        The view itself only has a reference on the EventLogs, not the ChatMessages directly,
        so each call of onBindView will call eventLog.getChatMessage() and the reference will only be temporary.
        It has been observed a race condition can occurs in which the JNI level can obtain a LocalRef from a WeakGlobalRef
        of an object that has been scheduled for destruction and for which the finalize() method will be called shortly after.
        The result is a Java object that has been finalized can have it's methods be called after the finalize() has terminated.*/
        
        /*The workaround we made here is to keep the nativePtr value even after the unref() only if the object is still reffed by the SDK,
        so the few methods calls following the finalize() will still work because we know the native pointer still exists.*/

        /*This solution will work as long as the objects still have a ref at C level, which is the case with ChatMessages because they are hold by the Events.
        Should we have this problem again in a situation where the C no longer holds a reference (resulting in the C object being  destroyed by finalize()), 
        another solution is possible (but more complex to implement).
        This solution would consist in implementing at C level, a queue of belle_sip_object_t that are loosing their last ref. 
        Instead of being deleted immediately by belle_sip_object_unref() calling belle_sip_object_delete(), 
        unref() would place the object onto this "C finalizer queue", the delete() call would then be performed later, 
        by a hook added into the main android looper.
        This solution should work because the actual destruction will be performed from the looper context, 
        so in a place where the local java object causing our troubles no longer exists for sure.
        This solution has however the inconvenience that it can work within an application that uses a single thread to call liblinphone API.*/
        
		if (nativePtr != 0) {
			boolean destroyed = unref(nativePtr);
            if (destroyed) {
			    nativePtr = 0;
            }
		}
		super.finalize();
	}

    @Override
    public void setUserData(Object data) {
        userData = data;
    }

    @Override
    public Object getUserData() {
        return userData;
    }

    @Override
    public String toString() {
        return "Java object [" + super.toString() + "], native pointer [" + String.format("0x%08x", nativePtr) + "]";
    }
}
